@@ -15,7 +15,7 @@
 */
 
 
-/* $Id: dpmain.c,v 1.8 2002/06/21 22:30:09 ghostgum Exp $ */
+/* $Id: dpmain.c 5251 2004-08-19 21:52:20Z ghostgum $ */
 /* Ghostscript DLL loader for OS/2 */
 /* For WINDOWCOMPAT (console mode) application */
 
@@ -40,7 +40,7 @@
 #include <sys/select.h>
 #include "gscdefs.h"
 #define GS_REVISION gs_revision
-#include "errors.h"
+#include "ierrors.h"
 #include "iapi.h"
 #include "gdevdsp.h"
 
@@ -985,14 +985,14 @@ display_callback display = {
 int
 main(int argc, char *argv[])
 {
-    int code;
+    int code, code1;
     int exit_code;
     int exit_status;
     int nargc;
     char **nargv;
     char dformat[64];
     ULONG version[3];
-    gs_main_instance *instance;
+    void *instance;
 
     if (DosQuerySysInfo(QSV_VERSION_MAJOR, QSV_VERSION_REVISION, 
 	    &version, sizeof(version)))
@@ -1051,7 +1051,9 @@ main(int argc, char *argv[])
 	code = gsdll.init_with_args(instance, nargc, nargv);
 	if (code == 0) 
 	    code = gsdll.run_string(instance, start_string, 0, &exit_code);
-	gsdll.exit(instance);
+	code1 = gsdll.exit(instance);
+	if (code == 0 || (code == e_Quit && code1 != 0))
+	    code = code1;
 
 	gsdll.delete_instance(instance);
     }
