@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-// $Id: dwinst.cpp,v 1.5 2002/03/12 20:55:22 ghostgum Exp $
+// $Id: dwinst.cpp 5510 2004-11-18 06:48:41Z ghostgum $
 
 #define STRICT
 #include <windows.h>
@@ -244,6 +244,16 @@ BOOL CInstall::InstallFiles(BOOL bNoCopy, BOOL *pbQuit)
 }
 
 
+void CInstall::AppendFileNew(const char *filename)
+{
+    FILE *f;
+    /* mark backup file for uninstall */
+    if ((f = fopen(m_szFileNew, "a")) != (FILE *)NULL) {
+	fputs(filename, f);
+	fputs("\n", f);
+	fclose(f);
+    }
+}
 
 // recursive mkdir
 // requires a full path to be specified, so ignores root \ 
