@@ -14,13 +14,14 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: dwtrace.h,v 1.4 2002/06/16 04:47:10 lpd Exp $ */
+/* $Id: dwtrace.h 5335 2004-09-15 19:41:01Z ray $ */
 /* The interface of Graphical trace server for Windows */
 
 #ifndef dwtrace_INCLUDED
 #  define dwtrace_INCLUDED
 
-struct vd_trace_interface_s visual_tracer;
+extern struct vd_trace_interface_s visual_tracer;
 void visual_tracer_init(void);
+void visual_tracer_close(void);
 
 #endif /* dwtrace_INCLUDED */
