@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevdbit.c,v 1.8 2002/09/21 00:26:04 dan Exp $ */
+/*$Id: gdevdbit.c 5218 2004-08-05 17:02:36Z stefan $ */
 /* Default device bitmap copying implementation */
 #include "gx.h"
 #include "gpcheck.h"
@@ -69,7 +69,7 @@ gx_default_copy_mono(gx_device * dev, const byte * data,
 	invert = true;
 	color = zero;
     }
-    color_set_pure(&devc, color);
+    set_nonclient_dev_color(&devc, color);
     return gx_dc_default_fill_masked
 	(&devc, data, dx, raster, id, x, y, w, h, dev, rop3_T, invert);
 }
@@ -107,6 +107,14 @@ gx_default_copy_color(gx_device * dev, const byte * data,
 	    if (depth >= 8) {
 		color = *ptr++;
 		switch (depth) {
+		    case 64:
+			color = (color << 8) + *ptr++;
+		    case 56:
+			color = (color << 8) + *ptr++;
+		    case 48:
+			color = (color << 8) + *ptr++;
+		    case 40:
+			color = (color << 8) + *ptr++;
 		    case 32:
 			color = (color << 8) + *ptr++;
 		    case 24:
@@ -409,7 +417,7 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
      (*proc_color)(dev, row, srcx, raster, id, tx, ty, tw, th) :\
      (*proc_mono)(dev, row, srcx, raster, id, tx, ty, tw, th, color0, color1));\
   if (code < 0) return_error(code);\
-  return_if_interrupt()
+  return_if_interrupt(dev->memory)
 #ifdef DEBUG
 #define copy_tile(srcx, tx, ty, tw, th, tid)\
   if_debug6('t', "   copy id=%lu sx=%d => x=%d y=%d w=%d h=%d\n",\
