@@ -14,10 +14,12 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevddrw.c,v 1.4 2002/02/21 22:24:51 giles Exp $ */
+/* $Id: gdevddrw.c 5567 2004-12-08 21:35:13Z stefan $ */
 /* Default polygon and image drawing device procedures */
+#include <assert.h>
 #include "math_.h"
 #include "memory_.h"
+#include "stdint_.h"
 #include "gx.h"
 #include "gpcheck.h"
 #include "gserrors.h"
@@ -28,6 +30,14 @@
 #include "gxdevice.h"
 #include "gxiparam.h"
 #include "gxistate.h"
+#include "gdevddrw.h"
+#include "vdtrace.h"
+/*
+#include "gxdtfill.h" - Do not remove this comment.
+                        "gxdtfill.h" is included below.
+*/
+
+#define VD_RECT_COLOR RGB(0, 0, 255)
 
 #define SWAP(a, b, t)\
   (t = a, a = b, b = t)
@@ -60,10 +70,21 @@ typedef struct trap_line_s {
     int di;
     fixed df;
 	/*
-	 * The current position within the scan line is x + xf/h + 1.
+	 * The intersection of the line with a scan line is x + xf/h + 1.
 	 * (The 1 refers to the least significant bit of x, not fixed_1;
 	 * similarly, the quotient refers to the l.s.b. of x.)
 	 * We know -h <= xf < 0.
+	 *
+	 * This rational value preciselly represents the mathematical line
+	 * (with no machine arithmetic error).
+	 *
+	 * Note that the fractional part is negative to simplify
+	 * some conditions in the Bresenham algorithm.
+	 * Due to that some expressions are inobvious.
+	 * We believe that it's a kind of archaic
+	 * for the modern hyperthreading architecture,
+	 * we still keep it because the code passed a huge testing
+	 * on various platforms.
 	 */
     fixed x, xf;
 	/*
@@ -74,6 +95,19 @@ typedef struct trap_line_s {
     fixed ldi, ldf;
 } trap_line;
 
+
+/*
+ * The linear color trapezoid fill algorithm uses trap_color structures to keep track of
+ * the color change during the Bresenham loop.
+ */
+typedef struct trap_gradient_s {
+	frac31 *c; /* integer part of the color in frac32 units. */
+	int32_t *f; /* the fraction part numerator */
+	int32_t *num; /* the gradient numerator */
+	int32_t den; /* color gradient denominator */
+} trap_gradient;
+
+
 /*
  * Compute the di and df members of a trap_line structure.  The x extent
  * (edge.end.x - edge.start.x) is a parameter; the y extent (h member)
@@ -131,177 +165,421 @@ compute_ldx(trap_line *tl, fixed ys)
     }
 }
 
+private inline void
+init_gradient(trap_gradient *g, const gs_fill_attributes *fa,
+		const gs_linear_color_edge *e, const gs_linear_color_edge *e1,
+		const trap_line *l, fixed ybot, int num_components)
+{
+    int i;
+    int64_t c;
+    int32_t d;
+
+    if (e->c1 == NULL || e->c0 == NULL)
+	g->den = 0; /* A wedge - the color is axial along another edge. */
+    else {
+	bool ends_from_fa = (e1->c1 == NULL || e1->c0 == NULL);
+
+	if (ends_from_fa)
+	    g->den = fa->yend - fa->ystart;
+	else {
+	    g->den = e->end.y - e->start.y;
+	    assert(g->den == l->h);
+	}
+	for (i = 0; i < num_components; i++) {
+	    g->num[i] = e->c1[i] - e->c0[i];
+	    c = (int64_t)g->num[i] * (uint32_t)(ybot - 
+		    (ends_from_fa ? fa->ystart : e->start.y));
+	    d = (int32_t)(c / g->den);
+	    g->c[i] = e->c0[i] + d;
+	    c -= (int64_t)d * g->den;
+	    if (c < 0) {
+		g->c[i]--;
+		c += g->den;
+	    }
+	    g->f[i] = (int32_t)c;
+	}
+    }
+}
+
+private inline void
+step_gradient(trap_gradient *g, int num_components)
+{
+    int i;
+
+    if (g->den == 0)
+	return;
+    for (i = 0; i < num_components; i++) {
+	int64_t fc = g->f[i] + (int64_t)g->num[i] * fixed_1;
+	int32_t fc32;
+
+	g->c[i] += (int32_t)(fc / g->den);
+	fc32 = (int32_t)(fc -  fc / g->den * g->den);
+	if (fc32 < 0) {
+	    fc32 += g->den;
+	    g->c[i]--;
+	}
+	g->f[i] = fc32;
+    }
+}
+
+private inline bool
+check_gradient_overflow(const gs_linear_color_edge *le, const gs_linear_color_edge *re,
+		int num_components)
+{
+    if (le->c1 == NULL || re->c1 == NULL) {
+	/* A wedge doesn't use a gradient by X. */
+	return false;
+    } else {
+	/* Check whether set_x_gradient, fill_linear_color_scanline can overflow. 
+
+	   dev_proc(dev, fill_linear_color_scanline) can perform its computation in 32-bit fractions,
+	   so we assume it never overflows. Devices which implement it with no this
+	   assumption must implement the check in gx_default_fill_linear_color_trapezoid,
+	   gx_default_fill_linear_color_triangle with a function other than this one.
+
+	   Since set_x_gradient perform computations in int64_t, which provides 63 bits
+	   while multiplying a 32-bits color value to a coordinate,
+	   we must restrict the X span with 63 - 32 = 31 bits.
+	 */
+	int32_t xl = min(le->start.x, le->end.x);
+	int32_t xr = min(re->start.x, re->end.x);
+	/* The pixel span boundaries : */
+	return arith_rshift_1(xr) - arith_rshift_1(xl) >= 0x3FFFFFFE;
+    }
+}
+
+
+private inline int
+set_x_gradient_nowedge(trap_gradient *xg, const trap_gradient *lg, const trap_gradient *rg, 
+	     const trap_line *l, const trap_line *r, int il, int ir, int num_components)
+{
+    /* Ignoring the ending coordinats fractions, 
+       so the gridient is slightly shifted to the left (in <1 'fixed' unit). */
+    int32_t xl = l->x - (l->xf == -l->h ? 1 : 0) - fixed_half; /* Revert the GX_FILL_TRAPEZOID shift. */
+    int32_t xr = r->x - (r->xf == -r->h ? 1 : 0) - fixed_half; /* Revert the GX_FILL_TRAPEZOID shift. */
+    /* The pixel span boundaries : */
+    int32_t x0 = int2fixed(il) + fixed_half; /* Shift to the pixel center. */
+    int32_t x1 = int2fixed(ir) + fixed_half; /* Shift to the pixel center. */
+    int i;
+
+#   ifdef DEBUG
+	if (arith_rshift_1(xr) - arith_rshift_1(xl) >= 0x3FFFFFFE) /* Can overflow ? */
+	    return_error(gs_error_unregistered); /* Must not happen. */
+#   endif
+    xg->den = fixed2int(x1 - x0);
+    for (i = 0; i < num_components; i++) {
+	/* Ignoring the ending colors fractions, 
+	   so the color gets a slightly smaller value
+	   (in <1 'frac31' unit), but it's not important due to 
+	   the further conversion to [0, 1 << cinfo->comp_bits[j]], 
+	   which drops the fraction anyway. */
+	int32_t cl = lg->c[i];
+	int32_t cr = rg->c[i];
+	int32_t c0 = (int32_t)(cl + ((int64_t)cr - cl) * (x0 - xl) / (xr - xl));
+	int32_t c1 = (int32_t)(cl + ((int64_t)cr - cl) * (x1 - xl) / (xr - xl));
+
+	xg->c[i] = c0;
+	xg->f[i] = 0; /* Insufficient bits to compute it better. 
+	                 The color so the color gets a slightly smaller value
+			 (in <1 'frac31' unit), but it's not important due to 
+			 the further conversion to [0, 1 << cinfo->comp_bits[j]], 
+			 which drops the fraction anyway. 
+			 So setting 0 appears pretty good and fast. */
+	xg->num[i] = c1 - c0;
+    }
+    return 0;
+}
+
+private inline int
+set_x_gradient(trap_gradient *xg, const trap_gradient *lg, const trap_gradient *rg, 
+	     const trap_line *l, const trap_line *r, int il, int ir, int num_components)
+{
+    if (lg->den == 0 || rg->den == 0) {
+	/* A wedge doesn't use a gradient by X. */
+	int i;
+
+	xg->den = 1;
+	for (i = 0; i < num_components; i++) {
+	    xg->c[i] = (lg->den == 0 ? rg->c[i] : lg->c[i]);
+	    xg->f[i] = 0; /* Compatible to set_x_gradient_nowedge. */
+	    xg->num[i] = 0;
+	}
+	return 0;
+    } else
+	return set_x_gradient_nowedge(xg, lg, rg, l, r, il, ir, num_components);
+}
+
 /*
- * Fill a trapezoid.  left.start => left.end and right.start => right.end
- * define the sides; ybot and ytop define the top and bottom.  Requires:
- *      {left,right}->start.y <= ybot <= ytop <= {left,right}->end.y.
- * Lines where left.x >= right.x will not be drawn.  Thanks to Paul Haeberli
- * for an early floating point version of this algorithm.
+ * Fill a trapezoid.
+ * Since we need several statically defined variants of this algorithm,
+ * we stored it in gxdtfill.h and include it configuring with
+ * macros defined here.
  */
+#define LINEAR_COLOR 0 /* Common for shading variants. */
+#define EDGE_TYPE gs_fixed_edge  /* Common for non-shading variants. */
+#define FILL_ATTRS gs_logical_operation_t  /* Common for non-shading variants. */
+
+#define GX_FILL_TRAPEZOID private int gx_fill_trapezoid_as_fd
+#define CONTIGUOUS_FILL 0
+#define SWAP_AXES 1
+#define FILL_DIRECT 1
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#define GX_FILL_TRAPEZOID private int gx_fill_trapezoid_as_nd
+#define CONTIGUOUS_FILL 0
+#define SWAP_AXES 1
+#define FILL_DIRECT 0
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#define GX_FILL_TRAPEZOID private int gx_fill_trapezoid_ns_fd
+#define CONTIGUOUS_FILL 0
+#define SWAP_AXES 0
+#define FILL_DIRECT 1
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#define GX_FILL_TRAPEZOID private int gx_fill_trapezoid_ns_nd
+#define CONTIGUOUS_FILL 0
+#define SWAP_AXES 0
+#define FILL_DIRECT 0
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#define GX_FILL_TRAPEZOID int gx_fill_trapezoid_cf_fd
+#define CONTIGUOUS_FILL 1
+#define SWAP_AXES 0
+#define FILL_DIRECT 1
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#define GX_FILL_TRAPEZOID int gx_fill_trapezoid_cf_nd
+#define CONTIGUOUS_FILL 1
+#define SWAP_AXES 0
+#define FILL_DIRECT 0
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#undef EDGE_TYPE
+#undef LINEAR_COLOR
+#undef FILL_ATTRS
+
+#define LINEAR_COLOR 1 /* Common for shading variants. */
+#define EDGE_TYPE gs_linear_color_edge /* Common for shading variants. */
+#define FILL_ATTRS const gs_fill_attributes *  /* Common for non-shading variants. */
+
+#define GX_FILL_TRAPEZOID private int gx_fill_trapezoid_ns_lc
+#define CONTIGUOUS_FILL 0
+#define SWAP_AXES 0
+#define FILL_DIRECT 1
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#define GX_FILL_TRAPEZOID private int gx_fill_trapezoid_as_lc
+#define CONTIGUOUS_FILL 0
+#define SWAP_AXES 1
+#define FILL_DIRECT 1
+#include "gxdtfill.h"
+#undef GX_FILL_TRAPEZOID
+#undef CONTIGUOUS_FILL 
+#undef SWAP_AXES
+#undef FILL_DIRECT
+
+#undef EDGE_TYPE
+#undef LINEAR_COLOR
+#undef FILL_ATTRS
+
+
 int
 gx_default_fill_trapezoid(gx_device * dev, const gs_fixed_edge * left,
     const gs_fixed_edge * right, fixed ybot, fixed ytop, bool swap_axes,
     const gx_device_color * pdevc, gs_logical_operation_t lop)
 {
-    const fixed ymin = fixed_pixround(ybot) + fixed_half;
-    const fixed ymax = fixed_pixround(ytop);
+    bool fill_direct = color_writes_pure(pdevc, lop);
 
-    if (ymin >= ymax)
-	return 0;		/* no scan lines to sample */
-    {
-	int iy = fixed2int_var(ymin);
-	const int iy1 = fixed2int_var(ymax);
-	trap_line l, r;
-	int rxl, rxr, ry;
-	const fixed
-	    x0l = left->start.x, x1l = left->end.x, x0r = right->start.x,
-	    x1r = right->end.x, dxl = x1l - x0l, dxr = x1r - x0r;
-	const fixed	/* partial pixel offset to first line to sample */
-	    ysl = ymin - left->start.y, ysr = ymin - right->start.y;
-	fixed fxl;
-	bool fill_direct = color_writes_pure(pdevc, lop);
-	int max_rect_height = 1;  /* max height to do fill as rectangle */
-	int code;
-
-	if_debug2('z', "[z]y=[%d,%d]\n", iy, iy1);
-
-	l.h = left->end.y - left->start.y;
-	r.h = right->end.y - right->start.y;
-	l.x = x0l + (fixed_half - fixed_epsilon);
-	r.x = x0r + (fixed_half - fixed_epsilon);
-	ry = iy;
-
-/*
- * Free variables of FILL_TRAP_RECT:
- *	swap_axes, pdevc, dev, lop
- * Free variables of FILL_TRAP_RECT_DIRECT:
- *	swap_axes, fill_rect, dev, cindex
- */
-#define FILL_TRAP_RECT(x,y,w,h)\
-  (swap_axes ? gx_fill_rectangle_device_rop(y, x, h, w, pdevc, dev, lop) :\
-   gx_fill_rectangle_device_rop(x, y, w, h, pdevc, dev, lop))
-#define FILL_TRAP_RECT_DIRECT(x,y,w,h)\
-  (swap_axes ? (*fill_rect)(dev, y, x, h, w, cindex) :\
-   (*fill_rect)(dev, x, y, w, h, cindex))
-
-	/* Compute the dx/dy ratios. */
-
-	/*
-	 * Compute the x offsets at the first scan line to sample.  We need
-	 * to be careful in computing ys# * dx#f {/,%} h# because the
-	 * multiplication may overflow.  We know that all the quantities
-	 * involved are non-negative, and that ys# is usually less than 1 (as
-	 * a fixed, of course); this gives us a cheap conservative check for
-	 * overflow in the multiplication.
-	 */
-#define YMULT_QUO(ys, tl)\
-  (ys < fixed_1 && tl.df < YMULT_LIMIT ? ys * tl.df / tl.h :\
-   fixed_mult_quo(ys, tl.df, tl.h))
-
-	/*
-	 * It's worth checking for dxl == dxr, since this is the case
-	 * for parallelograms (including stroked lines).
-	 * Also check for left or right vertical edges.
-	 */
-	if (fixed_floor(l.x) == fixed_pixround(x1l)) {
-	    /* Left edge is vertical, we don't need to increment. */
-	    l.di = 0, l.df = 0;
-	    fxl = 0;
-	} else {
-	    compute_dx(&l, dxl, ysl);
-	    fxl = YMULT_QUO(ysl, l);
-	    l.x += fxl;
-	}
-	if (fixed_floor(r.x) == fixed_pixround(x1r)) {
-	    /* Right edge is vertical.  If both are vertical, */
-	    /* we have a rectangle. */
-	    if (l.di == 0 && l.df == 0)
-		max_rect_height = max_int;
-	    else
-		r.di = 0, r.df = 0;
-	}
-	/*
-	 * The test for fxl != 0 is required because the right edge might
-	 * cross some pixel centers even if the left edge doesn't.
-	 */
-	else if (dxr == dxl && fxl != 0) {
-	    if (l.di == 0)
-		r.di = 0, r.df = l.df;
-	    else		/* too hard to do adjustments right */
-		compute_dx(&r, dxr, ysr);
-	    if (ysr == ysl && r.h == l.h)
-		r.x += fxl;
-	    else
-		r.x += YMULT_QUO(ysr, r);
-	} else {
-	    compute_dx(&r, dxr, ysr);
-	    r.x += YMULT_QUO(ysr, r);
-	}
-	rxl = fixed2int_var(l.x);
-	rxr = fixed2int_var(r.x);
-
-	/*
-	 * Take a shortcut if we're only sampling a single scan line,
-	 * or if we have a rectangle.
-	 */
-	if (iy1 - iy <= max_rect_height) {
-	    iy = iy1 - 1;
-	    if_debug2('z', "[z]rectangle, x=[%d,%d]\n", rxl, rxr);
-	} else {
-	    /* Compute one line's worth of dx/dy. */
-	    compute_ldx(&l, ysl);
-	    if (dxr == dxl && ysr == ysl && r.h == l.h)
-		r.ldi = l.ldi, r.ldf = l.ldf, r.xf = l.xf;
-	    else
-		compute_ldx(&r, ysr);
-	}
-
-#define STEP_LINE(ix, tl)\
-  tl.x += tl.ldi;\
-  if ( (tl.xf += tl.ldf) >= 0 ) tl.xf -= tl.h, tl.x++;\
-  ix = fixed2int_var(tl.x)
-
-	if (fill_direct) {
-	    gx_color_index cindex = pdevc->colors.pure;
-	    dev_proc_fill_rectangle((*fill_rect)) =
-		dev_proc(dev, fill_rectangle);
-
-	    while (++iy != iy1) {
-		int ixl, ixr;
-
-		STEP_LINE(ixl, l);
-		STEP_LINE(ixr, r);
-		if (ixl != rxl || ixr != rxr) {
-		    code = FILL_TRAP_RECT_DIRECT(rxl, ry, rxr - rxl, iy - ry);
-		    if (code < 0)
-			goto xit;
-		    rxl = ixl, rxr = ixr, ry = iy;
-		}
-	    }
-	    code = FILL_TRAP_RECT_DIRECT(rxl, ry, rxr - rxl, iy - ry);
-	} else {
-	    while (++iy != iy1) {
-		int ixl, ixr;
-
-		STEP_LINE(ixl, l);
-		STEP_LINE(ixr, r);
-		if (ixl != rxl || ixr != rxr) {
-		    code = FILL_TRAP_RECT(rxl, ry, rxr - rxl, iy - ry);
-		    if (code < 0)
-			goto xit;
-		    rxl = ixl, rxr = ixr, ry = iy;
-		}
-	    }
-	    code = FILL_TRAP_RECT(rxl, ry, rxr - rxl, iy - ry);
-	}
-#undef STEP_LINE
-xit:	if (code < 0 && fill_direct)
-	    return_error(code);
-	return_if_interrupt();
-	return code;
+    if (swap_axes) {
+	if (fill_direct)	
+	    return gx_fill_trapezoid_as_fd(dev, left, right, ybot, ytop, 0, pdevc, lop);
+	else
+	    return gx_fill_trapezoid_as_nd(dev, left, right, ybot, ytop, 0, pdevc, lop);
+    } else {
+	if (fill_direct)	
+	    return gx_fill_trapezoid_ns_fd(dev, left, right, ybot, ytop, 0, pdevc, lop);
+	else
+	    return gx_fill_trapezoid_ns_nd(dev, left, right, ybot, ytop, 0, pdevc, lop);
     }
 }
 
+private inline void
+middle_frac31_color(frac31 *c, const frac31 *c0, const frac31 *c2, int num_components)
+{
+    /* Assuming non-negative values. */
+    int i;
+
+    for (i = 0; i < num_components; i++)
+	c[i] = (int32_t)(((uint32_t)c0[i] + (uint32_t)c2[i]) >> 1);
+}
+
+private inline int
+fill_linear_color_trapezoid_nocheck(gx_device *dev, const gs_fill_attributes *fa,
+	const gs_linear_color_edge *le, const gs_linear_color_edge *re)
+{
+    fixed y02 = max(le->start.y, re->start.y), ymin = max(y02, fa->clip->p.y);
+    fixed y13 = min(le->end.y, re->end.y), ymax = min(y13, fa->clip->q.y);
+    int code;
+
+    code = (fa->swap_axes ? gx_fill_trapezoid_as_lc : gx_fill_trapezoid_ns_lc)(dev, 
+	    le, re, ymin, ymax, 0, NULL, fa);
+    if (code < 0)
+	return code;
+    return !code;
+}
+
+
+/*  Fill a trapezoid with a linear color.
+    [p0 : p1] - left edge, from bottom to top.
+    [p2 : p3] - right edge, from bottom to top.
+    The filled area is within Y-spans of both edges.
+
+    This implemetation actually handles a bilinear color,
+    in which the generatrix keeps a parallelizm to the X axis.
+    In general a bilinear function doesn't keep the generatrix parallelizm, 
+    so the caller must decompose/approximate such functions.
+
+    Return values : 
+    1 - success;
+    0 - Too big. The area isn't filled. The client must decompose the area.
+    <0 - error.
+ */
+int 
+gx_default_fill_linear_color_trapezoid(gx_device *dev, const gs_fill_attributes *fa,
+	const gs_fixed_point *p0, const gs_fixed_point *p1,
+	const gs_fixed_point *p2, const gs_fixed_point *p3,
+	const frac31 *c0, const frac31 *c1,
+	const frac31 *c2, const frac31 *c3)
+{
+    gs_linear_color_edge le, re;
+    int num_components = dev->color_info.num_components;
+
+    le.start = *p0;
+    le.end = *p1;
+    le.c0 = c0;
+    le.c1 = c1;
+    le.clip_x = fa->clip->p.x;
+    re.start = *p2;
+    re.end = *p3;
+    re.c0 = c2;
+    re.c1 = c3;
+    re.clip_x = fa->clip->q.x;
+    if (check_gradient_overflow(&le, &re, num_components))
+        return 0;
+    return fill_linear_color_trapezoid_nocheck(dev, fa, &le, &re);
+}
+
+private inline int 
+fill_linear_color_triangle(gx_device *dev, const gs_fill_attributes *fa,
+	const gs_fixed_point *p0, const gs_fixed_point *p1,
+	const gs_fixed_point *p2,
+	const frac31 *c0, const frac31 *c1, const frac31 *c2)
+{   /* p0 must be the lowest vertex. */
+    int code;
+    gs_linear_color_edge e0, e1, e2;
+    int num_components = dev->color_info.num_components;
+
+    if (p0->y == p1->y)
+	return gx_default_fill_linear_color_trapezoid(dev, fa, p0, p2, p1, p2, c0, c2, c1, c2);
+    if (p1->y == p2->y)
+	return gx_default_fill_linear_color_trapezoid(dev, fa, p0, p2, p0, p1, c0, c2, c0, c1);
+    e0.start = *p0;
+    e0.end = *p2;
+    e0.c0 = c0;
+    e0.c1 = c2;
+    e0.clip_x = fa->clip->p.x;
+    e1.start = *p0;
+    e1.end = *p1;
+    e1.c0 = c0;
+    e1.c1 = c1;
+    e1.clip_x = fa->clip->q.x;
+    if (p0->y < p1->y && p1->y < p2->y) {
+	e2.start = *p1;
+	e2.end = *p2;
+	e2.c0 = c1;
+	e2.c1 = c2;
+	e2.clip_x = fa->clip->q.x;
+	if (check_gradient_overflow(&e0, &e1, num_components))
+	    return 0;
+	if (check_gradient_overflow(&e0, &e2, num_components))
+	    return 0;
+	code = fill_linear_color_trapezoid_nocheck(dev, fa, &e0, &e1);
+	if (code <= 0) /* Sic! */
+	    return code;
+	return fill_linear_color_trapezoid_nocheck(dev, fa, &e0, &e2);
+    } else { /* p0->y < p2->y && p2->y < p1->y */
+	e2.start = *p2;
+	e2.end = *p1;
+	e2.c0 = c2;
+	e2.c1 = c1;
+	e2.clip_x = fa->clip->q.x;
+	if (check_gradient_overflow(&e0, &e1, num_components))
+	    return 0;
+	if (check_gradient_overflow(&e2, &e1, num_components))
+	    return 0;
+	code = fill_linear_color_trapezoid_nocheck(dev, fa, &e0, &e1);
+	if (code <= 0) /* Sic! */
+	    return code;
+	return fill_linear_color_trapezoid_nocheck(dev, fa, &e2, &e1);
+    }
+}
+
+/*  Fill a triangle with a linear color. */
+int 
+gx_default_fill_linear_color_triangle(gx_device *dev, const gs_fill_attributes *fa,
+	const gs_fixed_point *p0, const gs_fixed_point *p1,
+	const gs_fixed_point *p2,
+	const frac31 *c0, const frac31 *c1, const frac31 *c2)
+{
+    fixed dx1 = p1->x - p0->x, dy1 = p1->y - p0->y;
+    fixed dx2 = p2->x - p0->x, dy2 = p2->y - p0->y;
+    
+    if ((int64_t)dx1 * dy2 < (int64_t)dx2 * dy1) {
+	const gs_fixed_point *p = p1;
+	const frac31 *c = c1;
+	
+	p1 = p2; 
+	p2 = p;
+	c1 = c2;
+	c2 = c;
+    }
+    if (p0->y <= p1->y && p0->y <= p2->y)
+	return fill_linear_color_triangle(dev, fa, p0, p1, p2, c0, c1, c2);
+    if (p1->y <= p0->y && p1->y <= p2->y)
+	return fill_linear_color_triangle(dev, fa, p1, p2, p0, c1, c2, c0);
+    else
+	return fill_linear_color_triangle(dev, fa, p2, p0, p1, c2, c0, c1);
+}
+
 /* Fill a parallelogram whose points are p, p+a, p+b, and p+a+b. */
 /* We should swap axes to get best accuracy, but we don't. */
 /* We must be very careful to follow the center-of-pixel rule in all cases. */
@@ -488,7 +766,7 @@ gx_default_draw_thin_line(gx_device * dev,
     int itox = fixed2int_var(fx1);
     int itoy = fixed2int_var(fy1);
 
-    return_if_interrupt();
+    return_if_interrupt(dev->memory);
     if (itoy == iy) {		/* horizontal line */
 	return (ix <= itox ?
 		gx_fill_rectangle_device_rop(ix, iy, itox - ix + 1, 1,
