@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 2001-2004, Ghostgum Software Pty Ltd.  All rights reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
@@ -14,7 +14,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
  */
 
-/* $Id: gdevdsp2.h,v 1.6 2002/02/21 22:24:51 giles Exp $ */
+/* $Id: gdevdsp2.h 5154 2004-07-07 09:33:19Z ghostgum $ */
 /* gdevdsp2.c */
 
 #ifndef gdevdsp2_INCLUDED
@@ -28,7 +28,10 @@ typedef struct gx_device_display_s gx_device_display;
 	void *pHandle;\
 	int nFormat;\
 	void *pBitmap;\
-	unsigned long ulBitmapSize
+	unsigned long ulBitmapSize;\
+	int HWResolution_set;\
+        gs_devn_params devn_params;\
+        equivalent_cmyk_color_params equiv_cmyk_colors
 
 /* The device descriptor */
 struct gx_device_display_s {
