@@ -1,4 +1,23 @@
-/* $Id: gdevmacpictop.h,v 1.5 2002/02/21 22:24:51 giles Exp $ */
+/* Copyright (C) 1997-2002 artofcode LLC.  All rights reserved.                     
+
+  This software is provided AS-IS with no warranty, either express or
+  implied.
+
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
+
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+*/
+
+/* $Id: gdevmacpictop.h 3479 2003-01-06 23:37:58Z giles $ */
+
+/* Helpers for working with Classic MacOS Quickdraw pictures */
+/* (obsoleted by the DISPLAY device) */
 
 #ifndef gdevmacpictop_INCLUDED
 #  define gdevmacpictop_INCLUDED
@@ -666,7 +685,4 @@
 			PICT_RGBBkCol(ptr, rgb[0], rgb[1], rgb[2]);										\
 		}
 
-
-
-
 #endif /* gdevmacpictop_INCLUDED */
