@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevmrun.c,v 1.4 2002/02/21 22:24:51 giles Exp $ */
+/* $Id: gdevmrun.c 4163 2003-08-21 14:55:14Z igor $ */
 /* Run-length encoded memory device */
 #include "memory_.h"
 #include "gx.h"
@@ -619,37 +619,3 @@ run_fill_rectangle(gx_device *dev, int x, int y, int w, int h,
     return 0;
 }
 
-/* Debugging code */
-
-#ifdef DEBUG
-
-void
-debug_print_run(const run *data, run_index index, const char *prefix)
-{
-    const run *pr = data + index;
-
-    dlprintf6("%s%5d: length = %3d, value = 0x%lx, prev = %5u, next = %5u\n",
-	      prefix, index, pr->length, (ulong)pr->value, pr->prev, pr->next);
-}
-
-void
-debug_print_run_line(const run_line *line, const char *prefix)
-{
-    const run *data = CONST_RL_DATA(line);
-
-    dlprintf5("%sruns at 0x%lx: zero = 0x%lx, free = %u, xcur = %u,\n",
-	      prefix, (ulong)data, (ulong)line->zero, line->free, line->xcur);
-    dlprintf3("%s  rpcur = {ptr = 0x%lx, index = %u}\n",
-	      prefix, (ulong)line->rpcur.ptr, line->rpcur.index);
-    {
-	const_run_ptr rpc;
-
-	RP_TO_START(rpc, data);
-	while (!RP_AT_END(rpc)) {
-	    debug_print_run(data, rpc.index, prefix);
-	    RP_TO_NEXT(rpc, data, rpc);
-	}
-    }
-}
-
-#endif /* DEBUG */
