@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdfb.c,v 1.16 2002/06/16 05:48:55 lpd Exp $ */
+/* $Id: gdevpdfb.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Low-level bitmap image handling for PDF-writing driver */
 #include "string_.h"
 #include "gx.h"
@@ -23,6 +23,7 @@
 #include "gdevpdfg.h"
 #include "gdevpdfo.h"		/* for data stream */
 #include "gxcspace.h"
+#include "gxhldevc.h"
 
 /* We need this color space type for constructing temporary color spaces. */
 extern const gs_color_space_type gs_color_space_type_Indexed;
@@ -69,7 +70,7 @@ pdf_copy_mask_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     } else {
 	row_base = base;
 	row_step = raster;
-	in_line = nbytes <= MAX_INLINE_IMAGE_BYTES;
+	in_line = nbytes < pdev->MaxInlineImageSize;
 	pdf_put_image_matrix(pdev, &pim->ImageMatrix, 1.0);
 	/*
 	 * Check whether we've already made an XObject resource for this
@@ -125,7 +126,9 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	code = pdf_open_page(pdev, PDF_IN_STREAM);
 	if (code < 0)
 	    return code;
-	pdf_put_clip_path(pdev, pcpath);
+	code = pdf_put_clip_path(pdev, pcpath);
+	if (code < 0)
+	    return code;
     }
     /* We have 3 cases: mask, inverse mask, and solid. */
     if (zero == gx_no_color_index) {
@@ -133,8 +136,6 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	    return 0;
 	/* If a mask has an id, assume it's a character. */
 	if (id != gx_no_bitmap_id && sourcex == 0) {
-	    pdf_set_pure_color(pdev, one, &pdev->fill_color,
-			       &psdf_set_fill_color_commands);
 	    pres = pdf_find_resource_by_gs_id(pdev, resourceCharProc, id);
 	    if (pres == 0) {	/* Define the character in an embedded font. */
 		pdf_char_proc_t *pcp;
@@ -162,23 +163,30 @@ pdf_copy_mono(gx_device_pdf *pdev,
 		    return code;
 		pres = (pdf_resource_t *) pcp;
 		goto wr;
-	    }
+	    } else if (pdev->pte) {
+		/* We're under pdf_text_process. It set a high level color. */
+	    } else
+		pdf_set_pure_color(pdev, one, &pdev->saved_fill_color,
+				   &pdev->fill_used_process_color,
+				   &psdf_set_fill_color_commands);
 	    pdf_make_bitmap_matrix(&image.ImageMatrix, x, y, w, h, h);
 	    goto rx;
 	}
-	pdf_set_pure_color(pdev, one, &pdev->fill_color,
+	pdf_set_pure_color(pdev, one, &pdev->saved_fill_color,
+			   &pdev->fill_used_process_color,
 			   &psdf_set_fill_color_commands);
 	gs_image_t_init_mask(&image, false);
 	invert = 0xff;
     } else if (one == gx_no_color_index) {
 	gs_image_t_init_mask(&image, false);
-	pdf_set_pure_color(pdev, zero, &pdev->fill_color,
+	pdf_set_pure_color(pdev, zero, &pdev->saved_fill_color,
+			   &pdev->fill_used_process_color,
 			   &psdf_set_fill_color_commands);
     } else if (zero == pdev->black && one == pdev->white) {
-	gs_cspace_init_DeviceGray(&cs);
+	gs_cspace_init_DeviceGray(pdev->memory, &cs);
 	gs_image_t_init(&image, &cs);
     } else if (zero == pdev->white && one == pdev->black) {
-	gs_cspace_init_DeviceGray(&cs);
+	gs_cspace_init_DeviceGray(pdev->memory, &cs);
 	gs_image_t_init(&image, &cs);
 	invert = 0xff;
     } else {
@@ -194,12 +202,12 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	int ncomp = pdev->color_info.num_components;
 	byte *p;
 
-	code = pdf_cspace_init_Device(&cs_base, ncomp);
+	code = pdf_cspace_init_Device(pdev->memory, &cs_base, ncomp);
 	if (code < 0)
 	    return code;
 	c[0] = psdf_adjust_color_index((gx_device_vector *)pdev, zero);
 	c[1] = psdf_adjust_color_index((gx_device_vector *)pdev, one);
-	gs_cspace_init(&cs, &gs_color_space_type_Indexed, NULL);
+	gs_cspace_init(&cs, &gs_color_space_type_Indexed, pdev->memory, false);
 	cs.params.indexed.base_space = *(gs_direct_color_space *)&cs_base;
 	cs.params.indexed.hival = 1;
 	p = palette;
@@ -216,7 +224,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
     {
 	ulong nbytes = (ulong) ((w + 7) >> 3) * h;
 
-	in_line = nbytes <= MAX_INLINE_IMAGE_BYTES;
+	in_line = nbytes < pdev->MaxInlineImageSize;
 	if (in_line)
 	    pdf_put_image_matrix(pdev, &image.ImageMatrix, 1.0);
 	code = pdf_open_page(pdev, PDF_IN_STREAM);
@@ -334,7 +342,7 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     gs_color_space cs;
     cos_value_t cs_value;
     ulong nbytes;
-    int code = pdf_cspace_init_Device(&cs, bytes_per_pixel);
+    int code = pdf_cspace_init_Device(pdev->memory, &cs, bytes_per_pixel);
     const byte *row_base;
     int row_step;
     bool in_line;
@@ -357,7 +365,7 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     } else {
 	row_base = base;
 	row_step = raster;
-	in_line = nbytes <= MAX_INLINE_IMAGE_BYTES;
+	in_line = nbytes < pdev->MaxInlineImageSize;
 	pdf_put_image_matrix(pdev, &pim->ImageMatrix, 1.0);
 	/*
 	 * Check whether we've already made an XObject resource for this
@@ -411,7 +419,9 @@ gdev_pdf_copy_color(gx_device * dev, const byte * base, int sourcex,
     if (code < 0)
 	return code;
     /* Make sure we aren't being clipped. */
-    pdf_put_clip_path(pdev, NULL);
+    code = pdf_put_clip_path(pdev, NULL);
+    if (code < 0)
+	return code;
     code = pdf_copy_color_data(pdev, base, sourcex, raster, id, x, y, w, h,
 			       &image, &writer, 0);
     switch (code) {
@@ -494,7 +504,7 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 	gs_image_t image;
 	pdf_image_writer writer;
 	long image_bytes = ((long)tw * depth + 7) / 8 * th;
-	bool in_line = image_bytes <= MAX_INLINE_IMAGE_BYTES;
+	bool in_line = image_bytes < pdev->MaxInlineImageSize;
 	ulong tile_id =
 	    (tw == tiles->size.x && th == tiles->size.y ? tiles->id :
 	     gx_no_bitmap_id);
@@ -568,7 +578,9 @@ gdev_pdf_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 	if (code < 0)
 	    goto use_default;
 	/* Make sure we aren't being clipped. */
-	pdf_put_clip_path(pdev, NULL);
+	code = pdf_put_clip_path(pdev, NULL);
+	if (code < 0)
+	    return code;
 	s = pdev->strm;
 	/*
 	 * Because of bugs in Acrobat Reader's Print function, we can't
