@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdfp.c,v 1.30.2.5 2004/06/18 18:02:10 igor Exp $ */
+/* $Id: gdevpdfp.c 5479 2004-11-01 21:09:08Z igor $ */
 /* Get/put parameters for PDF-writing driver */
 #include "memory_.h"
 #include "string_.h"
@@ -74,6 +74,7 @@ private const gs_param_item_t pdf_param_items[] = {
     pi("ReEncodeCharacters", gs_param_type_bool, ReEncodeCharacters),
     pi("FirstObjectNumber", gs_param_type_long, FirstObjectNumber),
     pi("CompressFonts", gs_param_type_bool, CompressFonts),
+    pi("PrintStatistics", gs_param_type_bool, PrintStatistics),
     pi("MaxInlineImageSize", gs_param_type_long, MaxInlineImageSize),
 
 	/* PDF Encryption */
@@ -83,6 +84,9 @@ private const gs_param_item_t pdf_param_items[] = {
     pi("Permissions", gs_param_type_int, Permissions),
     pi("EncryptionR", gs_param_type_int, EncryptionR),
     pi("NoEncrypt", gs_param_type_string, NoEncrypt),
+#if PS2WRITE
+    pi("OrderResources", gs_param_type_bool, OrderResources), /* A temporary development purpose. */
+#endif
 #undef pi
     gs_param_item_end
 };
