@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdfv.c,v 1.20 2002/11/18 22:23:40 igor Exp $ */
+/* $Id: gdevpdfv.c 5479 2004-11-01 21:09:08Z igor $ */
 /* Color value writing for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -105,6 +105,7 @@ tile_size_ok(const gx_device_pdf *pdev, const gx_color_tile *p_tile,
 	(m_tile == 0 ? 0 : tile_size(&m_tile->tmask, 1));
     return (max(p_size, m_size) <= 65500);
 }
+
 private int
 pdf_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	    const gx_color_tile *p_tile, const gx_color_tile *m_tile,
@@ -141,6 +142,8 @@ pdf_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
     gs_make_identity(&smat);
     smat.xx = btile->rep_width / (pdev->HWResolution[0] / 72.0);
     smat.yy = btile->rep_height / (pdev->HWResolution[1] / 72.0);
+    smat.tx = tile->step_matrix.tx / (pdev->HWResolution[0] / 72.0);
+    smat.ty = tile->step_matrix.ty / (pdev->HWResolution[1] / 72.0);
     pres = *ppres;
     {
 	cos_dict_t *pcd_XObject = cos_dict_alloc(pdev, "pdf_pattern(XObject)");
@@ -189,6 +192,60 @@ pdf_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
     return 0;
 }
 
+/* Store pattern 1 parameters to cos dictionary. */
+int 
+pdf_store_pattern1_params(gx_device_pdf *pdev, pdf_resource_t *pres, 
+			gs_pattern1_instance_t *pinst)
+{
+    gs_pattern1_template_t *t = &pinst->template;
+    gs_matrix smat = ctm_only((gs_imager_state *)pinst->saved);
+    double scale_x = pdev->HWResolution[0] / 72.0;
+    double scale_y = pdev->HWResolution[1] / 72.0;
+    cos_dict_t *pcd = cos_stream_dict((cos_stream_t *)pres->object);
+    cos_dict_t *pcd_Resources = cos_dict_alloc(pdev, "pdf_pattern(Resources)");
+    char buf[60];
+    int code;
+
+    if (pcd == NULL || pcd_Resources == NULL)
+	return_error(gs_error_VMerror);
+    pdev->substream_Resources = pcd_Resources;
+    sprintf(buf, "[%g %g %g %g]", t->BBox.p.x, t->BBox.p.y, 
+				  t->BBox.q.x, t->BBox.q.y);
+    /* The graphics library assumes a shifted origin to provide 
+       positive bitmap pixel indices. Compensate it now. */
+    smat.tx += pinst->step_matrix.tx;
+    smat.ty += pinst->step_matrix.ty;
+    smat.xx /= scale_x;
+    smat.xy /= scale_x;
+    smat.yx /= scale_y;
+    smat.yy /= scale_y;
+    smat.tx /= scale_x;
+    smat.ty /= scale_y;
+    if (any_abs(smat.tx) < 0.0001)  /* Noise. */
+	smat.tx = 0;
+    if (any_abs(smat.ty) < 0.0001)
+	smat.ty = 0;
+    code = cos_dict_put_c_strings(pcd, "/Type", "/Pattern");
+    if (code >= 0)
+	code = cos_dict_put_c_key_int(pcd, "/PatternType", 1);
+    if (code >= 0)
+	code = cos_dict_put_c_key_int(pcd, "/PaintType", t->PaintType);
+    if (code >= 0)
+	code = cos_dict_put_c_key_int(pcd, "/TilingType", t->TilingType);
+    if (code >= 0)
+	code = cos_dict_put_string(pcd, (byte *)"/BBox", 5, (byte *)buf, strlen(buf));
+    if (code >= 0)
+	code = cos_dict_put_matrix(pcd, "/Matrix", &smat);
+    if (code >= 0)
+	code = cos_dict_put_c_key_real(pcd, "/XStep", t->XStep);
+    if (code >= 0)
+	code = cos_dict_put_c_key_real(pcd, "/YStep", t->YStep);
+    if (code >= 0)
+	code = cos_dict_put_c_key_object(pcd, "/Resources", COS_OBJECT(pcd_Resources));
+    pdev->skip_colors = (t->PaintType == 2);
+    return code;
+}
+
 /* Set the ImageMatrix, Width, and Height for a Pattern image. */
 private void
 pdf_set_pattern_image(gs_data_image_t *pic, const gx_strip_bitmap *tile)
@@ -205,7 +262,6 @@ pdf_put_pattern_mask(gx_device_pdf *pdev, const gx_color_tile *m_tile,
     int w = m_tile->tmask.rep_width, h = m_tile->tmask.rep_height;
     gs_image1_t image;
     pdf_image_writer writer;
-    cos_stream_t *pcs_image;
     int code;
 
     gs_image_t_init_mask_adjust(&image, true, false);
@@ -216,34 +272,33 @@ pdf_put_pattern_mask(gx_device_pdf *pdev, const gx_color_tile *m_tile,
 	(code = pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image, NULL, 0)) < 0
 	)
 	return code;
-    pcs_image = (cos_stream_t *)writer.pres->object;
     /* Pattern masks are specified in device coordinates, so invert Y. */
     if ((code = pdf_copy_mask_bits(writer.binary[0].strm, m_tile->tmask.data + (h - 1) * m_tile->tmask.raster, 0, -m_tile->tmask.raster, w, h, 0)) < 0 ||
 	(code = pdf_end_image_binary(pdev, &writer, h)) < 0 ||
 	(code = pdf_end_write_image(pdev, &writer)) < 0
 	)
 	return code;
-    *ppcs_mask = pcs_image;
+    *ppcs_mask = (cos_stream_t *)writer.pres->object;
     return 0;
 }
 
 /* Write an uncolored Pattern color. */
-/* (Single-use procedure for readability.) */
-private int
+int
 pdf_put_uncolored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
+			  const gs_color_space *pcs,
 			  const psdf_set_color_commands_t *ppscc,
-			  pdf_resource_t **ppres)
+			  bool have_pattern_streams, pdf_resource_t **ppres)
 {
     const gx_color_tile *m_tile = pdc->mask.m_tile;
     gx_drawing_color dc_pure;
 
-    color_set_pure(&dc_pure, gx_dc_pure_color(pdc));
-    if (m_tile == 0) {
+    if (!have_pattern_streams && m_tile == 0) {
 	/*
 	 * If m_tile == 0, this uncolored Pattern is all 1's,
 	 * equivalent to a pure color.
 	 */
 	*ppres = 0;
+	set_nonclient_dev_color(&dc_pure, gx_dc_pure_color(pdc));
 	return psdf_set_color((gx_device_vector *)pdev, &dc_pure, ppscc);
     } else {
 	cos_value_t v;
@@ -254,98 +309,120 @@ pdf_put_uncolored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 
 	if (!tile_size_ok(pdev, NULL, m_tile))
 	    return_error(gs_error_limitcheck);
-	if ((code = pdf_cs_Pattern_uncolored(pdev, &v)) < 0 ||
-	    (code = pdf_put_pattern_mask(pdev, m_tile, &pcs_image)) < 0 ||
-	    (code = pdf_pattern(pdev, pdc, NULL, m_tile, pcs_image, ppres)) < 0
-	    )
-	    return code;
+	if (!have_pattern_streams) {
+	    if ((code = pdf_cs_Pattern_uncolored(pdev, &v)) < 0 ||
+		(code = pdf_put_pattern_mask(pdev, m_tile, &pcs_image)) < 0 ||
+		(code = pdf_pattern(pdev, pdc, NULL, m_tile, pcs_image, ppres)) < 0
+		)
+		return code;
+	} else {
+	    code = pdf_cs_Pattern_uncolored_hl(pdev, pcs, &v);
+	    if (code < 0)
+		return code;
+	    *ppres = pdf_find_resource_by_gs_id(pdev, resourcePattern, pdc->mask.id);
+	    *ppres = pdf_substitute_pattern(*ppres);
+	    if (!pdev->AR4_save_bug && pdev->CompatibilityLevel <= 1.3) {
+		/* We reconnized AR4 behavior as reserving "q Q" stack elements 
+		 * on demand. It looks as processing a pattern stream
+		 * with PaintType 1 AR4 replaces the topmost stack element
+		 * instead allocating a new one, if it was not previousely allocated.
+		 * AR 5 doesn't have this bug. Working around the AR4 bug here.
+		 */
+		stream_puts(pdev->strm, "q q Q Q\n");
+		pdev->AR4_save_bug = true;
+	    }
+	}
 	cos_value_write(&v, pdev);
 	pprints1(s, " %s ", ppscc->setcolorspace);
+	if (have_pattern_streams)
+	    return 0;
+	set_nonclient_dev_color(&dc_pure, gx_dc_pure_color(pdc));
 	return psdf_set_color((gx_device_vector *)pdev, &dc_pure, &no_scc);
     }
 }
 
-/* Write a colored Pattern color. */
-/* (Single-use procedure for readability.) */
-private int
+int
 pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
+			const gs_color_space *pcs,
 			const psdf_set_color_commands_t *ppscc,
-			pdf_resource_t **ppres)
+			bool have_pattern_streams, pdf_resource_t **ppres)
 {
-    const gx_color_tile *m_tile = pdc->mask.m_tile;
     const gx_color_tile *p_tile = pdc->colors.pattern.p_tile;
-    int w = p_tile->tbits.rep_width, h = p_tile->tbits.rep_height;
     gs_color_space cs_Device;
     cos_value_t cs_value;
-    pdf_image_writer writer;
-    gs_image1_t image;
-    cos_stream_t *pcs_image;
-    cos_stream_t *pcs_mask = 0;
     cos_value_t v;
     int code;
+    gs_image1_t image;
+    const gx_color_tile *m_tile = NULL;
+    pdf_image_writer writer;
+    int w = p_tile->tbits.rep_width, h = p_tile->tbits.rep_height;
 
-    /*
-     * NOTE: We assume here that the color space of the cached Pattern
-     * is the same as the native color space of the device.  This will
-     * have to change in the future!
-     */
-    /*
-     * Check whether this colored pattern is actually a masked pure color,
-     * by testing whether all the colored pixels have the same color.
-     */
-    if (m_tile) {
-	if (p_tile && !(p_tile->depth & 7) && p_tile->depth <= sizeof(gx_color_index) * 8) {
-	    int depth_bytes = p_tile->depth >> 3;
-	    int width = p_tile->tbits.rep_width;
-	    int skip = p_tile->tbits.raster -
-		p_tile->tbits.rep_width * depth_bytes;
-	    const byte *bp;
-	    const byte *mp;
-	    int i, j, k;
-	    gx_color_index color = 0; /* init is arbitrary if not empty */
-	    bool first = true;
+    if (!have_pattern_streams) {
+	/*
+	 * NOTE: We assume here that the color space of the cached Pattern
+	 * is the same as the native color space of the device.  This will
+	 * have to change in the future!
+	 */
+	/*
+	 * Check whether this colored pattern is actually a masked pure color,
+	 * by testing whether all the colored pixels have the same color.
+	 */
+	m_tile = pdc->mask.m_tile;
+	if (m_tile) {
+	    if (p_tile && !(p_tile->depth & 7) && p_tile->depth <= sizeof(gx_color_index) * 8) {
+		int depth_bytes = p_tile->depth >> 3;
+		int width = p_tile->tbits.rep_width;
+		int skip = p_tile->tbits.raster -
+		    p_tile->tbits.rep_width * depth_bytes;
+		const byte *bp;
+		const byte *mp;
+		int i, j, k;
+		gx_color_index color = 0; /* init is arbitrary if not empty */
+		bool first = true;
 
-	    for (i = 0, bp = p_tile->tbits.data, mp = p_tile->tmask.data;
-		 i < p_tile->tbits.rep_height;
-		 ++i, bp += skip, mp += p_tile->tmask.raster) {
+		for (i = 0, bp = p_tile->tbits.data, mp = p_tile->tmask.data;
+		     i < p_tile->tbits.rep_height;
+		     ++i, bp += skip, mp += p_tile->tmask.raster) {
 
-		for (j = 0; j < width; ++j) {
-		    if (mp[j >> 3] & (0x80 >> (j & 7))) {
-			gx_color_index ci = 0;
-		    
-			for (k = 0; k < depth_bytes; ++k)
-			    ci = (ci << 8) + *bp++;
-			if (first)
-			    color = ci, first = false;
-			else if (ci != color)
-			    goto not_pure;
-		    } else
-			bp += depth_bytes;
+		    for (j = 0; j < width; ++j) {
+			if (mp[j >> 3] & (0x80 >> (j & 7))) {
+			    gx_color_index ci = 0;
+			
+			    for (k = 0; k < depth_bytes; ++k)
+				ci = (ci << 8) + *bp++;
+			    if (first)
+				color = ci, first = false;
+			    else if (ci != color)
+				goto not_pure;
+			} else
+			    bp += depth_bytes;
+		    }
 		}
-	    }
-	    {
-		/* Set the color, then handle as an uncolored pattern. */
-		gx_drawing_color dcolor;
+		{
+		    /* Set the color, then handle as an uncolored pattern. */
+		    gx_drawing_color dcolor;
 
-		dcolor = *pdc;
-		dcolor.colors.pure = color;
-		return pdf_put_uncolored_pattern(pdev, &dcolor, ppscc, ppres);
+		    dcolor = *pdc;
+		    dcolor.colors.pure = color;
+		    return pdf_put_uncolored_pattern(pdev, &dcolor, pcs, ppscc, 
+				have_pattern_streams, ppres);
+		}
+	    not_pure:
+		DO_NOTHING;		/* required by MSVC */
+	    }
+	    if (pdev->CompatibilityLevel < 1.3) {
+		/* Masked images are only supported starting in PDF 1.3. */
+		return_error(gs_error_rangecheck);
 	    }
-	not_pure:
-	    DO_NOTHING;		/* required by MSVC */
-	}
-	if (pdev->CompatibilityLevel < 1.3) {
-	    /* Masked images are only supported starting in PDF 1.3. */
-	    return_error(gs_error_rangecheck);
 	}
+	/* Acrobat Reader has a size limit for image Patterns. */
+	if (!tile_size_ok(pdev, p_tile, m_tile))
+	    return_error(gs_error_limitcheck);
     }
-    /* Acrobat Reader has a size limit for image Patterns. */
-    if (!tile_size_ok(pdev, p_tile, m_tile))
-	return_error(gs_error_limitcheck);
     code = pdf_cs_Pattern_colored(pdev, &v);
     if (code < 0)
 	return code;
-    pdf_cspace_init_Device(&cs_Device, pdev->color_info.num_components);
+    pdf_cspace_init_Device(pdev->memory, &cs_Device, pdev->color_info.num_components);
     /*
      * We don't have to worry about color space scaling: the color
      * space is always a Device space.
@@ -354,41 +431,50 @@ pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 			   &pdf_color_space_names, true);
     if (code < 0)
 	return code;
-    gs_image_t_init_adjust(&image, &cs_Device, false);
-    image.BitsPerComponent = 8;
-    pdf_set_pattern_image((gs_data_image_t *)&image, &p_tile->tbits);
-    if (m_tile) {
-	if ((code = pdf_put_pattern_mask(pdev, m_tile, &pcs_mask)) < 0)
+    if (!have_pattern_streams) {
+	cos_stream_t *pcs_mask = 0;
+	cos_stream_t *pcs_image;
+
+	gs_image_t_init_adjust(&image, &cs_Device, false);
+	image.BitsPerComponent = 8;
+	pdf_set_pattern_image((gs_data_image_t *)&image, &p_tile->tbits);
+	if (m_tile) {
+	    if ((code = pdf_put_pattern_mask(pdev, m_tile, &pcs_mask)) < 0)
+		return code;
+	}
+	if ((code = pdf_begin_write_image(pdev, &writer, gs_no_id, w, h, NULL, false, 1)) < 0 ||
+	    (code = psdf_setup_lossless_filters((gx_device_psdf *)pdev,
+						&writer.binary[0],
+						(gs_pixel_image_t *)&image)) < 0 ||
+	    (code = pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image, &cs_value, 0)) < 0
+	    )
 	    return code;
+	/* Pattern masks are specified in device coordinates, so invert Y. */
+	if ((code = pdf_copy_color_bits(writer.binary[0].strm, p_tile->tbits.data + (h - 1) * p_tile->tbits.raster, 0, -p_tile->tbits.raster, w, h, pdev->color_info.depth >> 3)) < 0 ||
+	    (code = pdf_end_image_binary(pdev, &writer, h)) < 0
+	    )
+	    return code;
+	pcs_image = (cos_stream_t *)writer.pres->object;
+	if ((pcs_mask != 0 &&
+	     (code = cos_dict_put_c_key_object(cos_stream_dict(pcs_image), "/Mask",
+					       COS_OBJECT(pcs_mask))) < 0) ||
+	    (code = pdf_end_write_image(pdev, &writer)) < 0
+	    )
+	    return code;
+	pcs_image = (cos_stream_t *)writer.pres->object; /* pdf_end_write_image may change it. */
+	code = pdf_pattern(pdev, pdc, p_tile, m_tile, pcs_image, ppres);
+	if (code < 0)
+	    return code;
+    } else {
+	*ppres = pdf_find_resource_by_gs_id(pdev, resourcePattern, p_tile->id);
+	*ppres = pdf_substitute_pattern(*ppres);
     }
-    if ((code = pdf_begin_write_image(pdev, &writer, gs_no_id, w, h, NULL, false, 1)) < 0 ||
-	(code = psdf_setup_lossless_filters((gx_device_psdf *)pdev,
-					    &writer.binary[0],
-					    (gs_pixel_image_t *)&image)) < 0 ||
-	(code = pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image, &cs_value, 0)) < 0
-	)
-	return code;
-    pcs_image = (cos_stream_t *)writer.pres->object;
-    /* Pattern masks are specified in device coordinates, so invert Y. */
-    if ((code = pdf_copy_color_bits(writer.binary[0].strm, p_tile->tbits.data + (h - 1) * p_tile->tbits.raster, 0, -p_tile->tbits.raster, w, h, pdev->color_info.depth >> 3)) < 0 ||
-	(code = pdf_end_image_binary(pdev, &writer, h)) < 0
-	)
-	return code;
-    pcs_image = (cos_stream_t *)writer.pres->object;
-    if ((pcs_mask != 0 &&
-	 (code = cos_dict_put_c_key_object(cos_stream_dict(pcs_image), "/Mask",
-					   COS_OBJECT(pcs_mask))) < 0) ||
-	(code = pdf_end_write_image(pdev, &writer)) < 0
-	)
-	return code;
-    code = pdf_pattern(pdev, pdc, p_tile, m_tile, pcs_image, ppres);
-    if (code < 0)
-	return code;
     cos_value_write(&v, pdev);
     pprints1(pdev->strm, " %s", ppscc->setcolorspace);
     return 0;
 }
 
+
 /* ---------------- PatternType 2 colors ---------------- */
 
 /* Write parameters common to all Shadings. */
@@ -619,7 +705,6 @@ put_float_mesh_data(cos_stream_t *pscs, shade_coord_stream_t *cs,
 }
 
 /* Write a mesh Shading. */
-/* (Single-use procedure for readability.) */
 private int
 pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
 		     const gs_range_t *pranges)
@@ -628,7 +713,7 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
     gs_color_space *pcs = psh->params.ColorSpace;
     const gs_shading_mesh_params_t *const pmp =
 	(const gs_shading_mesh_params_t *)&psh->params;
-    int code;
+    int code, code1;
     int bits_per_coordinate, bits_per_component, bits_per_flag;
     int num_comp;
     bool from_array = data_source_is_array(pmp->DataSource);
@@ -715,11 +800,14 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
 
 	data_params.num_points = 1;
 	data_params.num_components = num_comp;
-	if (from_array)
+	if (from_array) {
 	    while ((flag = shade_next_flag(&cs, 0)) >= 0)
 		if ((code = put_float_mesh_data(pscs, &cs, flag,
 						&data_params)) < 0)
 		    return code;
+	    if (!seofp(cs.s))
+		code = gs_note_error(gs_error_rangecheck);
+	}
 	if (bits_per_flag < 0)
 	    bits_per_flag = params->BitsPerFlag;
 	break;
@@ -735,14 +823,15 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
 		if ((code = put_float_mesh_data(pscs, &cs, -1,
 						&data_params)) < 0)
 		    return code;
-	return cos_dict_put_c_key_int(pscd, "/VerticesPerRow",
+	code = cos_dict_put_c_key_int(pscd, "/VerticesPerRow",
 				      params->VerticesPerRow);
+	return code;
     }
     case shading_type_Coons_patch: {
 	const gs_shading_Cp_params_t *const params =
 	    (const gs_shading_Cp_params_t *)pmp;
 
-	if (from_array)
+	if (from_array) {
 	    while ((flag = shade_next_flag(&cs, 0)) >= 0) {
 		data_params.num_points = (flag == 0 ? 12 : 8);
 		data_params.num_components = num_comp * (flag == 0 ? 4 : 2);
@@ -750,6 +839,9 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
 						&data_params)) < 0)
 		    return code;
 	    }
+	    if (!seofp(cs.s))
+		code = gs_note_error(gs_error_rangecheck);
+	}
 	if (bits_per_flag < 0)
 	    bits_per_flag = params->BitsPerFlag;
 	break;
@@ -758,7 +850,7 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
 	const gs_shading_Tpp_params_t *const params =
 	    (const gs_shading_Tpp_params_t *)pmp;
 
-	if (from_array)
+	if (from_array) {
 	    while ((flag = shade_next_flag(&cs, 0)) >= 0) {
 		data_params.num_points = (flag == 0 ? 16 : 12);
 		data_params.num_components = num_comp * (flag == 0 ? 4 : 2);
@@ -766,6 +858,9 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
 						&data_params)) < 0)
 		    return code;
 	    }
+	    if (!seofp(cs.s))
+		code = gs_note_error(gs_error_rangecheck);
+	}
 	if (bits_per_flag < 0)
 	    bits_per_flag = params->BitsPerFlag;
 	break;
@@ -773,12 +868,14 @@ pdf_put_mesh_shading(cos_stream_t *pscs, const gs_shading_t *psh,
     default:
 	return_error(gs_error_rangecheck);
     }
-    return cos_dict_put_c_key_int(pscd, "/BitsPerFlag", bits_per_flag);
+    code1 =  cos_dict_put_c_key_int(pscd, "/BitsPerFlag", bits_per_flag);
+    if (code1 < 0)
+	return code;
+    return code;
 }
 
 /* Write a PatternType 2 (shading pattern) color. */
-/* (Single-use procedure for readability.) */
-private int
+int
 pdf_put_pattern2(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 		 const psdf_set_color_commands_t *ppscc,
 		 pdf_resource_t **ppres)
@@ -793,6 +890,7 @@ pdf_put_pattern2(gx_device_pdf *pdev, const gx_drawing_color *pdc,
     cos_object_t *psco;
     const gs_range_t *pranges;
     int code = pdf_cs_Pattern_colored(pdev, &v);
+    int code1 = 0;
     gs_matrix smat;
 
     if (code < 0)
@@ -813,7 +911,7 @@ pdf_put_pattern2(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	code = pdf_put_shading_common(cos_stream_dict((cos_stream_t *)psco),
 				      psh, &pranges);
 	if (code >= 0)
-	    code = pdf_put_mesh_shading((cos_stream_t *)psco, psh, pranges);
+	    code1 = pdf_put_mesh_shading((cos_stream_t *)psco, psh, pranges);
     } else {
 	cos_become(psco, cos_type_dict);
 	code = pdf_put_shading_common((cos_dict_t *)psco, psh, &pranges);
@@ -841,42 +939,18 @@ pdf_put_pattern2(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	return code;
     cos_value_write(&v, pdev);
     pprints1(pdev->strm, " %s", ppscc->setcolorspace);
-    return 0;
+    return code1;
 }
 
-/* ---------------- Public procedure ---------------- */
-
-/* Write a color value. */
+/*
+    Include color space.
+ */
 int
-pdf_put_drawing_color(gx_device_pdf *pdev, const gx_drawing_color *pdc,
-		      const psdf_set_color_commands_t *ppscc)
+gdev_pdf_include_color_space(gx_device *dev, gs_color_space *cspace, const byte *res_name, int name_length)
 {
-    if (gx_dc_is_pure(pdc))
-	return psdf_set_color((gx_device_vector *) pdev, pdc, ppscc);
-    else {
-	/* We never halftone, so this must be a Pattern. */
-	int code;
-	pdf_resource_t *pres;
-	cos_value_t v;
+    gx_device_pdf * pdev = (gx_device_pdf *)dev;
+    cos_value_t cs_value;
 
-	if (pdc->type == gx_dc_type_pattern)
-	    code = pdf_put_colored_pattern(pdev, pdc, ppscc, &pres);
-	else if (pdc->type == &gx_dc_pure_masked)
-	    code = pdf_put_uncolored_pattern(pdev, pdc, ppscc, &pres);
-	else if (pdc->type == &gx_dc_pattern2)
-	    code = pdf_put_pattern2(pdev, pdc, ppscc, &pres);
-	else
-	    return_error(gs_error_rangecheck);
-	if (code < 0)
-	    return code;
-	/*
-	 * Uncolored patterns equivalent to a solid color don't need an
-	 * associated resource, and set pres = 0.
-	 */
-	if (pres) {
-	    cos_value_write(cos_resource_value(&v, pres->object), pdev);
-	    pprints1(pdev->strm, " %s\n", ppscc->setcolorn);
-	}
-	return 0;
-    }
+    return pdf_color_space_named(pdev, &cs_value, NULL, cspace,
+				&pdf_color_space_names, true, res_name, name_length);
 }
