@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdtd.c,v 1.6 2002/10/07 08:28:56 ghostgum Exp $ */
+/* $Id: gdevpdtd.c 5471 2004-10-28 17:34:34Z igor $ */
 /* FontDescriptor implementation for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
@@ -216,7 +216,9 @@ pdf_font_descriptor_alloc(gx_device_pdf *pdev, pdf_font_descriptor_t **ppfd,
 {
     pdf_font_descriptor_t *pfd;
     pdf_base_font_t *pbfont;
-    int code = pdf_base_font_alloc(pdev, &pbfont, font, false);
+    int code = pdf_base_font_alloc(pdev, &pbfont, font, 
+		(font->orig_FontMatrix.xx == 0 && font->orig_FontMatrix.xy == 0 
+		    ? &font->FontMatrix : &font->orig_FontMatrix), false, !embed);
 
     if (code < 0)
 	return code;
@@ -261,6 +263,15 @@ pdf_font_descriptor_embedding(const pdf_font_descriptor_t *pfd)
     return pfd->embed;
 }
 
+/*
+ * Check for subset font.
+ */
+bool
+pdf_font_descriptor_is_subset(const pdf_font_descriptor_t *pfd)
+{
+    return pdf_base_font_is_subset(pfd->base_font);
+}
+
 /*
  * Return a reference to the FontName of a FontDescriptor, similar to
  * pdf_base_font_name.
@@ -271,13 +282,22 @@ gs_string *pdf_font_descriptor_name(pdf_font_descriptor_t *pfd)
 }
 
 /*
- * Return the (copied, subset) font associated with a FontDescriptor.
+ * Return the (copied, subset or complete) font associated with a FontDescriptor.
  * This procedure probably shouldn't exist....
  */
 gs_font_base *
-pdf_font_descriptor_font(const pdf_font_descriptor_t *pfd)
+pdf_font_descriptor_font(const pdf_font_descriptor_t *pfd, bool complete)
 {
-    return pdf_base_font_font(pfd->base_font);
+    return pdf_base_font_font(pfd->base_font, complete);
+}
+
+/*
+ * Drop the copied complete font associated with a FontDescriptor.
+ */
+void
+pdf_font_descriptor_drop_complete_font(const pdf_font_descriptor_t *pfd)
+{
+    pdf_base_font_drop_complete(pfd->base_font);
 }
 
 /*
@@ -304,7 +324,7 @@ pdf_font_used_glyph(pdf_font_descriptor_t *pfd, gs_glyph glyph,
 int
 pdf_compute_font_descriptor(pdf_font_descriptor_t *pfd)
 {
-    gs_font_base *bfont = pdf_base_font_font(pfd->base_font);
+    gs_font_base *bfont = pdf_base_font_font(pfd->base_font, false);
     gs_glyph glyph, notdef;
     int index;
     int wmode = bfont->WMode;
@@ -319,12 +339,26 @@ pdf_compute_font_descriptor(pdf_font_descriptor_t *pfd)
     int x_height = min_int;
     int cap_height = 0;
     gs_rect bbox_colon, bbox_period, bbox_I;
+    bool is_cid = (bfont->FontType == ft_CID_encrypted ||
+		   bfont->FontType == ft_CID_TrueType);
     bool have_colon = false, have_period = false, have_I = false;
     int code;
 
     memset(&desc, 0, sizeof(desc));
-    desc.FontBBox.p.x = desc.FontBBox.p.y = max_int;
-    desc.FontBBox.q.x = desc.FontBBox.q.y = min_int;
+    if (is_cid && bfont->FontBBox.p.x != bfont->FontBBox.q.x &&
+		  bfont->FontBBox.p.y != bfont->FontBBox.q.y) {
+	int scale = (bfont->FontType == ft_TrueType || bfont->FontType == ft_CID_TrueType ? 1000 : 1);
+
+	desc.FontBBox.p.x = (int)(bfont->FontBBox.p.x * scale);
+	desc.FontBBox.p.y = (int)(bfont->FontBBox.p.y * scale);
+	desc.FontBBox.p.x = (int)(bfont->FontBBox.p.x * scale);
+	desc.FontBBox.q.y = (int)(bfont->FontBBox.q.y * scale);
+	desc.Ascent = desc.FontBBox.q.y;
+	members &= ~GLYPH_INFO_BBOX;
+    } else {
+	desc.FontBBox.p.x = desc.FontBBox.p.y = max_int;
+	desc.FontBBox.q.x = desc.FontBBox.q.y = min_int;
+    }
     /*
      * Embedded TrueType fonts use a 1000-unit character space, but the
      * font itself uses a 1-unit space.  Compensate for this here.
@@ -359,13 +393,16 @@ pdf_compute_font_descriptor(pdf_font_descriptor_t *pfd)
      */
     notdef = GS_NO_GLYPH;
     for (index = 0;
-	 (code = bfont->procs.enumerate_glyph((gs_font *)bfont, &index, GLYPH_SPACE_NAME, &glyph)) >= 0 &&
+	 (code = bfont->procs.enumerate_glyph((gs_font *)bfont, &index, 
+		(is_cid ? GLYPH_SPACE_INDEX : GLYPH_SPACE_NAME), &glyph)) >= 0 &&
 	     index != 0;
 	 ) {
 	gs_glyph_info_t info;
 	gs_const_string gname;
 
 	code = bfont->procs.glyph_info((gs_font *)bfont, glyph, pmat, members, &info);
+	if (code == gs_error_VMerror)
+	    return code;
 	if (code < 0) {
 	    /*
 	     * Since this function may be indirtectly called from gx_device_finalize,
@@ -375,21 +412,29 @@ pdf_compute_font_descriptor(pdf_font_descriptor_t *pfd)
 	     */
 	    continue;
 	}
-	rect_merge(desc.FontBBox, info.bbox);
-	if (!info.num_pieces)
-	    desc.Ascent = max(desc.Ascent, info.bbox.q.y);
+	if (members & GLYPH_INFO_BBOX) {
+	    /* rect_merge(desc.FontBBox, info.bbox); Expanding due to type cast :*/
+	    if (info.bbox.p.x < desc.FontBBox.p.x) desc.FontBBox.p.x = (int)info.bbox.p.x;
+	    if (info.bbox.q.x > desc.FontBBox.q.x) desc.FontBBox.q.x = (int)info.bbox.q.x;
+	    if (info.bbox.p.y < desc.FontBBox.p.y) desc.FontBBox.p.y = (int)info.bbox.p.y;
+	    if (info.bbox.q.y > desc.FontBBox.q.y) desc.FontBBox.q.y = (int)info.bbox.q.y;
+	    if (!info.num_pieces)
+		desc.Ascent = max(desc.Ascent, (int)info.bbox.q.y);
+	}
 	if (notdef == GS_NO_GLYPH && gs_font_glyph_is_notdef(bfont, glyph)) {
 	    notdef = glyph;
-	    desc.MissingWidth = info.width[wmode].x;
+	    desc.MissingWidth = (int)info.width[wmode].x;
 	}
 	if (info.width[wmode].y != 0)
 	    fixed_width = min_int;
 	else if (fixed_width == 0)
-	    fixed_width = info.width[wmode].x;
+	    fixed_width = (int)info.width[wmode].x;
 	else if (info.width[wmode].x != fixed_width)
 	    fixed_width = min_int;
 	if (desc.Flags & FONT_IS_SYMBOLIC)
 	    continue;		/* skip Roman-only computation */
+	if (is_cid)
+	    continue;
 	code = bfont->procs.glyph_name((gs_font *)bfont, glyph, &gname);
 	if (code < 0)
 	    continue;
@@ -507,14 +552,23 @@ int
 pdf_finish_FontDescriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
 {
     int code = 0;
+    cos_dict_t *pcd;
 
     if (!pfd->common.object->written &&
 	(code = pdf_compute_font_descriptor(pfd)) >= 0 &&
 	(!pfd->embed ||
 	 (code = pdf_write_embedded_font(pdev, pfd->base_font, 
-				&pfd->common.values.FontBBox, pfd->common.rid)) >= 0)
-	)
-	DO_NOTHING;
+				&pfd->common.values.FontBBox, 
+				pfd->common.rid
+#				if PDFW_DELAYED_STREAMS
+				    , &pcd
+#				endif
+				)) >= 0)
+	) {
+#	if PDFW_DELAYED_STREAMS
+	    pdf_set_FontFile_object(pfd->base_font, pcd);
+#	endif
+    }
     return code;
 }
 
@@ -590,5 +644,10 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
     stream_puts(s, ">>\n");
     pdf_end_separate(pdev);
     pfd->common.object->written = true;
+#   if PDFW_DELAYED_STREAMS
+	code = COS_WRITE_OBJECT(pdf_get_FontFile_object(pfd->base_font), pdev);
+	if (code < 0)
+	    return code;
+#   endif
     return 0;
 }
