@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdtf.h,v 1.16.2.2 2004/06/18 18:02:10 igor Exp $ */
+/* $Id: gdevpdtf.h 5431 2004-10-20 15:58:56Z igor $ */
 /* Font and CMap resource structure and API for pdfwrite */
 
 #ifndef gdevpdtf_INCLUDED
@@ -104,6 +104,16 @@ typedef struct pdf_base_font_s pdf_base_font_t;
 typedef struct pdf_font_descriptor_s pdf_font_descriptor_t;
 #endif
 
+#ifndef pdf_char_glyph_pair_DEFINED
+#  define pdf_char_glyph_pair_DEFINED
+typedef struct pdf_char_glyph_pair_s pdf_char_glyph_pair_t;
+#endif
+
+struct pdf_char_glyph_pair_s {
+    gs_char chr;
+    gs_glyph glyph;
+};
+
 /*
  * The write_contents procedure is set by the implementation when the
  * font resource is created.  It is called after generic code has opened
@@ -191,6 +201,7 @@ struct pdf_font_resource_s {
  	    gs_id glyphshow_font_id;
 	    double *Widths2;	/* [count * 2] (x, y) */
 	    double *v;		/* [count] */
+	    byte *used2;	/* [(count + 7) / 8] */
 	    pdf_font_resource_t *parent;
 
 	} cidfont;
@@ -320,7 +331,7 @@ int pdf_font_simple_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 int pdf_font_cidfont_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 			   gs_id rid, pdf_font_descriptor_t *pfd);
 int pdf_obtain_cidfont_widths_arrays(gx_device_pdf *pdev, pdf_font_resource_t *pdfont, 
-		    int wmode, double **w, double **v);
+		    int wmode, double **w, double **w0, double **v);
 int font_resource_encoded_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 			    gs_id rid, font_type ftype,
 			    pdf_font_write_contents_proc_t write_contents);
@@ -344,7 +355,7 @@ gs_font_base *pdf_font_resource_font(const pdf_font_resource_t *pdfont, bool com
  */
 pdf_font_embed_t pdf_font_embed_status(gx_device_pdf *pdev, gs_font *font,
 				       int *pindex,
-				       gs_glyph *glyphs, int num_glyphs);
+				       pdf_char_glyph_pair_t *pairs, int num_glyphs);
 
 /*
  * Compute the BaseFont of a font according to the algorithm described
