@@ -14,12 +14,14 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdtw.c,v 1.10 2002/10/18 14:39:58 igor Exp $ */
+/* $Id: gdevpdtw.c 5431 2004-10-20 15:58:56Z igor $ */
 /* Font resource writing for pdfwrite text */
 #include "memory_.h"
 #include "gx.h"
+#include "gserrors.h"
 #include "gxfcmap.h"
 #include "gxfont.h"
+#include "gscencs.h"
 #include "gdevpsf.h"
 #include "gdevpdfx.h"
 #include "gdevpdfo.h"		/* for object->written */
@@ -27,6 +29,12 @@
 #include "gdevpdtf.h"
 #include "gdevpdti.h"		/* for writing bitmap fonts Encoding */
 #include "gdevpdtw.h"
+#include "gdevpdtv.h"
+#include "sarc4.h"
+
+private const char *const encoding_names[] = {
+    KNOWN_REAL_ENCODING_NAMES
+};
 
 /* ================ Font resource writing ================ */
 
@@ -43,7 +51,7 @@ pdf_write_Widths(gx_device_pdf *pdev, int first, int last, const double *widths)
 	first = last = 0;
     pprintd2(s, "/FirstChar %d/LastChar %d/Widths[", first, last);
     for (i = first; i <= last; ++i)
-	pprintd1(s, (i & 15 ? " %d" : "\n%d"), (int)(widths[i] + 0.5));
+    	pprintg1(s, (i & 15 ? " %g" : "\n%g"), psdf_round(widths[i], 100, 10));
     stream_puts(s, "]\n");
     return 0;
 }
@@ -62,7 +70,7 @@ pdf_different_encoding_element(const pdf_font_resource_t *pdfont, int ch, int en
 {
     if (pdfont->u.simple.Encoding[ch].is_difference)
 	return 1;
-    else if (encoding_index >= 0) {
+    else if (encoding_index != ENCODING_INDEX_UNKNOWN) {
 	gs_glyph glyph0 = gs_c_known_encode(ch, encoding_index);
 	gs_glyph glyph1 = pdfont->u.simple.Encoding[ch].glyph;
 	gs_const_string str;
@@ -77,70 +85,214 @@ pdf_different_encoding_element(const pdf_font_resource_t *pdfont, int ch, int en
     return 0;
 }
 
-/* Write the Subtype and Encoding for a simple font. */
-private int
-pdf_write_simple_contents(gx_device_pdf *pdev,
-			  const pdf_font_resource_t *pdfont)
+/* Find an index of a different encoding element. */
+int
+pdf_different_encoding_index(const pdf_font_resource_t *pdfont, int ch0)
 {
-    stream *s = pdev->strm;
-    static const char *const encoding_names[] = {
-	KNOWN_REAL_ENCODING_NAMES
-    };
     gs_encoding_index_t base_encoding = pdfont->u.simple.BaseEncoding;
-    long diff_id = 0;
-    int ch = (pdfont->u.simple.Encoding ? 0 : 256);
-    int code = 0;
+    int ch, code;
 
-    for (; ch < 256; ++ch) {
+    for (ch = ch0; ch < 256; ++ch) {
 	code = pdf_different_encoding_element(pdfont, ch, base_encoding);
 	if (code < 0)
 	    return code; /* Must not happen */
 	if (code)
 	    break;
     }
-    if (ch < 256)
-	pprintld1(s, "/Encoding %ld 0 R",
-		  (diff_id = pdf_obj_ref(pdev)));
-    else if (base_encoding > 0)
+    return ch;
+}
+
+/* Check for unknown encode (simple fonts only). */
+private bool
+pdf_simple_font_needs_ToUnicode(const pdf_font_resource_t *pdfont)
+{
+    int ch;
+    unsigned char mask = (pdfont->FontType == ft_encrypted || pdfont->FontType == ft_encrypted2 
+		? GS_C_PDF_GOOD_GLYPH_MASK : GS_C_PDF_GOOD_NON_SYMBOL_MASK);
+
+    if (pdfont->u.simple.Encoding == NULL)
+	return true; /* Bitmap Type 3 fonts have no pdfont->u.simple.Encoding . */
+    for (ch = 0; ch < 256; ++ch) {
+	pdf_encoding_element_t *pet = &pdfont->u.simple.Encoding[ch];
+	gs_glyph glyph = pet->glyph;
+
+	if (glyph == GS_NO_GLYPH)
+	    continue;
+	if (glyph < gs_c_min_std_encoding_glyph || glyph >= GS_MIN_CID_GLYPH) {
+	    if (pet->str.size == 0)
+		return true;
+	    glyph = gs_c_name_glyph(pet->str.data, pet->str.size);
+	    if (glyph == GS_NO_GLYPH)
+		return true;
+	}
+        glyph -= gs_c_min_std_encoding_glyph;
+        if( glyph > GS_C_PDF_MAX_GOOD_GLYPH ||
+           !(gs_c_pdf_glyph_type[glyph >> 2] & (mask << (( glyph & 3 )<<1) ))) 
+          return true;
+    }
+    return false;
+}
+
+/* Write Encoding differencrs. */
+int 
+pdf_write_encoding(gx_device_pdf *pdev, const pdf_font_resource_t *pdfont, long id, int ch)
+{
+    /* Note : this truncates extended glyph names to original names. */
+    stream *s;
+    gs_encoding_index_t base_encoding = pdfont->u.simple.BaseEncoding;
+    const int sl = strlen(gx_extendeg_glyph_name_separator);
+    int prev = 256, code;
+
+    pdf_open_separate(pdev, id);
+    s = pdev->strm;
+    stream_puts(s, "<</Type/Encoding");
+    if (base_encoding > 0)
+	pprints1(s, "/BaseEncoding/%s", encoding_names[base_encoding]);
+    stream_puts(s, "/Differences[");
+    for (; ch < 256; ++ch) {
+	code = pdf_different_encoding_element(pdfont, ch, base_encoding);
+	if (code < 0)
+	    return code; /* Must not happen */
+	if (code == 0 && pdfont->FontType == ft_user_defined) {
+	    /* PDF 1.4 spec Appendix H Note 42 says that
+	     * Acrobat 4 can't properly handle Base Encoding. 
+	     * Enforce writing differences against that.
+	     */
+	    if (pdfont->used[ch >> 3] & 0x80 >> (ch & 7))
+		if (pdfont->u.simple.Encoding[ch].str.size)
+		    code = 1;
+	}
+	if (code) {
+	    const byte *d = pdfont->u.simple.Encoding[ch].str.data;
+	    int i, l = pdfont->u.simple.Encoding[ch].str.size;
+
+	    for (i = 0; i + sl < l; i++)
+		if (!memcmp(d + i, gx_extendeg_glyph_name_separator, sl)) {
+		    l = i;
+		    break;
+		}
+	    if (ch != prev + 1)
+		pprintd1(s, "\n%d", ch);
+	    pdf_put_name(pdev, d, l);
+	    prev = ch;
+	}
+    }
+    stream_puts(s, "]>>\n");
+    pdf_end_separate(pdev);
+    return 0;
+}
+
+/* Write Encoding reference. */
+int
+pdf_write_encoding_ref(gx_device_pdf *pdev,
+	  const pdf_font_resource_t *pdfont, long id)
+{
+    stream *s = pdev->strm;
+
+    if (id != 0)
+	pprintld1(s, "/Encoding %ld 0 R", id);
+    else if (pdfont->u.simple.BaseEncoding > 0) {
+	gs_encoding_index_t base_encoding = pdfont->u.simple.BaseEncoding;
 	pprints1(s, "/Encoding/%s", encoding_names[base_encoding]);
+    }
+    return 0;
+}
+
+/* Write the Subtype and Encoding for a simple font. */
+private int
+pdf_write_simple_contents(gx_device_pdf *pdev,
+			  const pdf_font_resource_t *pdfont)
+{
+    stream *s = pdev->strm;
+    long diff_id = 0;
+    int ch = (pdfont->u.simple.Encoding ? 0 : 256);
+    int code = 0;
+
+    ch = pdf_different_encoding_index(pdfont, ch);
+    if (ch < 256)
+	diff_id = pdf_obj_ref(pdev);
+    code = pdf_write_encoding_ref(pdev, pdfont, diff_id);
+    if (code < 0)
+	return code;
     pprints1(s, "/Subtype/%s>>\n",
 	     (pdfont->FontType == ft_TrueType ? "TrueType" :
 	      pdfont->u.simple.s.type1.is_MM_instance ? "MMType1" : "Type1"));
     pdf_end_separate(pdev);
     if (diff_id) {
-	int prev = 256;
-
-	pdf_open_separate(pdev, diff_id);
-	s = pdev->strm;
-	stream_puts(s, "<</Type/Encoding");
-	if (base_encoding > 0)
-	    pprints1(s, "/BaseEncoding/%s", encoding_names[base_encoding]);
-	stream_puts(s, "/Differences[");
-	for (; ch < 256; ++ch) {
-	    code = pdf_different_encoding_element(pdfont, ch, base_encoding);
-	    if (code < 0)
-		return code; /* Must not happen */
-	    if (code) {
-		if (ch != prev + 1)
-		    pprintd1(s, "\n%d", ch);
-		pdf_put_name(pdev, pdfont->u.simple.Encoding[ch].str.data,
-			     pdfont->u.simple.Encoding[ch].str.size);
-		prev = ch;
-	    }
-	}
-	stream_puts(s, "]>>\n");
-	pdf_end_separate(pdev);
+	code = pdf_write_encoding(pdev, pdfont, diff_id, ch);
+	if (code < 0)
+	    return code;
     }
     return 0;
 }
 
+/*
+ * Write the W[2] entries for a CIDFont.  *pdfont is known to be a
+ * CIDFont (type 0 or 2).
+ */
+private bool
+pdf_compute_CIDFont_default_widths(const pdf_font_resource_t *pdfont, int wmode, int *pdw, int *pdv)
+{
+    psf_glyph_enum_t genum;
+    gs_glyph glyph;
+    ushort counts[1500]; /* Some CID fonts use vertical widths 1026 .*/
+    int dw_count = 0, i, dwi = 0, neg_count = 0, pos_count = 0;
+    double *w = (wmode ? pdfont->u.cidfont.Widths2 : pdfont->Widths);
+
+    /* We don't wont to scan for both negative and positive widths,
+     * to save the C stack space.
+     * Doubtly they both are used in same font.
+     * So just count positive and negative widths separately
+     * and use the corresponding sign.
+     * fixme : implement 2 hystograms.
+     */
+    psf_enumerate_bits_begin(&genum, NULL, 
+			     wmode ? pdfont->u.cidfont.used2 : pdfont->used, 
+			     pdfont->count, GLYPH_SPACE_INDEX);
+    memset(counts, 0, sizeof(counts));
+    while (!psf_enumerate_glyphs_next(&genum, &glyph)) {
+        int i = glyph - GS_MIN_CID_GLYPH;
+
+	if ( i < pdfont->count) { /* safety */
+	    int width = (int)(w[i] + 0.5);
+
+	    counts[min(any_abs(width), countof(counts) - 1)]++;
+	    if (width > 0)
+		pos_count++;
+	    else if (width < 0)
+		neg_count++;
+	}
+    }
+    for (i = 1; i < countof(counts); ++i)
+	if (counts[i] > dw_count)
+	    dwi = i, dw_count = counts[i];
+    *pdw = (neg_count > pos_count ? -dwi : dwi);
+    *pdv = 0;
+    if (wmode) {
+	psf_enumerate_glyphs_reset(&genum);
+	while (!psf_enumerate_glyphs_next(&genum, &glyph)) {
+	    int i = glyph - GS_MIN_CID_GLYPH;
+
+	    if ( i < pdfont->count) { /* safety */
+		int width = (int)(w[i] + 0.5);
+
+		if (min(any_abs(width), countof(counts) - 1) == any_abs(dwi)) {
+		    *pdv = (int)(pdfont->u.cidfont.v[i * 2 + 1] + 0.5);
+		    break;
+		}
+	    }
+	}
+    }
+    return (dw_count + counts[0] > 0);
+}
+
 /*
  * Write the [D]W[2] entries for a CIDFont.  *pdfont is known to be a
  * CIDFont (type 0 or 2).
  */
 private int
 pdf_write_CIDFont_widths(gx_device_pdf *pdev,
-			 const pdf_font_resource_t *pdfont)
+			 const pdf_font_resource_t *pdfont, int wmode)
 {
     /*
      * The values of the CIDFont width keys are as follows:
@@ -148,63 +300,82 @@ pdf_write_CIDFont_widths(gx_device_pdf *pdev,
      *   W = [{c [w ...] | cfirst clast w}*]
      *   DW2 = [vy w1y] (default [880 -1000])
      *   W2 = [{c [w1y vx vy ...] | cfirst clast w1y vx vy}*]
-     * Currently we only write DW and W.
      */
     stream *s = pdev->strm;
     psf_glyph_enum_t genum;
     gs_glyph glyph;
-    int dw = 0;
+    int dw = 0, dv = 0, prev = -2;
+    const char *Widths_key = (wmode ? "/W2" : "/W");
+    double *w = (wmode ? pdfont->u.cidfont.Widths2 : pdfont->Widths);
 
-    psf_enumerate_bits_begin(&genum, NULL, pdfont->used, pdfont->count,
-			     GLYPH_SPACE_INDEX);
-
-    /* Use the most common width as DW. */
-
-    {
-	ushort counts[1001];
-	int dw_count = 0, i;
-
-	memset(counts, 0, sizeof(counts));
-	while (!psf_enumerate_glyphs_next(&genum, &glyph)) {
-	    int width = (int)(pdfont->Widths[glyph - GS_MIN_CID_GLYPH] + 0.5);
-
-	    counts[min(width, countof(counts) - 1)]++;
-	}
-	for (i = 0; i < countof(counts); ++i)
-	    if (counts[i] > dw_count)
-		dw = i, dw_count = counts[i];
-	if (dw != 0)
+    /* Compute and write default width : */
+    if (pdf_compute_CIDFont_default_widths(pdfont, wmode, &dw, &dv)) {
+	if (wmode) {
+	    pprintd2(s, "/DW2 [%d %d]\n", dv, dw);
+	} else
 	    pprintd1(s, "/DW %d\n", dw);
     }
 
     /*
-     * Now write all widths different from DW.  Currently we make no
+     * Now write all widths different from the default one.  Currently we make no
      * attempt to optimize this: we write every width individually.
      */
-
-    psf_enumerate_glyphs_reset(&genum);
+    psf_enumerate_bits_begin(&genum, NULL, 
+			     wmode ? pdfont->u.cidfont.used2 : pdfont->used, 
+			     pdfont->count, GLYPH_SPACE_INDEX);
     {
-	int prev = -2;
-
 	while (!psf_enumerate_glyphs_next(&genum, &glyph)) {
 	    int cid = glyph - GS_MIN_CID_GLYPH;
-	    int width = (int)(pdfont->Widths[cid] + 0.5);
+	    int width = (int)(w[cid] + 0.5);
 
-	    if (cid == prev + 1)
-		pprintd1(s, "\n%d", width);
-	    else if (width == dw)
+#if 0 /* Must write zero widths - see test file of the bug Bug 687681.
+	 We don't enumerate unused glyphs here due to pdfont->used. */
+	    if (width == 0)
+		continue; /* Don't write for unused glyphs. */
+#else
+	    {	/* Check whether copied font really have this glyph.
+	           debugged with 401-01.ps, which uses undefined CIDs. */
+		gs_font_base *pfont = pdf_font_resource_font(pdfont, false);
+		gs_glyph_info_t info;
+
+		if (pdfont != NULL) {
+		    if (pfont->procs.glyph_info((gs_font *)pfont, glyph, NULL, 0, &info) < 0)
+			continue;
+		}
+	    }
+#endif
+	    if (cid == prev + 1) {
+		if (wmode) {
+		    int vx = (int)(pdfont->u.cidfont.v[cid * 2 + 0] + 0.5);
+		    int vy = (int)(pdfont->u.cidfont.v[cid * 2 + 1] + 0.5);
+
+		    pprintd3(s, "\n%d %d %d", width, vx, vy);
+		} else
+		    pprintd1(s, "\n%d", width);
+	    } else if (width == dw && 
+		    (!wmode || (int)(pdfont->u.cidfont.v[cid * 2 + 0] + 0.5) == 
+				(int)(pdfont->Widths[cid] / 2 + 0.5)) &&
+		    (!wmode || (int)(pdfont->u.cidfont.v[cid * 2 + 1] + 0.5) == dv))
 		continue;
 	    else {
 		if (prev >= 0)
 		    stream_puts(s, "]\n");
-		else
-		    stream_puts(s, "/W[");
-		pprintd2(s, "%d[%d", cid, width);
+		else {
+		    stream_puts(s, Widths_key);
+		    stream_puts(s, "[");
+		}
+		if (wmode) {
+		    int vx = (int)(pdfont->u.cidfont.v[cid * 2 + 0] + 0.5);
+		    int vy = (int)(pdfont->u.cidfont.v[cid * 2 + 1] + 0.5);
+
+		    pprintd4(s, "%d[%d %d %d", cid, width, vx, vy);
+		} else
+		    pprintd2(s, "%d[%d", cid, width);
 	    }
 	    prev = cid;
 	}
 	if (prev >= 0)
-	    stream_puts(s, "]]");
+	    stream_puts(s, "]]\n");
     }    
 
     return 0;
@@ -272,10 +443,22 @@ write_contents_cid_common(gx_device_pdf *pdev, pdf_font_resource_t *pdfont,
 {
     /* Write [D]W[2], CIDSystemInfo, and Subtype, and close the object. */
     stream *s = pdev->strm;
-    int code = pdf_write_CIDFont_widths(pdev, pdfont);
+    int code;
 
-    if (code < 0)
-	return code;
+    if (pdfont->Widths != 0) {
+	code = pdf_write_CIDFont_widths(pdev, pdfont, 0);
+	if (code < 0)
+	    return code;
+    } else {
+	/* With a vertical font, the viewer uses /DW 
+	   to determine glyph width to compute its v-vector. */
+	stream_puts(s, "/DW 0\n");
+    }
+    if (pdfont->u.cidfont.Widths2 != 0) {
+	code = pdf_write_CIDFont_widths(pdev, pdfont, 1);
+	if (code < 0)
+	    return code;
+    }
     if (pdfont->u.cidfont.CIDSystemInfo_id)
 	pprintld1(s, "/CIDSystemInfo %ld 0 R",
 		  pdfont->u.cidfont.CIDSystemInfo_id);
@@ -319,9 +502,12 @@ pdf_write_contents_cid2(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
 	pdf_data_writer_t writer;
 	int i;
 
+#if !PDFW_DELAYED_STREAMS
 	pdf_open_separate(pdev, map_id);
 	stream_puts(pdev->strm, "<<");
-	pdf_begin_data(pdev, &writer);
+#endif
+	pdf_begin_data_stream(pdev, &writer,
+	    DATA_STREAM_BINARY | DATA_STREAM_COMPRESS | DATA_STREAM_ENCRYPT, map_id);
 	for (i = 0; i < count; ++i) {
 	    uint gid = pdfont->u.cidfont.CIDToGIDMap[i];
 
@@ -341,6 +527,20 @@ pdf_write_font_resource(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
 {
     stream *s;
 
+    if (pdfont->cmap_ToUnicode != NULL && pdfont->res_ToUnicode == NULL)
+	if (((pdfont->FontType == ft_composite) && 
+		!gs_cmap_is_identity(pdfont->cmap_ToUnicode, -1)) ||
+	    ((pdfont->FontType == ft_encrypted || pdfont->FontType == ft_encrypted2 || 
+		pdfont->FontType == ft_TrueType || pdfont->FontType == ft_user_defined) && 
+		pdf_simple_font_needs_ToUnicode(pdfont))
+	   ) {
+	    pdf_resource_t *prcmap;
+	    int code = pdf_cmap_alloc(pdev, pdfont->cmap_ToUnicode, &prcmap, -1);
+
+	    if (code < 0)
+		return code;
+	    pdfont->res_ToUnicode = prcmap;
+	}
     pdf_open_separate(pdev, pdf_font_id(pdfont));
     s = pdev->strm;
     stream_puts(s, "<<");
@@ -351,19 +551,13 @@ pdf_write_font_resource(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
     if (pdfont->FontDescriptor)
 	pprintld1(s, "/FontDescriptor %ld 0 R",
 		  pdf_font_descriptor_id(pdfont->FontDescriptor));
-    if (pdfont->ToUnicode)
+    if (pdfont->res_ToUnicode)
 	pprintld1(s, "/ToUnicode %ld 0 R",
-		  pdf_resource_id((const pdf_resource_t *)pdfont->ToUnicode));
-    /*
-     * The PDF Reference says that the /Name entry is obsolescent and
-     * should no longer be used, but the Ghostscript PDF interpreter
-     * currently requires it.
-     */
-#if 0
-    stream_puts(s, "/Type/Font\n");
-#else
-    pprintld1(s, "/Type/Font/Name/R%ld\n", pdf_font_id(pdfont));
-#endif
+		  pdf_resource_id((const pdf_resource_t *)pdfont->res_ToUnicode));
+    if (pdev->CompatibilityLevel > 1.0)
+	stream_puts(s, "/Type/Font\n");
+    else
+	pprintld1(s, "/Type/Font/Name/R%ld\n", pdf_font_id(pdfont));
     return pdfont->write_contents(pdev, pdfont);
 }
 
@@ -380,7 +574,7 @@ write_font_resources(gx_device_pdf *pdev, pdf_resource_list_t *prlist)
     for (j = 0; j < NUM_RESOURCE_CHAINS; ++j)
 	for (pres = prlist->chains[j]; pres != 0; pres = pres->next) {
 	    pdf_font_resource_t *const pdfont = (pdf_font_resource_t *)pres;
-	    int code = pdf_compute_BaseFont(pdev, pdfont);
+	    int code = pdf_compute_BaseFont(pdev, pdfont, true);
 
 	    if (code < 0)
 		return code;
@@ -418,8 +612,9 @@ pdf_close_text_document(gx_device_pdf *pdev)
      * the descriptors.
      */
 
-    if ((code = finish_font_descriptors(pdev, pdf_finish_FontDescriptor)) < 0 ||
-	(code = write_font_resources(pdev, &pdev->resources[resourceCIDFont])) < 0 ||
+    if ((code = pdf_write_resource_objects(pdev, resourceCharProc)) < 0 ||
+ 	(code = finish_font_descriptors(pdev, pdf_finish_FontDescriptor)) < 0 ||
+  	(code = write_font_resources(pdev, &pdev->resources[resourceCIDFont])) < 0 ||
 	(code = write_font_resources(pdev, &pdev->resources[resourceFont])) < 0 ||
 	(code = finish_font_descriptors(pdev, pdf_write_FontDescriptor)) < 0
 	)
@@ -435,29 +630,104 @@ pdf_close_text_document(gx_device_pdf *pdev)
 /*
  * Write the CIDSystemInfo for a CIDFont or a CMap.
  */
-int
-pdf_write_cid_system_info(gx_device_pdf *pdev,
-			  const gs_cid_system_info_t *pcidsi)
+private int
+pdf_write_cid_system_info_to_stream(gx_device_pdf *pdev, stream *s,
+			  const gs_cid_system_info_t *pcidsi, gs_id object_id)
 {
-    stream *s = pdev->strm;
+    byte Registry[32], Ordering[32];
 
+    if (pcidsi->Registry.size > sizeof(Registry))
+	return_error(gs_error_limitcheck);
+    if (pcidsi->Ordering.size > sizeof(Ordering))
+	return_error(gs_error_limitcheck);
+    memcpy(Registry, pcidsi->Registry.data, pcidsi->Registry.size);
+    memcpy(Ordering, pcidsi->Ordering.data, pcidsi->Ordering.size);
+    if (pdev->KeyLength) {
+	stream_arcfour_state sarc4;
+	int code; 
+
+	code = pdf_encrypt_init(pdev, object_id, &sarc4);
+	if (code < 0)
+	    return code;
+	s_arcfour_process_buffer(&sarc4, Registry, pcidsi->Registry.size);
+	code = pdf_encrypt_init(pdev, object_id, &sarc4);
+	if (code < 0)
+	    return code;
+	s_arcfour_process_buffer(&sarc4, Ordering, pcidsi->Ordering.size);
+    }
     stream_puts(s, "<<\n/Registry");
-    s_write_ps_string(s, pcidsi->Registry.data, pcidsi->Registry.size,
-		      PRINT_HEX_NOT_OK);
+    s_write_ps_string(s, Registry, pcidsi->Registry.size, PRINT_HEX_NOT_OK);
     stream_puts(s, "\n/Ordering");
-    s_write_ps_string(s, pcidsi->Ordering.data, pcidsi->Ordering.size,
-		      PRINT_HEX_NOT_OK);
+    s_write_ps_string(s, Ordering, pcidsi->Ordering.size, PRINT_HEX_NOT_OK);
     pprintd1(s, "\n/Supplement %d\n>>\n", pcidsi->Supplement);
     return 0;
 }
 
+int
+pdf_write_cid_system_info(gx_device_pdf *pdev,
+			  const gs_cid_system_info_t *pcidsi, gs_id object_id)
+{
+    return pdf_write_cid_system_info_to_stream(pdev, pdev->strm, pcidsi, object_id);
+}
+
+
 /*
  * Write a CMap resource.  We pass the CMap object as well as the resource,
  * because we write CMaps when they are created.
  */
+#if PDFW_DELAYED_STREAMS
 int
 pdf_write_cmap(gx_device_pdf *pdev, const gs_cmap_t *pcmap,
-	       pdf_resource_t *pres /*CMap*/)
+	       pdf_resource_t **ppres /*CMap*/, int font_index_only)
+{
+    int code;
+    pdf_data_writer_t writer;
+
+    code = pdf_begin_data_stream(pdev, &writer,
+				 DATA_STREAM_NOT_BINARY | DATA_STREAM_ENCRYPT |
+				 (pdev->CompressFonts ? 
+				  DATA_STREAM_COMPRESS : 0), gs_no_id);
+    if (code < 0)
+	return code;
+    *ppres = writer.pres;
+    if (!pcmap->ToUnicode) {
+	byte buf[200];
+	cos_dict_t *pcd = (cos_dict_t *)writer.pres->object;
+	stream s;
+
+	code = cos_dict_put_c_key_int(pcd, "/WMode", pcmap->WMode);
+	if (code < 0)
+	    return code;
+	buf[0] = '/';
+	memcpy(buf + 1, pcmap->CMapName.data, pcmap->CMapName.size);
+	code = cos_dict_put_c_key_string(pcd, "/CMapName", 
+			buf, pcmap->CMapName.size + 1);
+	if (code < 0)
+	    return code;
+	s_init(&s, pdev->memory);
+	swrite_string(&s, buf, sizeof(buf));
+	code = pdf_write_cid_system_info_to_stream(pdev, &s, pcmap->CIDSystemInfo, 
+		writer.pres->object->id);
+	if (code < 0)
+	    return code;
+	code = cos_dict_put_c_key_string(pcd, "/CIDSystemInfo", 
+			buf, stell(&s));
+	if (code < 0)
+	    return code;
+    }
+    code = psf_write_cmap(pdev->memory, writer.binary.strm, pcmap,
+			  pdf_put_name_chars_proc(pdev), NULL, font_index_only);
+    if (code < 0)
+	return code;
+    code = pdf_end_data(&writer);
+    if (code < 0)
+	return code;
+    return code;
+}
+#else
+int
+pdf_write_cmap(gx_device_pdf *pdev, const gs_cmap_t *pcmap,
+	       pdf_resource_t *pres /*CMap*/, int font_index_only)
 {
     pdf_data_writer_t writer;
     stream *s;
@@ -467,20 +737,23 @@ pdf_write_cmap(gx_device_pdf *pdev, const gs_cmap_t *pcmap,
 	return 0;
     pdf_open_separate(pdev, pres->object->id);
     s = pdev->strm;
-    pprintd1(s, "<</WMode %d/CMapName", pcmap->WMode);
-    pdf_put_name(pdev, pcmap->CMapName.data, pcmap->CMapName.size);
-    stream_puts(s, "/CIDSystemInfo");
-    code = pdf_write_cid_system_info(pdev, pcmap->CIDSystemInfo);
-    if (code < 0)
-	return code;
+    stream_puts(s, "<<");
+    if (!pcmap->ToUnicode) {
+	pprintd1(s, "/WMode %d/CMapName", pcmap->WMode);
+	pdf_put_name(pdev, pcmap->CMapName.data, pcmap->CMapName.size);
+	stream_puts(s, "/CIDSystemInfo");
+	code = pdf_write_cid_system_info(pdev, pcmap->CIDSystemInfo, pres->object->id);
+	if (code < 0)
+	    return code;
+    }
     code = pdf_begin_data_stream(pdev, &writer,
-				 DATA_STREAM_NOT_BINARY |
-				 (pdev->CompressFonts ?
-				  DATA_STREAM_COMPRESS : 0));
+				 DATA_STREAM_NOT_BINARY | DATA_STREAM_ENCRYPT |
+				 (pdev->CompressFonts ? 
+				  DATA_STREAM_COMPRESS : 0), pres->object->id);
     if (code < 0)
 	return code;
     code = psf_write_cmap(writer.binary.strm, pcmap,
-			  pdf_put_name_chars_proc(pdev), NULL);
+			  pdf_put_name_chars_proc(pdev), NULL, font_index_only);
     if (code < 0)
 	return code;
     code = pdf_end_data(&writer);
@@ -489,3 +762,4 @@ pdf_write_cmap(gx_device_pdf *pdev, const gs_cmap_t *pcmap,
     pres->object->written = true;
     return 0;
 }
+#endif
