@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevplnx.c,v 1.5 2002/08/22 07:12:28 henrys Exp $*/
+/* $Id: gdevplnx.c 5215 2004-08-04 19:36:13Z stefan $*/
 /* Plane extraction device */
 #include "gx.h"
 #include "gserrors.h"
@@ -158,7 +158,7 @@ reduce_drawing_color(gx_device_color *ppdc, gx_device_plane_extract *edev,
     if (gx_dc_is_pure(pdevc)) {
 	gx_color_index pixel = COLOR_PIXEL(edev, gx_dc_pure_color(pdevc));
 
-	color_set_pure(ppdc, pixel);
+	set_nonclient_dev_color(ppdc, pixel);
 	reduced = REDUCE_PURE(edev, pixel);
     } else if (gx_dc_is_binary_halftone(pdevc)) {
 	gx_color_index pixel0 =
@@ -167,7 +167,7 @@ reduce_drawing_color(gx_device_color *ppdc, gx_device_plane_extract *edev,
 	    TRANS_COLOR_PIXEL(edev, gx_dc_binary_color1(pdevc));
 
 	if (pixel0 == pixel1) {
-	    color_set_pure(ppdc, pixel0);
+	    set_nonclient_dev_color(ppdc, pixel0);
 	    reduced = REDUCE_PURE(edev, pixel0);
 	} else {
 	    *ppdc = *pdevc;
@@ -395,6 +395,7 @@ plane_device_init(gx_device_plane_extract *edev, gx_device *target,
     gx_device_init((gx_device *)edev,
 		   (const gx_device *)&gs_plane_extract_device,
 		   edev->memory, true);
+    check_device_separable((gx_device *)edev);
     gx_device_forward_fill_in_procs((gx_device_forward *)edev);
     gx_device_set_target((gx_device_forward *)edev, target);
     gx_device_copy_params((gx_device *)edev, target);
@@ -731,9 +732,9 @@ plane_strip_copy_rop(gx_device *dev,
     long sbuf[COPY_ROP_SOURCE_BUF_SIZE / sizeof(long)];
     long tbuf[COPY_ROP_TEXTURE_BUF_SIZE / sizeof(long)];
     const byte *plane_source;
-    uint plane_raster;
+    uint plane_raster = 0xbaadf00d; /* Initialize against indeterminizm. */
     gx_strip_bitmap plane_texture;
-    const gx_strip_bitmap *plane_textures;
+    const gx_strip_bitmap *plane_textures = NULL;
     int code;
 
     /* We should do better than this on transparency.... */
@@ -891,8 +892,15 @@ plane_cmap_rgb_alpha(frac r, frac g, frac b, frac alpha, gx_device_color * pdc,
 				(gx_device *)edev, select);
     reduce_drawing_color(pdc, edev, &dcolor, &lop);
 }
+private bool
+plane_cmap_is_halftoned(const gs_imager_state *pis_image, gx_device *dev)
+{
+    return false;
+}
+
 private const gx_color_map_procs plane_color_map_procs = {
-    plane_cmap_gray, plane_cmap_rgb, plane_cmap_cmyk, plane_cmap_rgb_alpha
+    plane_cmap_gray, plane_cmap_rgb, plane_cmap_cmyk, plane_cmap_rgb_alpha,
+    NULL, NULL, plane_cmap_is_halftoned
 };
 private const gx_color_map_procs *
 plane_get_cmap_procs(const gs_imager_state *pis, const gx_device *dev)
@@ -959,7 +967,7 @@ plane_begin_typed_image(gx_device * dev,
 	 * The drawing color won't be used, but if RasterOp is involved,
 	 * it may still be accessed in some anomalous cases.
 	 */
-	color_set_pure(&dcolor, (gx_color_index)0);
+	set_nonclient_dev_color(&dcolor, (gx_color_index)0);
     }
     info = gs_alloc_struct(memory, plane_image_enum_t, &st_plane_image_enum,
 			   "plane_image_begin_typed(info)");
@@ -977,7 +985,7 @@ plane_begin_typed_image(gx_device * dev,
     *((gx_image_enum_common_t *)info) = *info->info;
     info->procs = &plane_image_enum_procs;
     info->dev = (gx_device *)edev;
-    info->id = gs_next_ids(1);
+    info->id = gs_next_ids(memory, 1);
     info->memory = memory;
     info->pis = pis;
     info->pis_image = pis_image;
