@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpm.c,v 1.8 2002/10/29 09:51:33 ghostgum Exp $ */
+/* $Id: gdevpm.c 5359 2004-09-20 22:14:59Z dan $ */
 /*
  * OS/2 Presentation manager driver
  *
@@ -69,8 +69,8 @@
 
 /* Initial values for width and height */
 #define INITIAL_RESOLUTION 96
-#define INITIAL_WIDTH (INITIAL_RESOLUTION * 85 / 10 + 1)
-#define INITIAL_HEIGHT (INITIAL_RESOLUTION * 11 + 1)
+#define INITIAL_WIDTH ((INITIAL_RESOLUTION * 85 + 5) / 10)
+#define INITIAL_HEIGHT ((INITIAL_RESOLUTION * 110 + 5) / 10)
 
 /* A macro for casting the device argument */
 #define pmdev ((gx_device_pm *)dev)
@@ -617,9 +617,6 @@ pm_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 		return (gx_no_color_index);	/* not found - dither instead */
 	    }
 	case 4:
-	    if ((r == g) && (g == b) && (r >= gx_max_color_value / 3 * 2 - 1)
-		&& (r < gx_max_color_value / 4 * 3))
-		return ((gx_color_index) 8);	/* light gray */
 	    return pc_4bit_map_rgb_color(dev, cv);
     }
     return (gx_default_map_rgb_color(dev, cv));
@@ -652,10 +649,7 @@ pm_map_color_rgb(gx_device * dev, gx_color_index color,
 	    }
 	    break;
 	case 4:
-	    if (color == 8)	/* VGA light gray */
-		prgb[0] = prgb[1] = prgb[2] = (gx_max_color_value / 4 * 3);
-	    else
-		pc_4bit_map_color_rgb(dev, color, prgb);
+	    pc_4bit_map_color_rgb(dev, color, prgb);
 	    break;
 	default:
 	    prgb[0] = prgb[1] = prgb[2] =
@@ -1231,7 +1225,7 @@ pm_set_bits_per_pixel(gx_device_pm * pmdev, int bpp)
 	    pmdev->nColors = 64;
 	    break;
 	case 4:
-	    set_color_info(&dci, 3, 4, 3, 2);
+	    set_color_info(&dci, 3, 4, 1, 1);
 	    set_rgb_color_procs(pdev, pm_map_rgb_color, pm_map_color_rgb);
 	    pmdev->nColors = 16;
 	    break;
