@@ -17,7 +17,7 @@
   Author: Raph Levien <raph@artofcode.com>
 */
 
-/* $Id: gdevpnga.c,v 1.6.4.1 2004/01/20 19:16:22 giles Exp $ */
+/* $Id: gdevpnga.c 5046 2004-05-26 04:11:01Z dan $ */
 /* Test driver for PDF 1.4 transparency stuff */
 
 #include "gdevprn.h"
@@ -690,29 +690,6 @@ pnga_output_page(gx_device *dev, int num_copies, int flush)
     info_ptr->bit_depth = 8;
     info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA;
 
-    /* set the palette if there is one */
-    if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE) {
-	int i;
-	int num_colors = 1 << depth;
-	gx_color_value rgb[3];
-
-	info_ptr->palette =
-	    (void *)gs_alloc_bytes(mem, 256 * sizeof(png_color),
-				   "png palette");
-	if (info_ptr->palette == 0) {
-	    code = gs_note_error(gs_error_VMerror);
-	    goto done;
-	}
-	info_ptr->num_palette = num_colors;
-	info_ptr->valid |= PNG_INFO_PLTE;
-	for (i = 0; i < num_colors; i++) {
-	    (*dev_proc(dev, map_color_rgb)) ((gx_device *) dev,
-					      (gx_color_index) i, rgb);
-	    info_ptr->palette[i].red = gx_color_value_to_byte(rgb[0]);
-	    info_ptr->palette[i].green = gx_color_value_to_byte(rgb[1]);
-	    info_ptr->palette[i].blue = gx_color_value_to_byte(rgb[2]);
-	}
-    }
     /* add comment */
     sprintf(software_text, "%s %d.%02d", gs_product,
 	    (int)(gs_revision / 100), (int)(gs_revision % 100));
@@ -747,9 +724,6 @@ pnga_output_page(gx_device *dev, int num_copies, int flush)
     /* write the rest of the file */
     png_write_end(png_ptr, info_ptr);
 
-    /* if you alloced the palette, free it here */
-    gs_free_object(mem, info_ptr->palette, "png palette");
-
   done:
     /* free the structures */
     png_destroy_write_struct(&png_ptr, &info_ptr);
@@ -790,6 +764,7 @@ pnga_get_marking_device(gx_device *dev, const gs_imager_state *pis)
     if (code < 0)
 	return NULL;
 
+    check_device_separable((gx_device *)mdev);
     gx_device_fill_in_procs((gx_device *)mdev);
     mdev->pnga_dev = pdev;
     mdev->opacity = pis->opacity.alpha;
