@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevprn.c,v 1.11 2002/08/22 07:12:28 henrys Exp $ */
+/* $Id: gdevprn.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Generic printer driver support */
 #include "ctype_.h"
 #include "gdevprn.h"
@@ -141,7 +141,7 @@ open_c:
     clist_init_params(pclist_dev, base, space, pdev,
 		      ppdev->printer_procs.buf_procs,
 		      space_params->band, ppdev->is_async_renderer,
-		      (ppdev->bandlist_memory == 0 ? &gs_memory_default :
+		      (ppdev->bandlist_memory == 0 ? pdev->memory->non_gc_memory:
 		       ppdev->bandlist_memory),
 		      ppdev->free_up_bandlist_memory,
 		      ppdev->clist_disable_mask);
@@ -220,13 +220,14 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
     gx_device_memory * const pmemdev = (gx_device_memory *)pdev;
     byte *the_memory = 0;
     gdev_prn_space_params save_params;
-    int save_width, save_height;
-    bool is_command_list;
-    bool save_is_command_list;
+    int save_width = 0x0badf00d; /* Quiet compiler */
+    int save_height = 0x0badf00d; /* Quiet compiler */
+    bool is_command_list = false; /* Quiet compiler */
+    bool save_is_command_list = false; /* Quiet compiler */
     int ecode = 0;
     int pass;
     gs_memory_t *buffer_memory =
-	(ppdev->buffer_memory == 0 ? &gs_memory_default :
+	(ppdev->buffer_memory == 0 ? pdev->memory->non_gc_memory :
 	 ppdev->buffer_memory);
 
     /* If reallocate, find allocated memory & tear down buffer device */
@@ -306,7 +307,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 		the_memory = base;
 	}
 	if (!is_command_list && pass == 1 && PRN_MIN_MEMORY_LEFT != 0
-	    && buffer_memory == &gs_memory_default) {
+	    && buffer_memory == pdev->memory->non_gc_memory) {
 	    /* before using full memory buffer, ensure enough working mem left */
 	    byte * left = gs_alloc_bytes( buffer_memory,
 					  PRN_MIN_MEMORY_LEFT, "printer mem left");
@@ -389,6 +390,12 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	COPY_PROC(get_color_comp_index);
 	COPY_PROC(encode_color);
 	COPY_PROC(decode_color);
+	COPY_PROC(begin_image);
+	COPY_PROC(text_begin);
+	COPY_PROC(fill_path);
+	COPY_PROC(stroke_path);
+	COPY_PROC(fill_rectangle_hl_color);
+	COPY_PROC(update_spot_equivalent_colors);
 #undef COPY_PROC
 	/* If using a command list, already opened the device. */
 	if (is_command_list)
@@ -426,7 +433,7 @@ gdev_prn_free_memory(gx_device *pdev)
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
     byte *the_memory = 0;
     gs_memory_t *buffer_memory =
-	(ppdev->buffer_memory == 0 ? &gs_memory_default :
+	(ppdev->buffer_memory == 0 ? pdev->memory->non_gc_memory :
 	 ppdev->buffer_memory);
 
     gdev_prn_tear_down(pdev, &the_memory);
@@ -982,6 +989,8 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
     if (target == (gx_device *)mdev) {
 	/* The following is a special hack for setting up printer devices. */
 	assign_dev_procs(mdev, mdproto);
+        check_device_separable((gx_device *)mdev);
+	gx_device_fill_in_procs((gx_device *)mdev);
     } else
 	gs_make_mem_device(mdev, mdproto, mem, (for_band ? 1 : 0),
 			   (target == (gx_device *)mdev ? NULL : target));
