@@ -1,4 +1,4 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1997, 2000-2004 artofcode LLC. All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,10 +14,11 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevps.c,v 1.31 2002/10/24 20:28:43 alexcher Exp $ */
+/* $Id: gdevps.c 5408 2004-10-07 05:18:34Z ray $ */
 /* PostScript-writing driver */
 #include "math_.h"
 #include "memory_.h"
+#include "string_.h"
 #include "time_.h"
 #include "gx.h"
 #include "gserrors.h"
@@ -40,6 +41,8 @@
 /* Current ProcSet version */
 #define PSWRITE_PROCSET_VERSION 1
 
+/* Guaranteed size of operand stack accoeding to PLRM */
+#define MAXOPSTACK 500
 
 private int psw_open_printer(gx_device * dev);
 private int psw_close_printer(gx_device * dev);
@@ -176,7 +179,10 @@ const gx_device_pswrite gs_epswrite_device = {
 /* Vector device implementation */
 private int
     psw_beginpage(gx_device_vector * vdev),
-    psw_setcolors(gx_device_vector * vdev, const gx_drawing_color * pdc),
+    psw_can_handle_hl_color(gx_device_vector * vdev, const gs_imager_state * pis, 
+                  const gx_drawing_color * pdc),
+    psw_setcolors(gx_device_vector * vdev, const gs_imager_state * pis, 
+                  const gx_drawing_color * pdc),
     psw_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1,
 	       fixed y1, gx_path_type_t type),
     psw_beginpath(gx_device_vector * vdev, gx_path_type_t type),
@@ -202,6 +208,7 @@ private const gx_device_vector_procs psw_vector_procs = {
     psdf_setflat,
     psdf_setlogop,
 	/* Other state */
+    psw_can_handle_hl_color,
     psw_setcolors,		/* fill & stroke colors are the same */
     psw_setcolors,
 	/* Paths */
@@ -252,7 +259,7 @@ private const char *const psw_procset[] = {
     "/q/gsave #/Q/grestore #/rf{re fill}!",
     "/Y{P clip newpath}!/Y*{P eoclip newpath}!/rY{re Y}!",
 	/* <w> <h> <name> <data> <?> |= <w> <h> <data> */
-    "/|={pop exch 4 1 roll 3 array astore cvx exch 1 index def exec}!",
+    "/|={pop exch 4 1 roll 1 array astore cvx 3 array astore cvx exch 1 index def exec}!",
 	/* <w> <h> <name> <length> <src> | <w> <h> <data> */
     "/|{exch string readstring |=}!",
 	/* <w> <?> <name> (<length>|) + <w> <?> <name> <length> */
@@ -326,24 +333,32 @@ private const char *const psw_2_procset[] = {
 private int
 psw_begin_file(gx_device_pswrite *pdev, const gs_rect *pbbox)
 {
+    int code;
     FILE *f = pdev->file;
+    char const * const *p1;
+    char const * const *p2;
 
-    psw_begin_file_header(f, (gx_device *)pdev, pbbox,
-			  &pdev->pswrite_common,
-			  pdev->params.ASCII85EncodePages);
-    psw_print_lines(f, psw_procset);
     if (pdev->pswrite_common.LanguageLevel < 1.5) {
-	psw_print_lines(f, psw_1_x_procset);
-	psw_print_lines(f, psw_1_procset);
+	p1 = psw_1_x_procset;
+	p2 = psw_1_procset;
     } else if (pdev->pswrite_common.LanguageLevel > 1.5) {
-	psw_print_lines(f, psw_1_5_procset);
-	psw_print_lines(f, psw_2_procset);
+	p1 = psw_1_5_procset;
+	p2 = psw_2_procset;
     } else {
-	psw_print_lines(f, psw_1_x_procset);
-	psw_print_lines(f, psw_1_5_procset);
+	p1 = psw_1_x_procset;
+	p2 = psw_1_5_procset;
     }
-    psw_end_file_header(f);
-    fflush(f);
+
+    if ((code = psw_begin_file_header(f, (gx_device *)pdev, pbbox,
+			  &pdev->pswrite_common,
+			  pdev->params.ASCII85EncodePages)) < 0 ||
+        (code = psw_print_lines(f, psw_procset)) < 0 ||
+        (code = psw_print_lines(f, p1)) < 0 ||
+        (code = psw_print_lines(f, p2)) < 0 ||
+        (code = psw_end_file_header(f)) < 0)
+        return code;
+    if(fflush(f) == EOF)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -428,17 +443,20 @@ psw_image_stream_setup(gx_device_pswrite * pdev, bool binary_ok)
 }
 
 /* Clean up after writing an image. */
-private void
+private int
 psw_image_cleanup(gx_device_pswrite * pdev)
 {
+    int code = 0;
+    
     if (pdev->image_stream != 0) {
-	psdf_end_binary(pdev->image_writer);
+	code = psdf_end_binary(pdev->image_writer);
 	memset(pdev->image_writer, 0, sizeof(*pdev->image_writer));
     }
+    return code;
 }
 
 /* Write data for an image.  Assumes width > 0, height > 0. */
-private void
+private int
 psw_put_bits(stream * s, const byte * data, int data_x_bit, uint raster,
 	     uint width_bits, int height)
 {
@@ -446,7 +464,7 @@ psw_put_bits(stream * s, const byte * data, int data_x_bit, uint raster,
     int shift = data_x_bit & 7;
     int y;
 
-    for (y = 0; y < height; ++y, row += raster)
+    for (y = 0; y < height; ++y, row += raster) {
 	if (shift == 0)
 	    stream_write(s, row, (width_bits + 7) >> 3);
 	else {
@@ -459,15 +477,23 @@ psw_put_bits(stream * s, const byte * data, int data_x_bit, uint raster,
 	    if (wleft > 0)
 		stream_putc(s, (byte)((*src << shift) & (byte)(0xff00 >> wleft)));
 	}
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
+    }
+    return 0;
 }
 private int
 psw_put_image_bits(gx_device_pswrite *pdev, const char *op,
 		   const byte * data, int data_x, uint raster,
 		   int width, int height, int depth)
 {
+    int code;
+    
     pprints1(pdev->strm, "%s\n", op);
-    psw_put_bits(pdev->image_stream, data, data_x * depth, raster,
+    code = psw_put_bits(pdev->image_stream, data, data_x * depth, raster,
 		 width * depth, height);
+    if (code < 0)
+        return code;
     psw_image_cleanup(pdev);
     return 0;
 }
@@ -505,6 +531,8 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	sprintf(str, "%d%c", index / 26, index % 26 + 'A');
 	pprintd2(s, "%d %d ", x, y);
 	pprints2(s, "%s %s\n", str, imagestr);
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
 	return 0;
     }
     pprintd4(s, "%d %d %d %d ", x, y, width, height);
@@ -537,6 +565,8 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	op = cached[encode];
 	sprintf(endstr, "\n%s\n", imagestr);
     }
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     /*
      * In principle, we should put %%BeginData: / %%EndData around all data
      * sections.  However, as long as the data are ASCII (not binary), they
@@ -560,6 +590,7 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	 */
 	stream poss;
 
+	s_init(s, pdev->memory);
 	swrite_position_only(&poss);
 	pdev->strm = &poss;
 	code = psw_put_image(pdev, op, encode, data, data_x, raster,
@@ -575,6 +606,8 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	stream_puts(s, "\n%%EndData");
     }
     stream_puts(s, endstr);
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -624,6 +657,7 @@ psw_is_separate_pages(gx_device_vector *const vdev)
     const char *fmt;
     gs_parsed_file_name_t parsed;
     int code = gx_parse_output_file_name(&parsed, &fmt, vdev->fname, strlen(vdev->fname));
+    
     return (code >= 0 && fmt != 0);
 }
 
@@ -640,23 +674,40 @@ psw_beginpage(gx_device_vector * vdev)
     if (code < 0)
 	 return code;
 
-    if (pdev->first_page)
-	psw_begin_file(pdev, NULL);
+    if (pdev->first_page) {
+	code = psw_begin_file(pdev, NULL);
+        if (code < 0)
+	     return code;
+    }
+
+    code = psw_write_page_header(s, (gx_device *)vdev, &pdev->pswrite_common, true, 
+                          (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount + 1),
+                          image_cache_size);
+    if (code < 0)
+	 return code;
 
-    psw_write_page_header(s, (gx_device *)vdev, &pdev->pswrite_common, true, 
-                          (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount + 1));
     pdev->page_fill.color = gx_no_color_index;
     return 0;
 }
 
+
 private int
-psw_setcolors(gx_device_vector * vdev, const gx_drawing_color * pdc)
+psw_can_handle_hl_color(gx_device_vector * vdev, const gs_imager_state * pis1, 
+              const gx_drawing_color * pdc)
 {
+    return false; /* High level color is not implemented yet. */
+}
+
+private int
+psw_setcolors(gx_device_vector * vdev, const gs_imager_state * pis1, 
+              const gx_drawing_color * pdc)
+{
+    const gs_imager_state * pis = NULL; /* High level color is not implemented yet. */
     if (!gx_dc_is_pure(pdc))
 	return_error(gs_error_rangecheck);
     /* PostScript only keeps track of a single color. */
-    vdev->fill_color = *pdc;
-    vdev->stroke_color = *pdc;
+    gx_hld_save_color(pis, pdc, &vdev->saved_fill_color);
+    gx_hld_save_color(pis, pdc, &vdev->saved_stroke_color);
     {
 	stream *s = gdev_vector_stream(vdev);
 	gx_color_index color = gx_dc_pure_color(pdc);
@@ -677,6 +728,8 @@ psw_setcolors(gx_device_vector * vdev, const gx_drawing_color * pdc)
 	    pprintd2(s, "%d %d r5\n", g, b);
 	else
 	    pprintd3(s, "%d %d %d rG\n", r, g, b);
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -691,6 +744,8 @@ psw_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1, fixed y1,
     pprintg4(gdev_vector_stream(vdev), "%g %g %g %g rf\n",
 	     fixed2float(x0), fixed2float(y0),
 	     fixed2float(x1 - x0), fixed2float(y1 - y0));
+    if ((gdev_vector_stream(vdev))->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -736,6 +791,8 @@ psw_beginpath(gx_device_vector * vdev, gx_path_type_t type)
 
 	stream_puts(s, "Q q\n");
 	gdev_vector_reset(vdev);
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -759,6 +816,8 @@ psw_moveto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
     print_coord2(s, x, y, NULL);
     pdev->path_state.num_points = 1;
     pdev->path_state.move = 1;
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -776,7 +835,12 @@ psw_lineto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 	stream *s = gdev_vector_stream(vdev);
 	gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
 
-	if (pdev->path_state.num_points > 0 &&
+	if (pdev->path_state.num_points > MAXOPSTACK / 2 - 10) {
+ 	    stream_puts(s, (pdev->path_state.move ? "P\n" : "p\n"));
+            pdev->path_state.num_points = 0;
+            pdev->path_state.move = 0;
+        }
+        else if (pdev->path_state.num_points > 0 &&
 	    !(pdev->path_state.num_points & 7)
 	    )
 	    stream_putc(s, '\n');	/* limit line length for DSC compliance */
@@ -791,6 +855,8 @@ psw_lineto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 	pdev->path_state.dprev[1] = pdev->path_state.dprev[0];
 	pdev->path_state.dprev[0].x = dx;
 	pdev->path_state.dprev[0].y = dy;
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -823,6 +889,8 @@ psw_curveto(gx_device_vector * vdev, floatp x0, floatp y0,
     }
     pdev->path_state.num_points = 0;
     pdev->path_state.move = 0;
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -837,6 +905,8 @@ psw_closepath(gx_device_vector * vdev, floatp x0, floatp y0,
 	   "H\n" : "h\n"));
     pdev->path_state.num_points = 0;
     pdev->path_state.move = 0;
+    if ((gdev_vector_stream(vdev))->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -863,6 +933,8 @@ psw_endpath(gx_device_vector * vdev, gx_path_type_t type)
     }
     if (type & gx_path_type_clip)
 	pprints1(s, "Y%s\n", star);
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -907,6 +979,7 @@ psw_open(gx_device * dev)
 private int
 psw_output_page(gx_device * dev, int num_copies, int flush)
 {
+    int code;
     gx_device_vector *const vdev = (gx_device_vector *)dev;
     gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
     stream *s = gdev_vector_stream(vdev);
@@ -914,7 +987,9 @@ psw_output_page(gx_device * dev, int num_copies, int flush)
     /* Check for a legitimate empty page. */
     CHECK_BEGIN_PAGE(pdev);
     sflush(s);			/* sync stream and file */
-    psw_write_page_trailer(vdev->file, num_copies, flush);
+    code = psw_write_page_trailer(vdev->file, num_copies, flush);
+    if(code < 0)
+        return code;
     vdev->in_page = false;
     pdev->first_page = false;
     gdev_vector_reset(vdev);
@@ -924,7 +999,7 @@ psw_output_page(gx_device * dev, int num_copies, int flush)
 
     dev->PageCount ++;
     if (psw_is_separate_pages(vdev)) {
-	int code = psw_close_printer(dev);
+	code = psw_close_printer(dev);
 
 	if (code < 0) 
 	    return code;
@@ -1083,6 +1158,7 @@ psw_open_printer(gx_device * dev)
 private int 
 psw_close_printer(gx_device * dev)
 {
+    int code;
     gx_device_vector *const vdev = (gx_device_vector *)dev;
     gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
     FILE *f = vdev->file;
@@ -1091,7 +1167,9 @@ psw_close_printer(gx_device * dev)
     gx_device_bbox_bbox(vdev->bbox_device, &bbox);
     if (pdev->first_page & !vdev->in_page) {
 	/* Nothing has been written.  Write the file header now. */
-	psw_begin_file(pdev, &bbox);
+	code = psw_begin_file(pdev, &bbox);
+        if (code < 0)
+            return code;
     } else {
 	/* If there is an incomplete page, complete it now. */
 	if (vdev->in_page) { 
@@ -1100,14 +1178,18 @@ psw_close_printer(gx_device * dev)
 	     */
 
 	    stream *s = vdev->strm;
-	    psw_write_page_trailer(vdev->file, 1, 1);
+	    code = psw_write_page_trailer(vdev->file, 1, 1);
+            if(code < 0)
+                return code;
 	    sflush(s);
 
 	    dev->PageCount++;
 	}
     }
-    psw_end_file(f, dev, &pdev->pswrite_common, &bbox, 
+    code = psw_end_file(f, dev, &pdev->pswrite_common, &bbox, 
                  (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount));
+    if(code < 0)
+        return code;
 
     return gdev_vector_close_file(vdev);
 }
@@ -1133,9 +1215,9 @@ psw_copy_mono(gx_device * dev, const byte * data,
 	((gx_device *) vdev->bbox_device, data, data_x, raster, id,
 	 x, y, w, h, zero, one);
     if (one == gx_no_color_index) {
-	color_set_pure(&color, zero);
+	set_nonclient_dev_color(&color, zero);
 	code = gdev_vector_update_fill_color((gx_device_vector *) pdev,
-					     &color);
+					     NULL, &color);
 	op = "If";
     } else if (zero == vdev->black && one == vdev->white)
 	op = "1 I";
@@ -1145,9 +1227,9 @@ psw_copy_mono(gx_device * dev, const byte * data,
 	    if (code < 0)
 		return code;
 	}
-	color_set_pure(&color, one);
+	set_nonclient_dev_color(&color, one);
 	code = gdev_vector_update_fill_color((gx_device_vector *) pdev,
-					     &color);
+					     NULL, &color);
 	op = ",";
     }
     if (code < 0)
@@ -1248,7 +1330,9 @@ psw_stroke_path(gx_device * dev, const gs_imager_state * pis,
 		psw_put_matrix(s, &mat);
 		stream_puts(s, "concat\n");
 	    }
-	}
+            if (s->end_status == ERRC)
+                return_error(gs_error_ioerror);
+        }
 	code = gdev_vector_dopath(vdev, ppath, gx_path_type_stroke,
 				  (set_ctm ? &mat : (const gs_matrix *)0));
 	if (code < 0)
@@ -1284,7 +1368,7 @@ psw_fill_mask(gx_device * dev,
      */
     if (depth > 1 ||
 	gdev_vector_update_clip_path(vdev, pcpath) < 0 ||
-	gdev_vector_update_fill_color(vdev, pdcolor) < 0 ||
+	gdev_vector_update_fill_color(vdev, NULL, pdcolor) < 0 ||
 	gdev_vector_update_log_op(vdev, lop) < 0
 	)
 	return gx_default_fill_mask(dev, data, data_x, raster, id,
@@ -1319,7 +1403,7 @@ psw_begin_image(gx_device * dev,
     gdev_vector_image_enum_t *pie;
     const gs_color_space *pcs = pim->ColorSpace;
     const gs_color_space *pbcs = pcs;
-    const char *base_name;
+    const char *base_name = NULL;
     gs_color_space_index index;
     int num_components;
     bool binary = pdev->binary_ok;
@@ -1504,6 +1588,12 @@ psw_begin_image(gx_device * dev,
 	}
 	stream_putc(s, '\n');
 	pprints1((bs ? bs : s), "%s\n", op);
+        if (s->end_status == ERRC) {
+            gs_free_object(mem, bs, "psw_begin_image(buffer stream)");
+            gs_free_object(mem, buffer, "psw_begin_image(buffer)");
+            gs_free_object(mem, pie, "psw_begin_image");
+            return_error(gs_error_ioerror);
+        }
     }
     *pinfo = (gx_image_enum_common_t *) pie;
     return 0;
@@ -1535,6 +1625,8 @@ psw_image_plane_data(gx_image_enum_common_t * info,
 		     planes[pi].data_x * info->plane_depths[pi],
 		     planes[pi].raster, pie->bits_per_row,
 		     *rows_used);
+        if (pdev->image_stream->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     pie->y += *rows_used;
     return code;
@@ -1577,6 +1669,8 @@ psw_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 	    gs_free_object(mem, buffer, "psw_image_end_image(buffer)");
 	}
 	stream_puts(s, "\nQ\n");
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return code;
 }
