@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpsdi.c,v 1.25.2.2 2004/06/18 18:02:10 igor Exp $ */
+/* $Id: gdevpsdi.c 5290 2004-09-02 08:26:30Z igor $ */
 /* Image compression for PostScript and PDF writers */
 #include "stdio_.h"		/* for jpeglib.h */
 #include "jpeglib_.h"		/* for sdct.h */
@@ -121,8 +121,10 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
             orig_template = template = &s_DCTE_template;
         }
 	dict = pdip->ACSDict;
-    }
-    gs_c_param_list_read(dict);	/* ensure param list is in read mode */
+    } else if (!lossless)
+	return gs_error_rangecheck; /* Reject the alternative stream. */   
+    if (dict) /* NB: rather than dereference NULL lets continue on without a dict */
+	gs_c_param_list_read(dict);	/* ensure param list is in read mode */
     if (template == 0)	/* no compression */
 	return 0;
     if (pim->Width < 200 && pim->Height < 200) /* Prevent a fixed overflow. */
@@ -366,7 +368,7 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 	    gs_color_space *rgb_cs = gs_alloc_struct(mem, 
 		    gs_color_space, &st_color_space, "psdf_setup_image_filters");
 
-	    gs_cspace_init_DeviceRGB(rgb_cs);  /* idempotent initialization */
+	    gs_cspace_init_DeviceRGB(mem, rgb_cs);  /* idempotent initialization */
 	    pim->ColorSpace = rgb_cs;
 	}
 	if (params.Depth == -1)
