@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpsft.c,v 1.23 2002/10/07 08:28:56 ghostgum Exp $ */
+/* $Id: gdevpsft.c 5567 2004-12-08 21:35:13Z stefan $ */
 /* Write an embedded TrueType font */
 #include "memory_.h"
 #include <assert.h>
@@ -151,6 +151,8 @@ write_range(stream *s, gs_font_type42 *pfont, ulong start, uint length)
 		return code;
 	    size >>= 1;
 	}
+	if (code > 0 && size > code)
+	    size = code; /* Segmented data - see z42_string_proc. */
 	stream_write(s, ptr, size);
 	base += size;
     }
@@ -321,11 +323,11 @@ write_cmap(stream *s, gs_font *font, uint first_code, int num_glyphs,
 	    font->procs.encode_char(font, (gs_char)i, GLYPH_SPACE_INDEX);
 	uint glyph_index;
 
-	if (glyph == gs_no_glyph || glyph < gs_min_cid_glyph ||
+	if (glyph == gs_no_glyph || glyph < GS_MIN_GLYPH_INDEX ||
 	    glyph > max_glyph
 	    )
-	    glyph = gs_min_cid_glyph;
-	glyph_index = (uint)(glyph - gs_min_cid_glyph);
+	    glyph = GS_MIN_GLYPH_INDEX;
+	glyph_index = (uint)(glyph - GS_MIN_GLYPH_INDEX);
 	merge |= glyph_index;
 	put_u16(entries + 2 * i, glyph_index);
     }
@@ -383,6 +385,7 @@ size_cmap(gs_font *font, uint first_code, int num_glyphs, gs_glyph max_glyph,
 {
     stream poss;
 
+    s_init(&poss, NULL);
     swrite_position_only(&poss);
     write_cmap(&poss, font, first_code, num_glyphs, max_glyph, options, 0);
     return stell(&poss);
@@ -429,7 +432,7 @@ size_mtx(gs_font_type42 *pfont, gs_type42_mtx_t *pmtx, uint max_glyph,
 
 	if (code < 0)
 	    continue;
-	width = (int)(sbw[wmode + 2] * factor);
+	width = (int)(sbw[wmode + 2] * factor + 0.5);
 	if (width != prev_width)
 	    prev_width = width, last_width = i;
     }
@@ -550,7 +553,7 @@ compute_post(gs_font *font, post_t *post)
 	    post->glyphs[post->count].char_index = i;
 	    post->glyphs[post->count].size =
 		(mac_index < 0 ? str.size + 1 : 0);
-	    post->glyphs[post->count].glyph_index = glyph - gs_min_cid_glyph;
+	    post->glyphs[post->count].glyph_index = glyph - GS_MIN_GLYPH_INDEX;
 	    post->count++;
 	}
     }
@@ -652,6 +655,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
     uint glyf_length, loca_length;
     ulong glyf_checksum = 0L; /****** NO CHECKSUM ******/
     ulong loca_checksum[2] = {0L,0L};
+    ulong glyf_alignment = 0;
     uint numGlyphs = 0;		/* original value from maxp */
     byte head[56];		/* 0 mod 4 */
     gs_type42_mtx_t mtx[2];
@@ -783,11 +787,19 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 
 	if (glyph < gs_min_cid_glyph)
 	    return_error(gs_error_invalidfont);
-	glyph_index = glyph - gs_min_cid_glyph;
+	glyph_index = glyph  & ~GS_GLYPH_TAG;
 	if_debug1('L', "[L]glyph_index %u\n", glyph_index);
+	glyph_data.memory = pfont->memory;
 	if ((code = pfont->data.get_outline(pfont, glyph_index, &glyph_data)) >= 0) {
+	    /* Since indexToLocFormat==0 assumes even glyph lengths,
+	       round it up here. If later we choose indexToLocFormat==1,
+	       subtract the glyf_alignment to compensate it. */
+	    uint l = (glyph_data.bits.size + 1) & ~1;
+
 	    max_glyph = max(max_glyph, glyph_index);
-	    glyf_length += glyph_data.bits.size;
+	    glyf_length += l;
+	    if (l != glyph_data.bits.size)
+		glyf_alignment++;
 	    if_debug1('L', "[L]  size %u\n", glyph_data.bits.size);
 	    gs_glyph_data_free(&glyph_data, "psf_write_truetype_data");
 	}
@@ -803,14 +815,16 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	glyf_length = 0;
 	loca_length = 0;
     } else {
-	/* Acrobat Reader won't accept fonts with empty glyfs. */
-	if (glyf_length == 0)
-	    glyf_length = 1;
 	/*loca_length = (max_glyph + 2) << 2;*/
 	loca_length = (numGlyphs + 1) << 2;
 	indexToLocFormat = (glyf_length > 0x1fffc);
 	if (!indexToLocFormat)
 	    loca_length >>= 1;
+	else
+	    glyf_length -= glyf_alignment;
+	/* Acrobat Reader won't accept fonts with empty glyfs. */
+	if (glyf_length == 0)
+	    glyf_length = 1;
     }
     if_debug2('l', "[l]max_glyph = %lu, glyf_length = %lu\n",
 	      (ulong)max_glyph, (ulong)glyf_length);
@@ -844,7 +858,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
     numTables_out = numTables + 1 /* head */
 	+ !writing_stripped * 2	/* glyf, loca */
 	+ generate_mtx * (have_hvhea[0] + have_hvhea[1]) /* hmtx, vmtx */
-	+ !no_generate		/* OS/2 */
+	+ !have_OS_2		/* OS/2 */
 	+ !have_cmap + !have_name + !have_post;
     if (numTables_out >= MAX_NUM_TABLES)
 	return_error(gs_error_limitcheck);
@@ -873,7 +887,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 
 	if (!have_cmap) {
 	    cmap_length = size_cmap(font, TT_BIAS, 256,
-				    gs_min_cid_glyph + max_glyph, options);
+				    GS_MIN_GLYPH_INDEX + max_glyph, options);
 	    offset = put_table(tab, "cmap", 0L /****** NO CHECKSUM ******/,
 			       offset, cmap_length);
 	    tab += 16;
@@ -962,6 +976,26 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	    uint length = u32(tab + 12);
 
 	    switch (u32(tab)) {
+	    case W('O','S','/','2'):
+		if (!have_cmap) {
+		    /*
+		     * Adjust the first and last character indices in the OS/2
+		     * table to reflect the values in the generated cmap.
+		     */
+		    const byte *pos2;
+		    ttf_OS_2_t os2;
+
+		    ACCESS(OS_2_start, OS_2_length, pos2);
+		    memcpy(&os2, pos2, min(OS_2_length, sizeof(os2)));
+		    update_OS_2(&os2, TT_BIAS, 256);
+		    stream_write(s, &os2, OS_2_length);
+		    put_pad(s, OS_2_length);
+		} else {
+		    /* Just copy the existing OS/2 table. */
+		    write_range(s, pfont, OS_2_start, OS_2_length);
+		    put_pad(s, OS_2_length);
+		}
+	    break;
 	    case W('h','h','e','a'):
 	    case W('v','h','e','a'):
 		if (generate_mtx) {
@@ -981,20 +1015,23 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 
 	/* Write glyf. */
 
-	if (is_subset)
-	    psf_enumerate_glyphs_reset(penum);
-	else
-	    psf_enumerate_glyphs_begin(penum, font, NULL, max_glyph + 1,
-				       GLYPH_SPACE_INDEX);
+	psf_enumerate_glyphs_reset(penum);
 	for (offset = 0; psf_enumerate_glyphs_next(penum, &glyph) != 1; ) {
 	    gs_glyph_data_t glyph_data;
 
+	    glyph_data.memory = pfont->memory;
 	    if ((code = pfont->data.get_outline(pfont,
-						glyph - gs_min_cid_glyph,
+						glyph & ~GS_GLYPH_TAG,
 						&glyph_data)) >= 0
 		) {
+		uint l = glyph_data.bits.size, zero = 0;
+
+		if (!indexToLocFormat)
+		    l = (l + 1) & ~1;
 		stream_write(s, glyph_data.bits.data, glyph_data.bits.size);
-		offset += glyph_data.bits.size;
+		if (glyph_data.bits.size < l)
+		    stream_write(s, &zero, 1);
+		offset += l;
 		if_debug2('L', "[L]glyf index = %u, size = %u\n",
 			  i, glyph_data.bits.size);
 		gs_glyph_data_free(&glyph_data, "psf_write_truetype_data");
@@ -1009,55 +1046,40 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	/* Write loca. */
 
 	psf_enumerate_glyphs_reset(penum);
-	glyph_prev = gs_min_cid_glyph;
+	glyph_prev = 0;
 	for (offset = 0; psf_enumerate_glyphs_next(penum, &glyph) != 1; ) {
 	    gs_glyph_data_t glyph_data;
+	    uint glyph_index = glyph & ~GS_GLYPH_TAG;
 
-	    for (; glyph_prev <= glyph; ++glyph_prev)
+	    for (; glyph_prev <= glyph_index; ++glyph_prev)
 		put_loca(s, offset, indexToLocFormat);
-	    if ((code = pfont->data.get_outline(pfont, glyph - gs_min_cid_glyph,
+	    glyph_data.memory = pfont->memory;
+	    if ((code = pfont->data.get_outline(pfont, glyph_index,
 						&glyph_data)) >= 0
 		) {
-		offset += glyph_data.bits.size;
+		uint l = glyph_data.bits.size;
+
+		if (!indexToLocFormat)
+		    l = (l + 1) & ~1;
+		offset += l;
 		gs_glyph_data_free(&glyph_data, "psf_write_truetype_data");
 	    }
 
 	}
 	/* Pad to numGlyphs + 1 entries (including the trailing entry). */
-	for (; glyph_prev <= gs_min_cid_glyph + numGlyphs; ++glyph_prev)
+	for (; glyph_prev <= numGlyphs; ++glyph_prev)
 	    put_loca(s, offset, indexToLocFormat);
 	put_pad(s, loca_length);
 
 	/* If necessary, write cmap, name, and OS/2. */
 
 	if (!have_cmap)
-	    write_cmap(s, font, TT_BIAS, 256, gs_min_cid_glyph + max_glyph,
+	    write_cmap(s, font, TT_BIAS, 256, GS_MIN_GLYPH_INDEX + max_glyph,
 		       options, cmap_length);
 	if (!have_name)
 	    write_name(s, &font_name);
-#if TT_GENERATE_OS_2
 	if (!have_OS_2)
 	    write_OS_2(s, font, TT_BIAS, 256);
-	else
-#endif
-	    if (!have_cmap) {
-		/*
-		 * Adjust the first and last character indices in the OS/2
-		 * table to reflect the values in the generated cmap.
-		 */
-		const byte *pos2;
-		ttf_OS_2_t os2;
-
-		ACCESS(OS_2_start, OS_2_length, pos2);
-		memcpy(&os2, pos2, min(OS_2_length, sizeof(os2)));
-		update_OS_2(&os2, TT_BIAS, 256);
-		stream_write(s, &os2, OS_2_length);
-		put_pad(s, OS_2_length);
-	    } else if (!writing_cid) {
-		/* Just copy the existing OS/2 table. */
-		write_range(s, pfont, OS_2_start, OS_2_length);
-		put_pad(s, OS_2_length);
-	    }
 
 	/* If necessary, write [hv]mtx. */
 
@@ -1163,7 +1185,7 @@ psf_write_cid2_font(stream *s, gs_font_cid2 *pfont, int options,
 			     (subset_bits ? subset_size : 0),
 			     GLYPH_SPACE_INDEX);
     return psf_write_truetype_data(s, (gs_font_type42 *)font,
-				   WRITE_TRUETYPE_CID, &genum,
+				   options | WRITE_TRUETYPE_CID, &genum,
 				   subset_bits != 0, alt_font_name);
 }
 
