@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpsfx.c,v 1.16 2002/10/07 08:28:56 ghostgum Exp $ */
+/* $Id: gdevpsfx.c 4188 2003-09-04 14:28:47Z igor $ */
 /* Convert Type 1 Charstrings to Type 2 */
 #include "math_.h"
 #include "memory_.h"
@@ -40,6 +40,22 @@
 
 #define CE_OFFSET 32		/* offset for extended opcodes */
 
+typedef struct {
+    fixed v0, v1;		/* coordinates */
+    ushort index;		/* sequential index of hint */
+} cv_stem_hint;
+typedef struct {
+    int count;
+    int current;		/* cache cursor for search */
+    /*
+     * For dotsection and Type 1 Charstring hint replacement,
+     * we store active hints at the bottom of the table, and
+     * replaced hints at the top.
+     */
+    int replaced_count;		/* # of replaced hints at top */
+    cv_stem_hint data[max_total_stem_hints];
+} cv_stem_hint_table;
+
 /* Skip over the initial bytes in a Charstring, if any. */
 private void
 skip_iv(gs_type1_state *pcis)
@@ -65,11 +81,8 @@ private void
 type1_next_init(gs_type1_state *pcis, const gs_glyph_data_t *pgd,
 		gs_font_type1 *pfont)
 {
-    static const gs_log2_scale_point no_scale = {0, 0};
-
-    gs_type1_interp_init(pcis, NULL, NULL, &no_scale, false, 0, pfont);
+    gs_type1_interp_init(pcis, NULL, NULL, NULL, NULL, false, 0, pfont);
     pcis->flex_count = flex_max;
-    pcis->dotsection_flag = dotsection_out;
     pcis->ipstack[0].cs_data = *pgd;
     skip_iv(pcis);
 }
@@ -99,15 +112,15 @@ type1_callsubr(gs_type1_state *pcis, int index)
 
 /* Add 1 or 3 stem hints. */
 private int
-type1_stem1(gs_type1_state *pcis, stem_hint_table *psht, const fixed *pv,
+type1_stem1(gs_type1_state *pcis, cv_stem_hint_table *psht, const fixed *pv,
 	    fixed lsb, byte *active_hints)
 {
     fixed v0 = pv[0] + lsb, v1 = v0 + pv[1];
-    stem_hint *bot = &psht->data[0];
-    stem_hint *orig_top = bot + psht->count;
-    stem_hint *top = orig_top;
+    cv_stem_hint *bot = &psht->data[0];
+    cv_stem_hint *orig_top = bot + psht->count;
+    cv_stem_hint *top = orig_top;
 
-    if (psht->count >= max_stems)
+    if (psht->count >= max_total_stem_hints)
 	return_error(gs_error_limitcheck);
     while (top > bot &&
 	   (v0 < top[-1].v0 || (v0 == top[-1].v0 && v1 < top[-1].v1))
@@ -131,7 +144,7 @@ type1_stem1(gs_type1_state *pcis, stem_hint_table *psht, const fixed *pv,
     return 0;
 }
 private void
-type1_stem3(gs_type1_state *pcis, stem_hint_table *psht, const fixed *pv3,
+type1_stem3(gs_type1_state *pcis, cv_stem_hint_table *psht, const fixed *pv3,
 	    fixed lsb, byte *active_hints)
 {
     type1_stem1(pcis, psht, pv3, lsb, active_hints);
@@ -147,19 +160,21 @@ private int
 type1_next(gs_type1_state *pcis)
 {
     ip_state_t *ipsp = &pcis->ipstack[pcis->ips_count - 1];
-    const byte *cip;
+    const byte *cip, *cipe;
     crypt_state state;
 #define CLEAR (csp = pcis->ostack - 1)
     fixed *csp = &pcis->ostack[pcis->os_count - 1];
     const bool encrypted = pcis->pfont->data.lenIV >= 0;
-    int c, code, num_results;
+    int c, code, num_results, c0;
 
  load:
     cip = ipsp->ip;
+    cipe = ipsp->cs_data.bits.data + ipsp->cs_data.bits.size;
     state = ipsp->dstate;
     for (;;) {
-	uint c0 = *cip++;
-
+        if (cip >= cipe)
+	    return_error(gs_error_invalidfont);
+	c0 = *cip++;
 	charstring_next(c0, state, c, encrypted);
 	if (c >= c_num1) {
 	    /* This is a number, decode it and push it on the stack. */
@@ -214,6 +229,10 @@ type1_next(gs_type1_state *pcis)
 	    pcis->ips_count--;
 	    --ipsp;
 	    goto load;
+	case c_undoc15:
+	    /* See gstype1.h for information on this opcode. */
+	    CLEAR;
+	    continue;
 	case cx_escape:
 	    charstring_next(*cip, state, c, encrypted);
 	    ++cip;
@@ -346,7 +365,7 @@ type2_put_fixed(stream *s, fixed v)
 
 /* Put a stem hint table on a stream. */
 private void
-type2_put_stems(stream *s, int os_count, const stem_hint_table *psht, int op)
+type2_put_stems(stream *s, int os_count, const cv_stem_hint_table *psht, int op)
 {
     fixed prev = 0;
     int pushed = os_count;
@@ -379,6 +398,7 @@ type2_put_hintmask(stream *s, const byte *mask, uint size)
 
 /* ------ Main program ------ */
 
+
 /*
  * Convert a Type 1 Charstring to (unencrypted) Type 2.
  * For simplicity, we expand all Subrs in-line.
@@ -394,9 +414,15 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 			   gs_font_type1 *pfont)
 {
     gs_type1_state cis;
+    cv_stem_hint_table hstem_hints;	/* horizontal stem hints */
+    cv_stem_hint_table vstem_hints;	/* vertical stem hints */
     bool first = true;
     bool replace_hints = false;
     bool hints_changed = false;
+    enum {
+	dotsection_in = 0,
+	dotsection_out = -1
+    } dotsection_flag = dotsection_out;
     byte active_hints[(max_total_stem_hints + 7) / 8];
     byte dot_save_hints[(max_total_stem_hints + 7) / 8];
     uint hintmask_size;
@@ -429,12 +455,14 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 	CLEAR_OP();\
     }\
   END
+    fixed mx0 = 0, my0 = 0; /* See ce1_setcurrentpoint. */
 
     /*
      * Do a first pass to collect hints.  Note that we must also process
      * [h]sbw, because the hint coordinates are relative to the lsb.
      */
-    reset_stem_hints(&cis);
+    hstem_hints.count = hstem_hints.replaced_count = hstem_hints.current = 0;
+    vstem_hints.count = vstem_hints.replaced_count = vstem_hints.current = 0;
     type1_next_init(&cis, pgd, pfont);
     for (;;) {
 	int c = type1_next(&cis);
@@ -450,20 +478,20 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 	    gs_type1_sbw(&cis, cis.ostack[0], fixed_0, cis.ostack[1], fixed_0);
 	    goto clear;
 	case cx_hstem:
-	    type1_stem1(&cis, &cis.hstem_hints, csp - 1, cis.lsb.y, NULL);
+	    type1_stem1(&cis, &hstem_hints, csp - 1, cis.lsb.y, NULL);
 	    goto clear;
 	case cx_vstem:
-	    type1_stem1(&cis, &cis.vstem_hints, csp - 1, cis.lsb.x, NULL);
+	    type1_stem1(&cis, &vstem_hints, csp - 1, cis.lsb.x, NULL);
 	    goto clear;
 	case CE_OFFSET + ce1_sbw:
 	    gs_type1_sbw(&cis, cis.ostack[0], cis.ostack[1],
 			 cis.ostack[2], cis.ostack[3]);
 	    goto clear;
 	case CE_OFFSET + ce1_vstem3:
-	    type1_stem3(&cis, &cis.vstem_hints, csp - 5, cis.lsb.x, NULL);
+	    type1_stem3(&cis, &vstem_hints, csp - 5, cis.lsb.x, NULL);
 	    goto clear;
 	case CE_OFFSET + ce1_hstem3:
-	    type1_stem3(&cis, &cis.hstem_hints, csp - 5, cis.lsb.y, NULL);
+	    type1_stem3(&cis, &hstem_hints, csp - 5, cis.lsb.y, NULL);
 	clear:
 	    type1_clear(&cis);
 	    continue;
@@ -489,14 +517,14 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
     {
 	int i;
 
-	for (i = 0; i < cis.hstem_hints.count; ++i)
-	    cis.hstem_hints.data[i].index = i;
-	for (i = 0; i < cis.vstem_hints.count; ++i)
-	    cis.vstem_hints.data[i].index = i + cis.hstem_hints.count;
+	for (i = 0; i < hstem_hints.count; ++i)
+	    hstem_hints.data[i].index = i;
+	for (i = 0; i < vstem_hints.count; ++i)
+	    vstem_hints.data[i].index = i + hstem_hints.count;
     }
     if (replace_hints) {
 	hintmask_size =
-	    (cis.hstem_hints.count + cis.vstem_hints.count + 7) / 8;
+	    (hstem_hints.count + vstem_hints.count + 7) / 8;
 	memset(active_hints, 0, hintmask_size);
     } else 
 	hintmask_size = 0;
@@ -530,33 +558,40 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 	    type1_clear(&cis);
 	    continue;
 	case cx_hstem:
-	    type1_stem1(&cis, &cis.hstem_hints, csp - 1, cis.lsb.y, active_hints);
+	    type1_stem1(&cis, &hstem_hints, csp - 1, cis.lsb.y, active_hints);
 	hint:
 	    HINTS_CHANGED();
 	    type1_clear(&cis);
 	    continue;
 	case cx_vstem:
-	    type1_stem1(&cis, &cis.vstem_hints, csp - 1, cis.lsb.x, active_hints);
+	    type1_stem1(&cis, &vstem_hints, csp - 1, cis.lsb.x, active_hints);
 	    goto hint;
 	case CE_OFFSET + ce1_vstem3:
-	    type1_stem3(&cis, &cis.vstem_hints, csp - 5, cis.lsb.x, active_hints);
+	    type1_stem3(&cis, &vstem_hints, csp - 5, cis.lsb.x, active_hints);
 	    goto hint;
 	case CE_OFFSET + ce1_hstem3:
-	    type1_stem3(&cis, &cis.hstem_hints, csp - 5, cis.lsb.y, active_hints);
+	    type1_stem3(&cis, &hstem_hints, csp - 5, cis.lsb.y, active_hints);
 	    goto hint;
 	case CE_OFFSET + ce1_dotsection:
-	    if (cis.dotsection_flag == dotsection_out) {
+	    if (dotsection_flag == dotsection_out) {
 		memcpy(dot_save_hints, active_hints, hintmask_size);
 		memset(active_hints, 0, hintmask_size);
-		cis.dotsection_flag = dotsection_in;
+		dotsection_flag = dotsection_in;
 	    } else {
 		memcpy(active_hints, dot_save_hints, hintmask_size);
-		cis.dotsection_flag = dotsection_out;
+		dotsection_flag = dotsection_out;
 	    }
 	    HINTS_CHANGED();
 	    continue;
 	case c1_closepath:
+	    continue;
 	case CE_OFFSET + ce1_setcurrentpoint:
+	    if (first) {
+		/*  A workaround for fonts which use ce1_setcurrentpoint 
+		    in an illegal way for shifting a path. 
+		    See t1_hinter__setcurrentpoint for more information. */
+		mx0 = csp[-1], my0 = *csp;
+	    }
 	    continue;
 	case cx_vmoveto:
 	    mx = 0, my = *csp;
@@ -572,7 +607,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 	    if (first) {
 		if (cis.os_count)
 		    type2_put_fixed(s, *csp); /* width */
-		mx += cis.lsb.x, my += cis.lsb.y;
+		mx += cis.lsb.x + mx0, my += cis.lsb.y + my0;
 		first = false;
 	    }
 	    if (cis.flex_count != flex_max) {
@@ -614,17 +649,17 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 		cis.ostack[0] -= pfont->data.nominalWidthX;
 		cis.os_count = 1;
 	    }
-	    if (cis.hstem_hints.count) {
+	    if (hstem_hints.count) {
 		if (cis.os_count)
 		    type2_put_fixed(s, cis.ostack[0]);
-		type2_put_stems(s, cis.os_count, &cis.hstem_hints,
+		type2_put_stems(s, cis.os_count, &hstem_hints,
 				(replace_hints ? c2_hstemhm : cx_hstem));
 		cis.os_count = 0;
 	    }
-	    if (cis.vstem_hints.count) {
+	    if (vstem_hints.count) {
 		if (cis.os_count)
 		    type2_put_fixed(s, cis.ostack[0]);
-		type2_put_stems(s, cis.os_count, &cis.vstem_hints,
+		type2_put_stems(s, cis.os_count, &vstem_hints,
 				(replace_hints ? c2_vstemhm : cx_vstem));
 		cis.os_count = 0;
 	    }
