@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpsu.c,v 1.15.2.2 2004/03/24 08:38:53 giles Exp $ */
+/* $Id: gdevpsu.c 5408 2004-10-07 05:18:34Z ray $ */
 /* PostScript-writing utilities */
 #include "math_.h"
 #include "time_.h"
@@ -37,7 +37,7 @@ psw_print_lines(FILE *f, const char *const lines[])
     int i;
     for (i = 0; lines[i] != 0; ++i) {
 	if (fprintf(f, "%s\n", lines[i]) < 0)
-	    return_error(gs_error_ioerror);
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -60,6 +60,7 @@ psw_print_procset_name(FILE *f, const gx_device *dev,
     byte buf[100];		/* arbitrary */
     stream s;
 
+    s_init(&s, dev->memory);
     swrite_file(&s, f, buf, sizeof(buf));
     psw_put_procset_name(&s, dev, pdpc);
     sflush(&s);
@@ -151,7 +152,7 @@ private const char *const psw_end_prolog[] = {
  */
 private bool
 is_seekable(FILE *f)
-{
+{ 
     struct stat buf;
 
     if(fstat(fileno(f), &buf))
