@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevtrac.c,v 1.4 2002/02/21 22:24:52 giles Exp $ */
+/* $Id: gdevtrac.c 3988 2003-06-06 17:50:27Z igor $ */
 /* Tracing device (including sample high-level implementation) */
 #include "gx.h"
 #include "gserrors.h"
@@ -538,7 +538,7 @@ trace_text_begin(gx_device * dev, gs_imager_state * pis,
 	    int wmode = font->WMode;
 	    gs_glyph glyph =
 		((gs_font_base *)font)->procs.encode_char(font, ch,
-							  GLYPH_SPACE_INDEX);
+							  GLYPH_SPACE_NAME);
 	    gs_glyph_info_t info;
 
 	    if (glyph != gs_no_glyph &&
