@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevvec.c,v 1.16 2002/08/22 07:12:28 henrys Exp $ */
+/* $Id: gdevvec.c 5381 2004-09-27 04:55:00Z ray $ */
 /* Utilities for "vector" devices */
 #include "math_.h"
 #include "memory_.h"
@@ -252,10 +252,10 @@ gdev_vector_reset(gx_device_vector * vdev)
     {gs_imager_state_initial(1)};
 
     vdev->state = state_initial;
-    color_unset(&vdev->fill_color);
-    color_unset(&vdev->stroke_color);
+    gx_hld_saved_color_init(&vdev->saved_fill_color);
+    gx_hld_saved_color_init(&vdev->saved_stroke_color);
     vdev->clip_path_id =
-	vdev->no_clip_path_id = gs_next_ids(1);
+	vdev->no_clip_path_id = gs_next_ids(vdev->memory, 1);
 }
 
 /* Open the output file and stream. */
@@ -316,7 +316,8 @@ gdev_vector_open_file_options(gx_device_vector * vdev, uint strmbuf_size,
      */
     vdev->strm->procs.close = vdev->strm->procs.flush;
     if (vdev->bbox_device) {
-	gx_device_bbox_init(vdev->bbox_device, NULL);
+	gx_device_bbox_init(vdev->bbox_device, NULL, vdev->v_memory);
+        rc_increment(vdev->bbox_device);
 	gx_device_set_resolution((gx_device *) vdev->bbox_device,
 				 vdev->HWResolution[0],
 				 vdev->HWResolution[1]);
@@ -342,19 +343,6 @@ gdev_vector_stream(gx_device_vector * vdev)
     return vdev->strm;
 }
 
-/* Compare two drawing colors. */
-/* Right now we don't attempt to handle non-pure colors. */
-private bool
-drawing_color_eq(const gx_drawing_color * pdc1, const gx_drawing_color * pdc2)
-{
-    return (gx_dc_is_pure(pdc1) ?
-	    gx_dc_is_pure(pdc2) &&
-	    gx_dc_pure_color(pdc1) == gx_dc_pure_color(pdc2) :
-	    gx_dc_is_null(pdc1) ?
-	    gx_dc_is_null(pdc2) :
-	    false);
-}
-
 /* Update the logical operation. */
 int
 gdev_vector_update_log_op(gx_device_vector * vdev, gs_logical_operation_t lop)
@@ -371,27 +359,47 @@ gdev_vector_update_log_op(gx_device_vector * vdev, gs_logical_operation_t lop)
     return 0;
 }
 
+/* Update color (fill or stroke). */
+private int
+gdev_vector_update_color(gx_device_vector * vdev,
+			      const gs_imager_state * pis,
+			      const gx_drawing_color * pdcolor,
+			      gx_hl_saved_color *sc,
+			      int (*setcolor) (gx_device_vector * vdev, 
+			                       const gs_imager_state * pis, 
+					       const gx_drawing_color * pdc))
+{
+    gx_hl_saved_color temp;
+    int code;
+    bool hl_color = (*vdev_proc(vdev, can_handle_hl_color)) (vdev, pis, pdcolor);
+    const gs_imager_state *pis_for_hl_color = (hl_color ? pis : NULL);
+    
+    gx_hld_save_color(pis_for_hl_color, pdcolor, &temp);
+    if (gx_hld_saved_color_equal(&temp, sc))
+	return 0;
+    code = (*setcolor) (vdev, pis_for_hl_color, pdcolor);
+    if (code < 0)
+	return code;
+    *sc = temp;
+    return 0;
+}
+
 /* Update the fill color. */
 int
 gdev_vector_update_fill_color(gx_device_vector * vdev,
+			      const gs_imager_state * pis,
 			      const gx_drawing_color * pdcolor)
 {
-    if (!drawing_color_eq(pdcolor, &vdev->fill_color)) {
-	int code = (*vdev_proc(vdev, setfillcolor)) (vdev, pdcolor);
-
-	if (code < 0)
-	    return code;
-	vdev->fill_color = *pdcolor;
-    }
-    return 0;
+    return gdev_vector_update_color(vdev, pis, pdcolor, &vdev->saved_fill_color, 
+                                    vdev_proc(vdev, setfillcolor));
 }
 
 /* Update the state for filling a region. */
 private int
-update_fill(gx_device_vector * vdev, const gx_drawing_color * pdcolor,
-	    gs_logical_operation_t lop)
+update_fill(gx_device_vector * vdev, const gs_imager_state * pis, 
+	    const gx_drawing_color * pdcolor, gs_logical_operation_t lop)
 {
-    int code = gdev_vector_update_fill_color(vdev, pdcolor);
+    int code = gdev_vector_update_fill_color(vdev, pis, pdcolor);
 
     if (code < 0)
 	return code;
@@ -410,7 +418,7 @@ gdev_vector_prepare_fill(gx_device_vector * vdev, const gs_imager_state * pis,
 	    return code;
 	vdev->state.flatness = params->flatness;
     }
-    return update_fill(vdev, pdcolor, pis->log_op);
+    return update_fill(vdev, pis, pdcolor, pis->log_op);
 }
 
 /* Compare two dash patterns. */
@@ -509,13 +517,11 @@ gdev_vector_prepare_stroke(gx_device_vector * vdev,
 	}
     }
     if (pdcolor) {
-	if (!drawing_color_eq(pdcolor, &vdev->stroke_color)) {
-	    int code = (*vdev_proc(vdev, setstrokecolor)) (vdev, pdcolor);
+	int code = gdev_vector_update_color(vdev, pis, pdcolor, 
+		    &vdev->saved_stroke_color, vdev_proc(vdev, setstrokecolor));
 
-	    if (code < 0)
-		return code;
-	    vdev->stroke_color = *pdcolor;
-	}
+	if (code < 0)
+	    return code;
     }
     return 0;
 }
@@ -621,7 +627,7 @@ gdev_vector_dopath_segment(gdev_vector_dopath_state_t *state, int pe_op,
 	    if (state->first)
 		state->start = vp[0], state->first = false;
 	    code = vdev_proc(vdev, moveto)
-		(vdev, state->prev.x, state->prev.y, vp[0].x, vp[0].y,
+		(vdev, 0/*unused*/, 0/*unused*/, vp[0].x, vp[0].y,
 		 state->type);
 	    state->prev = vp[0];
 	    break;
@@ -800,7 +806,8 @@ gdev_vector_close_file(gx_device_vector * vdev)
     vdev->file = 0;
     err = ferror(f);
     /* We prevented sclose from closing the file. */
-    if (fclose(f) != 0 || err != 0)
+    if (gx_device_close_output_file((gx_device *)vdev, vdev->fname, f) != 0 
+	|| err != 0)
 	return_error(gs_error_ioerror);
     return 0;
 }
@@ -840,7 +847,7 @@ gdev_vector_begin_image(gx_device_vector * vdev,
 	(code = gdev_vector_update_clip_path(vdev, pcpath)) < 0 ||
 	((pim->ImageMask ||
 	  (pim->CombineWithColor && rop3_uses_T(pis->log_op))) &&
-	 (code = gdev_vector_update_fill_color(vdev, pdcolor)) < 0) ||
+	 (code = gdev_vector_update_fill_color(vdev, pis, pdcolor)) < 0) ||
 	(vdev->bbox_device &&
 	 (code = (*dev_proc(vdev->bbox_device, begin_image))
 	  ((gx_device *) vdev->bbox_device, pis, pim, format, prect,
@@ -1006,15 +1013,18 @@ gdev_vector_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     /* Ignore the initial fill with white. */
     if (!vdev->in_page && color == vdev->white)
 	return 0;
-    color_set_pure(&dcolor, color);
+    /*
+     * The original colorspace and client color are unknown so use
+     * set_nonclient_dev_color instead of color_set_pure.
+     */
+    set_nonclient_dev_color(&dcolor, color);
     {
-	int code = update_fill(vdev, &dcolor, rop3_T);
+	/* Make sure we aren't being clipped. */
+	int code = gdev_vector_update_clip_path(vdev, NULL);
 
 	if (code < 0)
 	    return code;
-	/* Make sure we aren't being clipped. */
-	code = gdev_vector_update_clip_path(vdev, NULL);
-	if (code < 0)
+	if ((code = update_fill(vdev, NULL, &dcolor, rop3_T)) < 0)
 	    return code;
     }
     if (vdev->bbox_device) {
@@ -1097,7 +1107,7 @@ gdev_vector_fill_trapezoid(gx_device * dev, const gs_fixed_edge * left,
 
 #define y0 ybot
 #define y1 ytop
-    int code = update_fill(vdev, pdevc, lop);
+    int code = update_fill(vdev, NULL, pdevc, lop);
     gs_fixed_point points[4];
 
     if (code < 0)
@@ -1137,7 +1147,7 @@ gdev_vector_fill_parallelogram(gx_device * dev,
 		  const gx_device_color * pdevc, gs_logical_operation_t lop)
 {
     fixed pax = px + ax, pay = py + ay;
-    int code = update_fill(vdev, pdevc, lop);
+    int code = update_fill(vdev, NULL, pdevc, lop);
     gs_fixed_point points[4];
 
     if (code < 0)
@@ -1167,7 +1177,7 @@ gdev_vector_fill_triangle(gx_device * dev,
 		 fixed px, fixed py, fixed ax, fixed ay, fixed bx, fixed by,
 		  const gx_device_color * pdevc, gs_logical_operation_t lop)
 {
-    int code = update_fill(vdev, pdevc, lop);
+    int code = update_fill(vdev, NULL, pdevc, lop);
     gs_fixed_point points[3];
 
     if (code < 0)
