@@ -1,4 +1,4 @@
-/* Copyright (C) 1994 - 1997 artofcode LLC.  All rights reserved.
+/* Copyright (C) 1994 - 2003 artofcode LLC.  All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,15 +14,12 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gp_mac.h,v 1.4 2002/02/21 22:24:52 giles Exp $ */
+/* $Id: gp_mac.h 4133 2003-08-15 20:19:21Z giles $ */
 
 #ifndef gp_mac_INCLUDED
 #  define gp_mac_INCLUDED
 
-
-FILE *  gp_fopen (const char * fname, const char * mode );
-FILE *  gp_open_scratch_file (const char *prefix, char *fname, const char *mode);
-
+/* no special definitions for macos */
 
 #endif 				/* gp_mac_INCLUDED */
 
