@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gp_os2.c,v 1.16 2002/11/17 22:50:32 ray Exp $ */
+/* $Id: gp_os2.c 5383 2004-09-27 21:14:00Z ghostgum $ */
 /* Common platform-specific routines for OS/2 and MS-DOS */
 /* compiled with GCC/EMX */
 
@@ -54,6 +54,7 @@
 #include "stdlib.h"		/* need _osmode, exit */
 #include "time_.h"
 #include <time.h>		/* should this be in time_.h? */
+#include "gp_os2.h"
 #include "gdevpm.h"
 #ifdef __EMX__
 #include <sys/emxload.h>
@@ -152,11 +153,28 @@ gp_file_is_console(FILE * f)
 	return ((regs.h.dl & 0x80) != 0 && (regs.h.dl & 3) != 0);
     }
 #endif
-    if ((f == gs_stdin) || (f == gs_stdout) || (f == gs_stderr))
+    if (fileno(f) <= 2)
 	return true;
     return false;
 }
 
+/* ------ Persistent data cache ------*/
+  
+/* insert a buffer under a (type, key) pair */
+int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
+{ 
+    /* not yet implemented */
+    return 0;
+} 
+ 
+/* look up a (type, key) in the cache */
+int gp_cache_query(int type, byte* key, int keylen, void **buffer,
+    gp_cache_alloc alloc, void *userdata)
+{
+    /* not yet implemented */
+    return -1;
+}
+
 /* ------ File naming and accessing ------ */
 
 /* Define the character used for separating file names in a list. */
@@ -179,70 +197,6 @@ const char gp_fmode_binary_suffix[] = "b";
 const char gp_fmode_rb[] = "rb";
 const char gp_fmode_wb[] = "wb";
 
-/* Answer whether a path_string can meaningfully have a prefix applied */
-bool
-gp_pathstring_not_bare(const char *fname, uint len)
-{   /* A file name is not bare if it contains a drive specifications	*/
-    /* (second character is a :) or if it starts with a '.', '/' or '\\'*/
-    /* or it contains '/../' (parent reference)				*/
-    if ((len > 0) && ((*fname == '/') || (*fname == '\\') ||
-	  (*fname == '.') || ((len > 2) && (fname[1] == ':'))))
-	return true;
-    while (len-- > 3) {
-        int c = *fname++;
-
-	if (((c == '/') || (c == '\\')) &&
-	    ((len >= 3) && (bytes_compare(fname, 2, "..", 2) == 0) &&
-			((fname[2] == '/') || (fname[2] == '\\'))))
-	    return true;
-    }
-    return false;
-}
-
-/* Answer whether the file_name references the directory	*/
-/* containing the specified path (parent). 			*/
-bool
-gp_file_name_references_parent(const char *fname, unsigned len)
-{
-    int i = 0, last_sep_pos = -1;
-
-    /* A file name references its parent directory if it starts */
-    /* with ../ or ..\  or if one of these strings follows / or \ */
-    while (i < len) {
-	if (fname[i] == '/' || fname[i] == '\\') {
-	    last_sep_pos = i++;
-	    continue;
-	}
-	if (fname[i++] != '.')
-	    continue;
-        if (i > last_sep_pos + 2 || (i < len && fname[i] != '.'))
-	    continue;
-	i++;
-	/* have separator followed by .. */
-	if (i < len && (fname[i] == '/' || fname[i++] == '\\'))
-	    return true;
-    }
-    return false;
-}
-
-
-/* Answer the string to be used for combining a directory/device prefix */
-/* with a base file name. The prefix directory/device is examined to	*/
-/* determine if a separator is needed and may return an empty string	*/
-const char *
-gp_file_name_concat_string(const char *prefix, uint plen)
-{
-    if (plen > 0)
-	switch (prefix[plen - 1]) {
-	    case ':':
-	    case '/':
-	    case '\\':
-		return "";
-	};
-    return "/";
-}
-
-
 /* ------ File enumeration ------ */
 
 
@@ -444,9 +398,7 @@ gp_do_exit(int exit_status)
 }
 
 /* ------ Printer accessing ------ */
-private int pm_find_queue(char *queue_name, char *driver_name);
 private int is_os2_spool(const char *queue);
-private int pm_spool(char *filename, const char *queue);
 
 /* Put a printer file (which might be stdout) into binary or text mode. */
 /* This is not a standard gp procedure, */
@@ -547,7 +499,7 @@ gp_setmode_binary(FILE * pfile, bool binary)
 /* If strlen(queue_name)==0, return default queue and driver name */
 /* If queue_name supplied, return driver_name */
 /* returns 0 if OK, non-zero for error */
-private int
+int
 pm_find_queue(char *queue_name, char *driver_name)
 {
     SPLERR splerr;
@@ -600,9 +552,9 @@ pm_find_queue(char *queue_name, char *driver_name)
 		    } else {
 			/* list queue details */
 			if (prq->fsType & PRQ3_TYPE_APPDEFAULT)
-			    eprintf1("  %s  (DEFAULT)\n", prq->pszName);
+			    eprintf1("  \042%s\042  (DEFAULT)\n", prq->pszName);
 			else
-			    eprintf1("  %s\n", prq->pszName);
+			    eprintf1("  \042%s\042\n", prq->pszName);
 		    }
 		    prq++;
 		}		/*endfor cReturned */
@@ -647,7 +599,7 @@ is_os2_spool(const char *queue)
 /* Spool file to queue */
 /* return 0 if successful, non-zero if error */
 /* if filename is NULL, return 0 if spool queue is valid, non-zero if error */
-private int
+int
 pm_spool(char *filename, const char *queue)
 {
     HSPL hspl;
@@ -748,12 +700,13 @@ FILE *
 gp_open_scratch_file(const char *prefix, char fname[gp_file_name_sizeof],
 		     const char *mode)
 {
+    FILE *f;
 #ifdef __IBMC__
     char *temp = 0;
     char *tname;
     int prefix_length = strlen(prefix);
 
-    if (!gp_pathstring_not_bare(prefix, prefix_length)) {
+    if (!gp_file_name_is_absolute(prefix, prefix_length)) {
 	temp = getenv("TMPDIR");
 	if (temp == 0)
 	    temp = getenv("TEMP");
@@ -773,7 +726,7 @@ gp_open_scratch_file(const char *prefix, char fname[gp_file_name_sizeof],
     int prefix_length = strlen(prefix);
     int len = gp_file_name_sizeof - prefix_length - 7;
 
-    if (gp_pathstring_not_bare(prefix, prefix_length) ||
+    if (gp_file_name_is_absolute(prefix, prefix_length) ||
 	gp_gettmpdir(fname, &len) != 0)
 	*fname = 0;
     else {
@@ -797,7 +750,10 @@ gp_open_scratch_file(const char *prefix, char fname[gp_file_name_sizeof],
     strcat(fname, "XXXXXX");
     mktemp(fname);
 #endif
-    return gp_fopentemp(fname, mode);
+    f = gp_fopentemp(fname, mode);
+    if (f == NULL)
+	eprintf1("**** Could not open temporary file %s\n", fname);
+    return f;
 }
 
 /* Open a file with the given name, as a stream of uninterpreted bytes. */
@@ -807,3 +763,101 @@ gp_fopen(const char *fname, const char *mode)
     return fopen(fname, mode);
 }
 
+/* -------------- Helpers for gp_file_name_combine_generic ------------- */
+
+uint gp_file_name_root(const char *fname, uint len)
+{   int i = 0;
+    
+    if (len == 0)
+	return 0;
+    if (len > 1 && fname[0] == '\\' && fname[1] == '\\') {
+	/* A network path: "\\server\share\" */
+	int k = 0;
+
+	for (i = 2; i < len; i++)
+	    if (fname[i] == '\\' || fname[i] == '/')
+		if (k++) {
+		    i++;
+		    break;
+		}
+    } else if (fname[0] == '/' || fname[0] == '\\') {
+	/* Absolute with no drive. */
+	i = 1;
+    } else if (len > 1 && fname[1] == ':') {
+	/* Absolute with a drive. */
+	i = (len > 2 && (fname[2] == '/' || fname[2] == '\\') ? 3 : 2);
+    }
+    return i;
+}
+
+uint gs_file_name_check_separator(const char *fname, int len, const char *item)
+{   if (len > 0) {
+	if (fname[0] == '/' || fname[0] == '\\')
+	    return 1;
+    } else if (len < 0) {
+	if (fname[-1] == '/' || fname[-1] == '\\')
+	    return 1;
+    }
+    return 0;
+}
+
+bool gp_file_name_is_parent(const char *fname, uint len)
+{   return len == 2 && fname[0] == '.' && fname[1] == '.';
+}
+
+bool gp_file_name_is_current(const char *fname, uint len)
+{   return len == 1 && fname[0] == '.';
+}
+
+const char *gp_file_name_separator(void)
+{   return "/";
+}
+
+const char *gp_file_name_directory_separator(void)
+{   return "/";
+}
+
+const char *gp_file_name_parent(void)
+{   return "..";
+}
+
+const char *gp_file_name_current(void)
+{   return ".";
+}
+
+bool gp_file_name_is_partent_allowed(void)
+{   return true;
+}
+
+bool gp_file_name_is_empty_item_meanful(void)
+{   return false;
+}
+
+gp_file_name_combine_result
+gp_file_name_combine(const char *prefix, uint plen, const char *fname, uint flen, 
+		    bool no_sibling, char *buffer, uint *blen)
+{
+    return gp_file_name_combine_generic(prefix, plen, 
+	    fname, flen, no_sibling, buffer, blen);
+}
+
+/* ------ Font enumeration ------ */
+ 
+ /* This is used to query the native os for a list of font names and
+  * corresponding paths. The general idea is to save the hassle of
+  * building a custom fontmap file.
+  */
+ 
+void *gp_enumerate_fonts_init(gs_memory_t *mem)
+{
+    return NULL;
+}
+         
+int gp_enumerate_fonts_next(void *enum_state, char **fontname, char **path)
+{
+    return 0;
+}
+                         
+void gp_enumerate_fonts_free(void *enum_state)
+{
+}           
