@@ -14,11 +14,12 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gscdef.c,v 1.33.2.4 2004/08/17 19:52:31 giles Exp $ */
+/* $Id: gscdef.c 6423 2005-12-31 22:22:28Z giles $ */
 /* Configuration scalars */
+
 #include "std.h"
 #include "gscdefs.h"		/* interface */
-#include "gconf.h"		/* for #defines */
+#include "gconfigd.h"		/* for #defines */
 
 /* ---------------- Miscellaneous system parameters ---------------- */
 
@@ -33,7 +34,7 @@ CONFIG_CONST long gs_buildtime = GS_BUILDTIME;
 
 #ifndef GS_COPYRIGHT
 #  define GS_COPYRIGHT\
-	"Copyright (C) 2004 artofcode LLC, Benicia, CA.  All rights reserved."
+	"Copyright (C) 2005 artofcode LLC, Benicia, CA.  All rights reserved."
 #endif
 const char *CONFIG_CONST gs_copyright = GS_COPYRIGHT;
 
