@@ -14,9 +14,11 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gscdevn.c,v 1.14 2002/10/08 00:49:49 dan Exp $ */
+/* $Id: gscdevn.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* DeviceN color space and operation definition */
+
 #include "memory_.h"
+#include "string_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gscdevn.h"
@@ -34,6 +36,7 @@
 #include "gzstate.h"
 #include "gxdevcli.h"
 #include "gsovrc.h"
+#include "stream.h"
 
 /* ---------------- Color space ---------------- */
 
@@ -53,6 +56,7 @@ private cs_proc_remap_concrete_color(gx_remap_concrete_DeviceN);
 private cs_proc_install_cspace(gx_install_DeviceN);
 private cs_proc_set_overprint(gx_set_overprint_DeviceN);
 private cs_proc_adjust_cspace_count(gx_adjust_cspace_DeviceN);
+private cs_proc_serialize(gx_serialize_DeviceN);
 const gs_color_space_type gs_color_space_type_DeviceN = {
     gs_color_space_index_DeviceN, true, false,
     &st_color_space_DeviceN, gx_num_components_DeviceN,
@@ -62,7 +66,9 @@ const gs_color_space_type gs_color_space_type_DeviceN = {
     gx_concretize_DeviceN, gx_remap_concrete_DeviceN,
     gx_default_remap_color, gx_install_DeviceN,
     gx_set_overprint_DeviceN,
-    gx_adjust_cspace_DeviceN, gx_no_adjust_color_count
+    gx_adjust_cspace_DeviceN, gx_no_adjust_color_count,
+    gx_serialize_DeviceN,
+    gx_cspace_is_linear_default
 };
 
 /* GC procedures */
@@ -173,6 +179,7 @@ alloc_device_n_map(gs_device_n_map ** ppmap, gs_memory_t * mem,
     return 0;
 }
 
+#if 0 /* Unused; Unsupported by gx_serialize_device_n_map. */
 /*
  * Set the DeviceN tint transformation procedure.
  */
@@ -196,6 +203,7 @@ gs_cspace_set_devn_proc(gs_color_space * pcspace,
     pimap->cache_valid = false;
     return 0;
 }
+#endif
 
 /*
  * Check if we are using the alternate color space.
@@ -422,7 +430,7 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_state * pgs)
     uint name_size;
     gs_devicen_color_map * pcolor_component_map
 	= &pgs->color_component_map;
-    const gx_device * dev = pgs->device;
+    gx_device * dev = pgs->device;
     const char none_str[] = "None";
     const uint none_size = strlen(none_str);
     bool non_match = false;
@@ -447,7 +455,7 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_state * pgs)
 	/*
 	 * Get the character string and length for the component name.
 	 */
-	pcs->params.device_n.get_colorname_string(names[i], &pname, &name_size);
+	pcs->params.device_n.get_colorname_string(dev->memory, names[i], &pname, &name_size);
 	/*
          * Postscript does not accept /None as a color component but it is
          * allowed in PDF so we accept it.  It is also accepted as a
@@ -473,7 +481,7 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_state * pgs)
 	     * SeparationOrder list.
 	     */
 	    colorant_number = (*dev_proc(dev, get_color_comp_index))
-				    (dev, (const char *)pname, name_size, i);
+		    (dev, (const char *)pname, name_size, SEPARATION_NAME);
 	    if (colorant_number >= 0) {		/* If valid colorant name */
 		pcolor_component_map->color_map[i] =
 		    (colorant_number == GX_DEVICE_COLOR_MAX_COMPONENTS) ? -1
@@ -498,9 +506,16 @@ gx_install_DeviceN(const gs_color_space * pcs, gs_state * pgs)
     pgs->color_space->params.device_n.use_alt_cspace =
 	using_alt_color_space(pgs);
     if (pgs->color_space->params.device_n.use_alt_cspace)
-        return (*pcs->params.device_n.alt_space.type->install_cspace)
+        code = (*pcs->params.device_n.alt_space.type->install_cspace)
 	((const gs_color_space *) & pcs->params.device_n.alt_space, pgs);
-    return 0;
+    /*
+     * Give the device an opportunity to capture equivalent colors for any
+     * spot colors which might be present in the color space.
+     */
+    if (code >= 0)
+        code = dev_proc(pgs->device, update_spot_equivalent_colors)
+							(pgs->device, pgs);
+    return code;
 }
 
 /* Set overprint information for a DeviceN color space */
@@ -542,3 +557,38 @@ gx_adjust_cspace_DeviceN(const gs_color_space * pcs, int delta)
     (*pcs->params.device_n.alt_space.type->adjust_cspace_count)
 	((const gs_color_space *)&pcs->params.device_n.alt_space, delta);
 }
+
+/* ---------------- Serialization. -------------------------------- */
+
+int 
+gx_serialize_device_n_map(const gs_color_space * pcs, gs_device_n_map * m, stream * s)
+{
+    const gs_function_t *pfn;
+
+    if (m->tint_transform != map_devn_using_function)
+	return_error(gs_error_unregistered); /* Unimplemented. */
+    pfn = (const gs_function_t *)m->tint_transform_data;
+    return gs_function_serialize(pfn, s);
+}
+
+private int 
+gx_serialize_DeviceN(const gs_color_space * pcs, stream * s)
+{
+    const gs_device_n_params * p = &pcs->params.device_n;
+    uint n;
+    int code = gx_serialize_cspace_type(pcs, s);
+
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->num_components, sizeof(p->num_components), &n);
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->names[0], sizeof(p->names[0]) * p->num_components, &n);
+    if (code < 0)
+	return code;
+    code = cs_serialize((const gs_color_space *)&p->alt_space, s);
+    if (code < 0)
+	return code;
+    return gx_serialize_device_n_map(pcs, p->map, s);
+    /* p->use_alt_cspace isn't a property of the space. */
+}
