@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gscie.h,v 1.12 2002/06/16 08:45:42 lpd Exp $ */
+/* $Id: gscie.h 4198 2003-09-09 14:25:10Z igor $ */
 /* Structures for CIE color algorithms */
 /* (requires gscspace.h, gscolor2.h) */
 
@@ -801,4 +801,7 @@ extern int
     gs_cie_defx_set_lookup_table(gs_color_space * pcspace, int *pdims,
 				 const gs_const_string * ptable);
 
+/* Serialize common CIE elements. */
+int gx_serialize_cie_common_elements(const gs_color_space * pcs, stream * s);
+
 #endif /* gscie_INCLUDED */
