@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsdparam.c,v 1.12 2002/11/02 17:22:29 giles Exp $ */
+/* $Id: gsdparam.c 5122 2004-06-29 21:01:41Z dan $ */
 /* Default device parameters for Ghostscript library */
 #include "memory_.h"		/* for memcpy */
 #include "string_.h"		/* for strlen */
@@ -73,7 +73,6 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     /* Standard page device parameters: */
 
-    int mns = 1;
     bool seprs = false;
     gs_param_string dns, pcms;
     gs_param_float_array msa, ibba, hwra, ma;
@@ -85,6 +84,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     /* Non-standard parameters: */
 
     int colors = dev->color_info.num_components;
+    int mns = colors;
     int depth = dev->color_info.depth;
     int GrayValues = dev->color_info.max_gray + 1;
     int HWSize[2];
@@ -169,7 +169,8 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     if (colors > 1) {
 	int RGBValues = dev->color_info.max_color + 1;
-	long ColorValues = (depth >= 32 ? -1 : 1L << depth);
+	long ColorValues = (depth >= (8 * sizeof(gx_color_index)) ? -1
+							: 1L << depth);
 
 	if ((code = param_write_int(plist, "RedValues", &RGBValues)) < 0 ||
 	    (code = param_write_int(plist, "GreenValues", &RGBValues)) < 0 ||
@@ -639,8 +640,7 @@ nce:
 	if ((code = param_check_string(plist, "ProcessColorModel", pcms, (pcms != NULL))) < 0)
 	    ecode = code;
     }
-    if ((code = param_check_int(plist, "MaxSeparations", 1, true)) < 0)
-	ecode = code;
+    IGNORE_INT_PARAM("MaxSeparations")
     if ((code = param_check_bool(plist, "Separations", false, true)) < 0)
 	ecode = code;
 
@@ -662,13 +662,13 @@ nce:
 	ecode = code;
     if ((code = param_check_long(plist, "PageCount", dev->PageCount, true)) < 0)
 	ecode = code;
-    if ((code = param_check_int(plist, "RedValues", RGBValues, colors > 1)) < 0)
+    if ((code = param_check_int(plist, "RedValues", RGBValues, true)) < 0)
 	ecode = code;
-    if ((code = param_check_int(plist, "GreenValues", RGBValues, colors > 1)) < 0)
+    if ((code = param_check_int(plist, "GreenValues", RGBValues, true)) < 0)
 	ecode = code;
-    if ((code = param_check_int(plist, "BlueValues", RGBValues, colors > 1)) < 0)
+    if ((code = param_check_int(plist, "BlueValues", RGBValues, true)) < 0)
 	ecode = code;
-    if ((code = param_check_long(plist, "ColorValues", ColorValues, colors > 1)) < 0)
+    if ((code = param_check_long(plist, "ColorValues", ColorValues, true)) < 0)
 	ecode = code;
     if (param_read_string(plist, "HWColorMap", &cms) != 1) {
 	byte palette[3 << 8];
