@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsexit.h,v 1.8 2002/06/16 08:45:42 lpd Exp $ */
+/* $Id: gsexit.h 5215 2004-08-04 19:36:13Z stefan $ */
 /* Declarations for exits */
 
 #ifndef gsexit_INCLUDED
@@ -25,17 +25,17 @@
  *  normally they do exit cleanup and error messaging
  *  without calling system exit() returning to the caller.
  */
-void gs_to_exit(int exit_status);
+int gs_to_exit(const gs_memory_t *mem, int exit_status);
 
 /** some clients prefer this to return the postscript error code
  * to the caller otherwise the same as gs_to_exit()
  */
-void gs_to_exit_with_code(int exit_status, int code);
+int gs_to_exit_with_code(const gs_memory_t *mem, int exit_status, int code);
 
 /** The client must provide this.  
  * After possible cleanup it may call gp_do_exit() which calls exit() in a platform
  * independent way.  This is a fatal error so returning is not a good idea.
  */
-void gs_abort(void);
+void gs_abort(const gs_memory_t *mem);
 
 #endif /* gsexit_INCLUDED */
