@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsfunc3.c,v 1.13 2002/09/19 18:00:01 ray Exp $ */
+/* $Id: gsfunc3.c 5203 2004-07-28 10:09:20Z igor $ */
 /* Implementation of LL3 Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -23,6 +23,7 @@
 #include "gsfunc3.h"
 #include "gsparam.h"
 #include "gxfunc.h"
+#include "stream.h"
 
 /* ---------------- Utilities ---------------- */
 
@@ -123,33 +124,16 @@ fn_ElIn_evaluate(const gs_function_t * pfn_common, const float *in, float *out)
 /* Test whether an Exponential function is monotonic.  (They always are.) */
 private int
 fn_ElIn_is_monotonic(const gs_function_t * pfn_common,
-		     const float *lower, const float *upper,
-		     gs_function_effort_t effort)
+		     const float *lower, const float *upper)
 {
     const gs_function_ElIn_t *const pfn =
 	(const gs_function_ElIn_t *)pfn_common;
-    int i, result;
 
     if (lower[0] > pfn->params.Domain[1] ||
 	upper[0] < pfn->params.Domain[0]
 	)
 	return_error(gs_error_rangecheck);
-    for (i = 0, result = 0; i < pfn->params.n; ++i) {
-	double diff =
-	    (pfn->params.C1 == 0 ? 1.0 : pfn->params.C1[i]) -
-	    (pfn->params.C0 == 0 ? 0.0 : pfn->params.C0[i]);
-
-	if (pfn->params.N < 0)
-	    diff = -diff;
-	else if (pfn->params.N == 0)
-	    diff = 0;
-	result |=
-	    (diff < 0 ? FN_MONOTONIC_DECREASING :
-	     diff > 0 ? FN_MONOTONIC_INCREASING :
-	     FN_MONOTONIC_DECREASING | FN_MONOTONIC_INCREASING) <<
-	    (2 * i);
-    }
-    return result;
+    return 1;
 }
 
 /* Write Exponential Interpolation function parameters on a parameter list. */
@@ -225,6 +209,25 @@ gs_function_ElIn_free_params(gs_function_ElIn_params_t * params,
     fn_common_free_params((gs_function_params_t *) params, mem);
 }
 
+/* Serialize. */
+private int
+gs_function_ElIn_serialize(const gs_function_t * pfn, stream *s)
+{
+    uint n;
+    const gs_function_ElIn_params_t * p = (const gs_function_ElIn_params_t *)&pfn->params;
+    int code = fn_common_serialize(pfn, s);
+
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->C0[0], sizeof(p->C0[0]) * p->n, &n);
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->C1[0], sizeof(p->C1[0]) * p->n, &n);
+    if (code < 0)
+	return code;
+    return sputs(s, (const byte *)&p->N, sizeof(p->N), &n);
+}
+
 /* Allocate and initialize an Exponential Interpolation function. */
 int
 gs_function_ElIn_init(gs_function_t ** ppfn,
@@ -240,7 +243,8 @@ gs_function_ElIn_init(gs_function_t ** ppfn,
 	    (fn_get_params_proc_t) fn_ElIn_get_params,
 	    (fn_make_scaled_proc_t) fn_ElIn_make_scaled,
 	    (fn_free_params_proc_t) gs_function_ElIn_free_params,
-	    fn_common_free
+	    fn_common_free,
+	    (fn_serialize_proc_t) gs_function_ElIn_serialize,
 	}
     };
     int code;
@@ -271,7 +275,7 @@ gs_function_ElIn_init(gs_function_t ** ppfn,
 	pfn->params.m = 1;
 	pfn->head = function_ElIn_head;
 	pfn->head.is_monotonic =
-	    fn_domain_is_monotonic((gs_function_t *)pfn, EFFORT_MODERATE);
+	    fn_domain_is_monotonic((gs_function_t *)pfn);
 	*ppfn = (gs_function_t *) pfn;
     }
     return 0;
@@ -310,8 +314,11 @@ fn_1ItSg_evaluate(const gs_function_t * pfn_common, const float *in, float *out)
     b0 = (i == 0 ? pfn->params.Domain[0] : pfn->params.Bounds[i - 1]);
     b1 = (i == k - 1 ? pfn->params.Domain[1] : pfn->params.Bounds[i]);
     e0 = pfn->params.Encode[2 * i];
-    encoded =
-	(arg - b0) * (pfn->params.Encode[2 * i + 1] - e0) / (b1 - b0) + e0;
+    if (b1 == b0)
+	encoded = e0;
+    else
+	encoded =
+	    (arg - b0) * (pfn->params.Encode[2 * i + 1] - e0) / (b1 - b0) + e0;
     if_debug3('~', "[~]1ItSg %g in %d => %g\n", arg, i, encoded);
     return gs_function_evaluate(pfn->params.Functions[i], &encoded, out);
 }
@@ -319,8 +326,7 @@ fn_1ItSg_evaluate(const gs_function_t * pfn_common, const float *in, float *out)
 /* Test whether a 1-Input Stitching function is monotonic. */
 private int
 fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
-		      const float *lower, const float *upper,
-		      gs_function_effort_t effort)
+		      const float *lower, const float *upper)
 {
     const gs_function_1ItSg_t *const pfn =
 	(const gs_function_1ItSg_t *)pfn_common;
@@ -328,8 +334,10 @@ fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
     float d0 = pfn->params.Domain[0], d1 = pfn->params.Domain[1];
     int k = pfn->params.k;
     int i;
-    int result = 0;
 
+    if (v0 > v1) {
+	v0 = v1; v1 = lower[0];
+    }
     if (v0 > d1 || v1 < d0)
 	return_error(gs_error_rangecheck);
     if (v0 < d0)
@@ -339,42 +347,37 @@ fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
     for (i = 0; i < pfn->params.k; ++i) {
 	float b0 = (i == 0 ? d0 : pfn->params.Bounds[i - 1]);
 	float b1 = (i == k - 1 ? d1 : pfn->params.Bounds[i]);
+	const float small = 0.0000001 * (b1 - b0);
 	float e0, e1;
 	float w0, w1;
-	int code;
+	float vv0, vv1;
 
-	if (v0 >= b1 || v1 <= b0)
+	if (v0 >= b1)
 	    continue;
+	if (v0 >= b1 - small)
+	    continue; /* Ignore a small noize */
+	vv0 = max(b0, v0);
+	vv1 = v1;
+	if (vv1 > b1 && v1 < b1 + small)
+	    vv1 = b1; /* Ignore a small noize */
+	if (vv0 == vv1)
+	    return 1;
+	if (vv0 < b1 && vv1 > b1)
+	    return 0; /* Consider stitches as monotonity beraks. */
 	e0 = pfn->params.Encode[2 * i];
 	e1 = pfn->params.Encode[2 * i + 1];
-	w0 = (max(v0, b0) - b0) * (e1 - e0) / (b1 - b0) + e0;
-	w1 = (min(v1, b1) - b0) * (e1 - e0) / (b1 - b0) + e0;
+	w0 = (max(vv0, b0) - b0) * (e1 - e0) / (b1 - b0) + e0;
+	w1 = (min(vv1, b1) - b0) * (e1 - e0) / (b1 - b0) + e0;
 	/* Note that w0 > w1 is now possible if e0 > e1. */
-	if (w0 > w1) {
-	    code = gs_function_is_monotonic(pfn->params.Functions[i],
-					    &w1, &w0, effort);
-	    if (code <= 0)
-		return code;
-	    /* Swap the INCREASING and DECREASING flags. */
-	    code = ((code & MASK1) << 1) | ((code & (MASK1 << 1)) >> 1);
-	} else {
-	    code = gs_function_is_monotonic(pfn->params.Functions[i],
-					    &w0, &w1, effort);
-	    if (code <= 0)
-		return code;
-	}
-	if (result == 0)
-	    result = code;
-	else {
-	    result &= code;
-	    /* Check that result is still monotonic in every position. */
-	    code = result | ((result & MASK1) << 1) |
-		((result & (MASK1 << 1)) >> 1);
-	    if (code != (1 << (2 * pfn->params.n)) - 1)
-		return 0;
-	}
+	if (w0 > w1)
+	    return gs_function_is_monotonic(pfn->params.Functions[i],
+					    &w1, &w0);
+	else
+	    return gs_function_is_monotonic(pfn->params.Functions[i],
+					    &w0, &w1);
     }
-    return result;
+    /* v0 is equal to the range end. */
+    return 1; 
 }
 
 /* Return 1-Input Stitching function information. */
@@ -454,6 +457,31 @@ gs_function_1ItSg_free_params(gs_function_1ItSg_params_t * params,
     fn_common_free_params((gs_function_params_t *) params, mem);
 }
 
+/* Serialize. */
+private int
+gs_function_1ItSg_serialize(const gs_function_t * pfn, stream *s)
+{
+    uint n;
+    const gs_function_1ItSg_params_t * p = (const gs_function_1ItSg_params_t *)&pfn->params;
+    int code = fn_common_serialize(pfn, s);
+    int k;
+
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->k, sizeof(p->k), &n);
+    if (code < 0)
+	return code;
+
+    for (k = 0; k < p->k && code >= 0; k++) 
+	code = gs_function_serialize(p->Functions[k], s);
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->Bounds[0], sizeof(p->Bounds[0]) * (p->k - 1), &n);
+    if (code < 0)
+	return code;
+    return sputs(s, (const byte *)&p->Encode[0], sizeof(p->Encode[0]) * (p->k * 2), &n);
+}
+
 /* Allocate and initialize a 1-Input Stitching function. */
 int
 gs_function_1ItSg_init(gs_function_t ** ppfn,
@@ -468,7 +496,8 @@ gs_function_1ItSg_init(gs_function_t ** ppfn,
 	    (fn_get_params_proc_t) fn_1ItSg_get_params,
 	    (fn_make_scaled_proc_t) fn_1ItSg_make_scaled,
 	    (fn_free_params_proc_t) gs_function_1ItSg_free_params,
-	    fn_common_free
+	    fn_common_free,
+	    (fn_serialize_proc_t) gs_function_1ItSg_serialize,
 	}
     };
     int n = (params->Range == 0 ? 0 : params->n);
@@ -507,7 +536,7 @@ gs_function_1ItSg_init(gs_function_t ** ppfn,
 	pfn->params.n = n;
 	pfn->head = function_1ItSg_head;
 	pfn->head.is_monotonic =
-	    fn_domain_is_monotonic((gs_function_t *)pfn, EFFORT_MODERATE);
+	    fn_domain_is_monotonic((gs_function_t *)pfn);
 	*ppfn = (gs_function_t *) pfn;
     }
     return 0;
@@ -558,23 +587,20 @@ fn_AdOt_evaluate(const gs_function_t *pfn_common, const float *in0, float *out)
 /* Test whether an Arrayed Output function is monotonic. */
 private int
 fn_AdOt_is_monotonic(const gs_function_t * pfn_common,
-		     const float *lower, const float *upper,
-		     gs_function_effort_t effort)
+		     const float *lower, const float *upper)
 {
     const gs_function_AdOt_t *const pfn =
 	(const gs_function_AdOt_t *)pfn_common;
-    int i, result;
+    int i;
 
-    for (i = 0, result = 0; i < pfn->params.n; ++i) {
+    for (i = 0; i < pfn->params.n; ++i) {
 	int code =
-	    gs_function_is_monotonic(pfn->params.Functions[i], lower, upper,
-				     effort);
+	    gs_function_is_monotonic(pfn->params.Functions[i], lower, upper);
 
 	if (code <= 0)
 	    return code;
-	result |= code << (2 * i);
     }
-    return result;
+    return 1;
 }
 
 /* Return Arrayed Output function information. */
@@ -625,6 +651,21 @@ gs_function_AdOt_free_params(gs_function_AdOt_params_t * params,
     fn_common_free_params((gs_function_params_t *) params, mem);
 }
 
+/* Serialize. */
+private int
+gs_function_AdOt_serialize(const gs_function_t * pfn, stream *s)
+{
+    const gs_function_AdOt_params_t * p = (const gs_function_AdOt_params_t *)&pfn->params;
+    int code = fn_common_serialize(pfn, s);
+    int k;
+
+    if (code < 0)
+	return code;
+    for (k = 0; k < p->n && code >= 0; k++) 
+	code = gs_function_serialize(p->Functions[k], s);
+    return code;
+}
+
 /* Allocate and initialize an Arrayed Output function. */
 int
 gs_function_AdOt_init(gs_function_t ** ppfn,
@@ -639,7 +680,8 @@ gs_function_AdOt_init(gs_function_t ** ppfn,
 	    fn_common_get_params,	/****** WHAT TO DO ABOUT THIS? ******/
 	    (fn_make_scaled_proc_t) fn_AdOt_make_scaled,
 	    (fn_free_params_proc_t) gs_function_AdOt_free_params,
-	    fn_common_free
+	    fn_common_free,
+	    (fn_serialize_proc_t) gs_function_AdOt_serialize,
 	}
     };
     int m = params->m, n = params->n;
@@ -655,7 +697,7 @@ gs_function_AdOt_init(gs_function_t ** ppfn,
 
 	if (psubfn->params.m != m || psubfn->params.n != 1)
 	    return_error(gs_error_rangecheck);
-	sub_mono = fn_domain_is_monotonic(psubfn, EFFORT_MODERATE);
+	sub_mono = fn_domain_is_monotonic(psubfn);
 	if (i == 0 || sub_mono < 0)
 	    is_monotonic = sub_mono;
 	else if (is_monotonic >= 0)
