@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsfunc4.c,v 1.13.2.1 2003/12/12 22:08:03 giles Exp $ */
+/* $Id: gsfunc4.c 5408 2004-10-07 05:18:34Z ray $ */
 /* Implementation of FunctionType 4 (PostScript Calculator) Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -545,8 +545,7 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 /* Test whether a PostScript Calculator function is monotonic. */
 private int
 fn_PtCr_is_monotonic(const gs_function_t * pfn_common,
-		     const float *lower, const float *upper,
-		     gs_function_effort_t effort)
+		     const float *lower, const float *upper)
 {
     /*
      * No reasonable way to tell.  Eventually we should check for
@@ -690,6 +689,7 @@ fn_PtCr_get_info(const gs_function_t *pfn_common, gs_function_info_t *pfi)
     {
 	stream s;
 
+	s_init(&s, NULL);
 	swrite_position_only(&s);
 	calc_put(&s, pfn);
 	pfi->data_size = stell(&s);
@@ -854,7 +854,7 @@ gs_function_PtCr_init(gs_function_t ** ppfn,
 	pfn->data_source.access = calc_access;
 	pfn->head = function_PtCr_head;
 	pfn->head.is_monotonic =
-	    fn_domain_is_monotonic((gs_function_t *)pfn, EFFORT_MODERATE);
+	    fn_domain_is_monotonic((gs_function_t *)pfn);
 	*ppfn = (gs_function_t *) pfn;
     }
     return 0;
