@@ -14,9 +14,10 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsht.c,v 1.16 2002/10/16 08:46:39 igor Exp $ */
+/*$Id: gsht.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* setscreen operator for Ghostscript library */
 #include "memory_.h"
+#include "string_.h"
 #include <stdlib.h>		/* for qsort */
 #include "gx.h"
 #include "gserrors.h"
@@ -221,15 +222,22 @@ gs_setscreenphase(gs_state * pgs, int x, int y, gs_color_select_t select)
     return code;
 }
 
+int
+gs_currentscreenphase_pis(const gs_imager_state * pis, gs_int_point * pphase,
+		      gs_color_select_t select)
+{
+    if (select < 0 || select >= gs_color_select_count)
+	return_error(gs_error_rangecheck);
+    *pphase = pis->screen_phase[select];
+    return 0;
+}
+
 /* .currentscreenphase */
 int
 gs_currentscreenphase(const gs_state * pgs, gs_int_point * pphase,
 		      gs_color_select_t select)
 {
-    if (select < 0 || select >= gs_color_select_count)
-	return_error(gs_error_rangecheck);
-    *pphase = pgs->screen_phase[select];
-    return 0;
+    return gs_currentscreenphase_pis((const gs_imager_state *)pgs, pphase, select);
 }
 
 /* currenthalftone */
@@ -621,7 +629,11 @@ gx_device_halftone_release(gx_device_halftone * pdht, gs_memory_t * mem)
  *
  * Two other checks are also made.  If the name is "Default" then a value
  * of GX_DEVICE_COLOR_MAX_COMPONENTS is returned.  This is done to
- * simplify the handling of default halftones.
+ * simplify the handling of default halftones.  Note:  The device also
+ * uses GX_DEVICE_COLOR_MAX_COMPONENTS to indicate colorants which are
+ * known but not being used due to the SeparationOrder parameter.  In this
+ * case we return -1 since the colorant is not currently being used by the
+ * device.
  *
  * If the halftone type is colorscreen or multiple colorscreen, then we
  * also check for Red/Cyan, Green/Magenta, Blue/Yellow, and Gray/Black
@@ -631,16 +643,16 @@ gx_device_halftone_release(gx_device_halftone * pdht, gs_memory_t * mem)
  * A negative value is returned if the color name is not found.
  */
 int
-gs_color_name_component_number(const gx_device * dev, const char * pname,
+gs_color_name_component_number(gx_device * dev, const char * pname,
 				int name_size, int halftonetype)
 {
     int num_colorant;
 
 #define check_colorant_name(dev, name) \
-    ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), 0))
+    ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), NO_COMP_NAME_TYPE))
 
 #define check_colorant_name_length(dev, name, length) \
-    ((*dev_proc(dev, get_color_comp_index)) (dev, name, length, 0))
+    ((*dev_proc(dev, get_color_comp_index)) (dev, name, length, NO_COMP_NAME_TYPE))
 
 #define check_name(str, pname, length) \
     ((strlen(str) == length) && (strncmp(pname, str, length) == 0))
@@ -649,8 +661,18 @@ gs_color_name_component_number(const gx_device * dev, const char * pname,
      * Check if this is a device colorant.
      */
     num_colorant = check_colorant_name_length(dev, pname, name_size);
-    if (num_colorant >= 0)
+    if (num_colorant >= 0) {
+	/*
+	 * The device will return GX_DEVICE_COLOR_MAX_COMPONENTS if the
+	 * colorant is logically present in the device but not being used
+	 * because a SeparationOrder parameter is specified.  Since we are
+	 * using this value to indicate 'Default', we use -1 to indicate
+	 * that the colorant is not really being used.
+	 */
+	if (num_colorant == GX_DEVICE_COLOR_MAX_COMPONENTS)
+	    num_colorant = -1;
 	return num_colorant;
+    }
 
     /*
      * Check if this is the default component
@@ -690,7 +712,7 @@ int
 gs_cname_to_colorant_number(gs_state * pgs, byte * pname, uint name_size,
 		int halftonetype)
 {
-    const gx_device * dev = pgs->device;
+    gx_device * dev = pgs->device;
 
     return gs_color_name_component_number(dev, (char *)pname, name_size,
 		    halftonetype);
@@ -940,7 +962,7 @@ gx_imager_dev_ht_install(
     /* construct the new device halftone structure */
     memset(&dht.order, 0, sizeof(dht.order));
     /* the rc field is filled in later */
-    dht.id = gs_next_ids(1);
+    dht.id = gs_next_ids(pis->memory, 1);
     dht.type = type;
     dht.components =  gs_alloc_struct_array(
                           pis->memory,
@@ -1082,7 +1104,7 @@ gx_imager_dev_ht_install(
                     code = gs_error_VMerror;
                 else {
                     porder->cache = pcache;
-                    gx_ht_init_cache(pcache, porder);
+                    gx_ht_init_cache(pis->memory, pcache, porder);
                 }
             }
         }
@@ -1242,7 +1264,7 @@ gx_ht_install(gs_state * pgs, const gs_halftone * pht,
  * A value of -1 indicates that the name is not valid.
  */
 #define check_colorant_name(name, dev) \
-   ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), 0))
+   ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), NO_NAME_TYPE))
 
 /* Reestablish the effective transfer functions, taking into account */
 /* any overrides from halftone dictionaries. */
