@@ -14,9 +14,10 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsht1.c,v 1.13 2002/11/02 07:33:48 raph Exp $ */
+/*$Id: gsht1.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Extended halftone operators for Ghostscript library */
 #include "memory_.h"
+#include "string_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gsstruct.h"
@@ -345,7 +346,7 @@ process_transfer(gx_ht_order * porder, gs_state * pgs,
 		      "process_transfer");
     pmap->proc = proc;		/* 0 => use closure */
     pmap->closure = *pmc;
-    pmap->id = gs_next_ids(1);
+    pmap->id = gs_next_ids(mem, 1);
     load_transfer_map(pgs, pmap, 0.0);
     porder->transfer = pmap;
     return 0;
