@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gshtscr.c,v 1.11 2002/11/02 07:33:48 raph Exp $ */
+/* $Id: gshtscr.c 4342 2003-11-04 01:25:31Z dan $ */
 /* Screen (Type 1) halftone processing for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -363,6 +363,8 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
   try_size:
     better = false;
     {
+        double fm0 = u0 * rt;
+        double fn0 = v0 * rt;
         int m0 = (int)floor(u0 * rt + 0.0001);
         int n0 = (int)floor(v0 * rt + 0.0001);
         gx_ht_cell_params_t p;
@@ -415,17 +417,23 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
                 if_debug5('h', " ==> d=%d, wt=%ld, wt_size=%ld, f=%g, a=%g\n",
                           p.D, wt, bitmap_raster(wt) * wt, ft, at);
 
-                /*
-                 * Minimize angle and frequency error within the
-                 * permitted maximum super-cell size.
-                 */
-
                 {
-                    double err = f_err * a_err;
+                    /*
+		     * Compute the error in position between ideal location.
+		     * and the current integer location.
+		     */
 
-                    if (err > e_best)
+		    double error =
+			(fn0 - p.N) * (fn0 - p.N) + (fm0 - p.M) * (fm0 - p.M);
+		    /*
+		     * Adjust the error by the length of the vector.  This gives
+		     * a slight bias toward larger cell sizzes.
+		     */
+		    error /= p.N * p.N + p.M * p.M;
+		    error = sqrt(error); /* The previous calcs. gave value squared */
+                    if (error > e_best)
                         continue;
-                    e_best = err;
+                    e_best = error;
                 }
                 *phcp = p;
                 f = ft, a = at;
