@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsimage.c,v 1.11 2002/10/10 14:42:50 igor Exp $ */
+/* $Id: gsimage.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Image setup procedures for Ghostscript library */
 #include "memory_.h"
 #include "gx.h"
@@ -219,7 +219,7 @@ gs_image_init(gs_image_enum * penum, const gs_image_t * pim, bool multi,
              * non-current color space is potentially incorrect, but
              * it appears this case doesn't arise.
              */
-            gs_cspace_init_DeviceGray(&cs);
+            gs_cspace_init_DeviceGray(pgs->memory, &cs);
 	    image.ColorSpace = &cs;
         }
     }
