@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsstate.c,v 1.19 2002/11/13 21:51:34 raph Exp $ */
+/* $Id: gsstate.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Miscellaneous graphics state operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -251,7 +251,7 @@ gs_state_alloc(gs_memory_t * mem)
     pgs->effective_clip_path = pgs->clip_path;
     pgs->effective_clip_shared = true;
     /* Initialize things so that gx_remap_color won't crash. */
-    gs_cspace_init_DeviceGray(pgs->color_space);
+    gs_cspace_init_DeviceGray(pgs->memory, pgs->color_space);
     pgs->in_cachedevice = 0;
     gx_set_device_color_1(pgs); /* sets colorspace and client color */
     pgs->device = 0;		/* setting device adjusts refcts */
@@ -282,10 +282,11 @@ fail:
 /* This should only be done to a newly created state. */
 void
 gs_state_set_client(gs_state * pgs, void *pdata,
-		    const gs_state_client_procs * pprocs)
+		    const gs_state_client_procs * pprocs, bool client_has_pattern_streams)
 {
     pgs->client_data = pdata;
     pgs->client_procs = *pprocs;
+    pgs->have_pattern_streams = client_has_pattern_streams;
 }
 
 /* Get the client data from a graphics state. */
@@ -321,6 +322,7 @@ gs_gsave(gs_state * pgs)
      *	rc_increment(pnew->clip_stack);
      */
     pnew->clip_stack = 0;
+    rc_increment(pnew->dfilter_stack);
     pgs->saved = pnew;
     if (pgs->show_gstate == pgs)
 	pgs->show_gstate = pnew->show_gstate = pnew;
@@ -478,6 +480,7 @@ gs_state_copy(gs_state * pgs, gs_memory_t * mem)
     pgs->view_clip = 0;
     pnew = gstate_clone(pgs, mem, "gs_gstate", copy_for_gstate);
     rc_increment(pnew->clip_stack);
+    rc_increment(pnew->dfilter_stack);
     pgs->view_clip = view_clip;
     if (pnew == 0)
 	return 0;
@@ -753,6 +756,20 @@ gs_currentlimitclamp(const gs_state * pgs)
     return pgs->clamp_coordinates;
 }
 
+/* settextrenderingmode */
+void
+gs_settextrenderingmode(gs_state * pgs, uint trm)
+{
+    pgs->text_rendering_mode = trm;
+}
+
+/* currenttextrenderingmode */
+uint
+gs_currenttextrenderingmode(const gs_state * pgs)
+{
+    return pgs->text_rendering_mode;
+}
+
 /* ------ Internal routines ------ */
 
 /* Free the privately allocated parts of a gstate. */
@@ -911,6 +928,7 @@ gstate_free_contents(gs_state * pgs)
 
     rc_decrement(pgs->device, cname);
     rc_decrement(pgs->clip_stack, cname);
+    rc_decrement(pgs->dfilter_stack, cname);
     cs_adjust_counts(pgs, -1);
     if (pgs->client_data != 0)
 	(*pgs->client_procs.free) (pgs->client_data, mem);
@@ -967,6 +985,7 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
     rc_pre_assign(pto->element, pfrom->element, cname)
     RCCOPY(device);
     RCCOPY(clip_stack);
+    RCCOPY(dfilter_stack);
     {
 	struct gx_pattern_cache_s *pcache = pto->pattern_cache;
 	void *pdata = pto->client_data;
@@ -995,3 +1014,9 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
 	(pfrom->show_gstate == pfrom ? pto : 0);
     return 0;
 }
+
+/* Accessories. */
+gs_id gx_get_clip_path_id(gs_state *pgs)
+{
+    return pgs->clip_path->id;
+}
