@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsstruct.h,v 1.13 2002/06/16 08:45:42 lpd Exp $ */
+/* $Id: gsstruct.h 5215 2004-08-04 19:36:13Z stefan $ */
 /* Definitions for Ghostscript modules that define allocatable structures */
 /* Requires gstypes.h */
 
@@ -336,7 +336,7 @@ struct_proc_reloc_ptrs(basic_reloc_ptrs);
      /* Begin enumeration */
 
 #define ENUM_PTRS_BEGIN_PROC(proc)\
-  gs_ptr_type_t proc(EV_CONST void *vptr, uint size, int index, enum_ptr_t *pep, const gs_memory_struct_type_t *pstype, gc_state_t *gcst)
+  gs_ptr_type_t proc(const gs_memory_t *mem, EV_CONST void *vptr, uint size, int index, enum_ptr_t *pep, const gs_memory_struct_type_t *pstype, gc_state_t *gcst)
 #define ENUM_PTRS_BEGIN(proc)\
   ENUM_PTRS_BEGIN_PROC(proc)\
   { switch ( index ) { default:
@@ -374,6 +374,8 @@ extern gs_ptr_type_t
 
 #define ENUM_PTR(i, typ, elt)\
   case i: return ENUM_OBJ_ELT(typ, elt)
+#define ENUM_PTR2(i, typ, e1, e2) /* just an abbreviation */\
+  ENUM_PTR(i, typ, e1); ENUM_PTR((i)+1, typ, e2)
 #define ENUM_PTR3(i, typ, e1, e2, e3) /* just an abbreviation */\
   ENUM_PTR(i, typ, e1); ENUM_PTR((i)+1, typ, e2); ENUM_PTR((i)+2, typ, e3)
 #define ENUM_STRING_PTR(i, typ, elt)\
@@ -434,6 +436,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 
 #define RELOC_PTR(typ, elt)\
   RELOC_OBJ_ELT(typ, elt)
+#define RELOC_PTR2(typ, e1, e2) /* just an abbreviation */\
+  RELOC_PTR(typ,e1); RELOC_PTR(typ,e2)
 #define RELOC_PTR3(typ, e1, e2, e3) /* just an abbreviation */\
   RELOC_PTR(typ,e1); RELOC_PTR(typ,e2); RELOC_PTR(typ,e3)
 #define RELOC_OFFSET_PTR(typ, elt, offset)\
@@ -453,7 +457,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
     /* Subclass support */
 
 #define ENUM_USING(supst, ptr, size, index)\
-  (*(supst).enum_ptrs)(ptr, size, index, pep, &(supst), gcst)
+  (*(supst).enum_ptrs)(mem, ptr, size, index, pep, &(supst), gcst)
 
 #define RELOC_USING(supst, ptr, size)\
   (*(supst).reloc_ptrs)(ptr, size, &(supst), gcst)
@@ -516,7 +520,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
  * Boilerplate for clear_marks procedures.
  */
 #define CLEAR_MARKS_PROC(proc)\
-  void proc(void *vptr, uint size, const gs_memory_struct_type_t *pstype)
+  void proc(const gs_memory_t *cmem, void *vptr, uint size, const gs_memory_struct_type_t *pstype)
 
 	/* Complex structures with their own clear_marks, */
 	/* enum, reloc, and finalize procedures. */
@@ -838,6 +842,19 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add2(stname, stype, sname, penum, preloc, supstname, e1, e2)\
   gs__st_suffix_add2(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2)
 
+	/* Suffix subclasses with 2 additional pointers and 1 string. */
+
+#define gs__st_suffix_add2_string1(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT2(stype, e1, e2),\
+    GC_STRING_ELT(stype, e3)\
+  };\
+  gs__st_basic_super(scope_st, stname, stype, sname, penum, preloc, &supstname, 0)
+#define gs_public_st_suffix_add2_string1(stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
+  gs__st_suffix_add2_string1(public_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)
+#define gs_private_st_suffix_add2_string1(stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
+  gs__st_suffix_add2_string1(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)
+
 	/* Suffix subclasses with 2 additional pointers and finalization. */
 	/* See above regarding finalization and subclasses. */
 
@@ -863,6 +880,19 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add3(stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
   gs__st_suffix_add3(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)
 
+	/* Suffix subclasses with 3 additional pointers and 1 string. */
+
+#define gs__st_suffix_add3_string1(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT3(stype, e1, e2, e3),\
+    GC_STRING_ELT(stype, e4)\
+  };\
+  gs__st_basic_super(scope_st, stname, stype, sname, penum, preloc, &supstname, 0)
+#define gs_public_st_suffix_add3_string1(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
+  gs__st_suffix_add3_string1(public_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)
+#define gs_private_st_suffix_add3_string1(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
+  gs__st_suffix_add3_string1(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)
+
 	/* Suffix subclasses with 3 additional pointers and finalization. */
 	/* See above regarding finalization and subclasses. */
 
@@ -977,6 +1007,19 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add10(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)\
   gs__st_suffix_add10(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)
 
+	/* Suffix subclasses with 11 additional pointers. */
+
+#define gs__st_suffix_add11(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT3(stype, e1, e2, e3), GC_OBJ_ELT3(stype, e4, e5, e6),\
+    GC_OBJ_ELT3(stype, e7, e8, e9), GC_OBJ_ELT2(stype, e10, e11)\
+  };\
+  gs__st_basic_super(scope_st, stname, stype, sname, penum, preloc, &supstname, 0)
+#define gs_public_st_suffix_add11(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
+  gs__st_suffix_add11(public_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)
+#define gs_private_st_suffix_add11(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
+  gs__st_suffix_add11(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)
+
 /* ---------------- General subclasses ---------------- */
 
 	/* General subclasses with no additional pointers. */
