@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gstype42.c,v 1.30.2.3 2004/04/03 17:37:53 igor Exp $ */
+/* $Id: gstype42.c 5515 2004-11-19 04:39:11Z ray $ */
 /* Type 42 (TrueType) font library routines */
 #include "memory_.h"
 #include "gx.h"
@@ -39,14 +39,9 @@
 public_st_gs_font_type42();
 
 /* Forward references */
-#if NEW_TT_INTERPRETER
 private int append_outline_fitted(uint glyph_index, const gs_matrix * pmat,
 	       gx_path * ppath, cached_fm_pair * pair, 
 	       const gs_log2_scale_point * pscale, bool design_grid);
-#else
-private int append_outline(uint glyph_index, const gs_matrix_fixed * pmat,
-			   gx_path * ppath, gs_font_type42 * pfont);
-#endif
 private uint default_get_glyph_index(gs_font_type42 *pfont, gs_glyph glyph);
 private int default_get_outline(gs_font_type42 *pfont, uint glyph_index,
 				gs_glyph_data_t *pgd);
@@ -167,9 +162,8 @@ gs_type42_font_init(gs_font_type42 * pfont)
 	pfont->FontBBox.q.x = S16(head_box + 4) / upem;
 	pfont->FontBBox.q.y = S16(head_box + 6) / upem;
     }
-#if NEW_TT_INTERPRETER
     pfont->data.warning_patented = false;
-#endif
+    pfont->data.warning_bad_instruction = false;
     pfont->data.get_glyph_index = default_get_glyph_index;
     pfont->data.get_outline = default_get_outline;
     pfont->data.get_metrics = gs_type42_default_get_metrics;
@@ -266,14 +260,17 @@ total_points(gs_font_type42 *pfont, uint glyph_index)
 {
     gs_glyph_data_t glyph_data;
     int code;
-    int ocode = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
-    const byte *gdata = glyph_data.bits.data;
+    int ocode;
+    const byte *gdata;
     int total;
 
+    glyph_data.memory = pfont->memory;
+    ocode = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
     if (ocode < 0)
 	return ocode;
     if (glyph_data.bits.size == 0)
 	return 0;
+    gdata = glyph_data.bits.data;
     if (S16(gdata) != -1) {
 	/* This is a simple glyph. */
 	int numContours = S16(gdata);
@@ -366,7 +363,7 @@ default_get_outline(gs_font_type42 * pfont, uint glyph_index,
 	    uint left = glyph_length;
 
 	    /* 'code' is the returned length */
-	    buf = (byte *)gs_alloc_string(pfont->memory, glyph_length, "default_get_outline");
+	    buf = (byte *)gs_alloc_string(pgd->memory, glyph_length, "default_get_outline");
 	    if (buf == 0)
 		return_error(gs_error_VMerror);
 	    gs_glyph_data_from_string(pgd, buf, glyph_length, (gs_font *)pfont);
@@ -419,7 +416,7 @@ gs_type42_get_outline_from_TT_file(gs_font_type42 * pfont, stream *s, uint glyph
 	byte *buf;
 
 	sseek(s, pfont->data.glyf + glyph_start);
-	buf = (byte *)gs_alloc_string(pfont->memory, glyph_length, "default_get_outline");
+	buf = (byte *)gs_alloc_string(pgd->memory, glyph_length, "default_get_outline");
 	if (buf == 0)
 	    return_error(gs_error_VMerror);
 	gs_glyph_data_from_string(pgd, buf, glyph_length, (gs_font *)pfont);
@@ -439,8 +436,10 @@ parse_pieces(gs_font_type42 *pfont, gs_glyph glyph, gs_glyph *pieces,
 			? glyph - GS_MIN_GLYPH_INDEX 
 			: pfont->data.get_glyph_index(pfont, glyph));
     gs_glyph_data_t glyph_data;
-    int code = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
+    int code;
 
+    glyph_data.memory = pfont->memory;
+    code = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
     if (code < 0)
 	return code;
     if (glyph_data.bits.size != 0 && S16(glyph_data.bits.data) == -1) {
@@ -466,7 +465,7 @@ parse_pieces(gs_font_type42 *pfont, gs_glyph glyph, gs_glyph *pieces,
 /* Define the font procedures for a Type 42 font. */
 int
 gs_type42_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,
-			gx_path *ppath)
+			gx_path *ppath, double sbw[4])
 {
     gs_font_type42 *const pfont = (gs_font_type42 *)font;
     uint glyph_index = (glyph >= GS_MIN_GLYPH_INDEX 
@@ -475,11 +474,7 @@ gs_type42_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matri
     gs_fixed_point origin;
     int code;
     gs_glyph_info_t info;
-#if !NEW_TT_INTERPRETER
-    gs_matrix_fixed fmat;
-#endif
     static const gs_matrix imat = { identity_matrix_body };
-#if NEW_TT_INTERPRETER
     bool design_grid = true;
     const gs_log2_scale_point log2_scale = {0, 0}; 
     /* fixme : The subpixel numbers doesn't pass through the font_proc_glyph_outline interface.
@@ -493,19 +488,11 @@ gs_type42_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matri
 
     if (code < 0)
 	return code;
-#endif
-
     if (pmat == 0)
 	pmat = &imat;
     if ((code = gx_path_current_point(ppath, &origin)) < 0 ||
-#if NEW_TT_INTERPRETER
 	(code = append_outline_fitted(glyph_index, pmat, ppath, pair, 
 					&log2_scale, design_grid)) < 0 ||
-#else
-	(code = gs_matrix_fixed_from_matrix(&fmat, pmat)) < 0 ||
-	(code = append_outline(glyph_index, &fmat, ppath, pfont)) < 0 ||
-	/* fixme : don't call glyph_info. */
-#endif
 	(code = font->procs.glyph_info(font, glyph, pmat,
 				       GLYPH_INFO_WIDTH0 << WMode, &info)) < 0
 	)
@@ -527,6 +514,7 @@ gs_type42_glyph_info_by_gid(gs_font *font, gs_glyph glyph, const gs_matrix *pmat
     gs_glyph_data_t outline;
     int code = 0;
 
+    outline.memory = pfont->memory;
     if (default_members) {
 	code = gs_default_glyph_info(font, glyph, pmat, default_members, info);
 
@@ -603,8 +591,10 @@ gs_type42_enumerate_glyph(gs_font *font, int *pindex,
     while (++*pindex <= pfont->data.numGlyphs) {
 	gs_glyph_data_t outline;
 	uint glyph_index = *pindex - 1;
-	int code = pfont->data.get_outline(pfont, glyph_index, &outline);
+	int code;
 
+	outline.memory = pfont->memory;
+	code = pfont->data.get_outline(pfont, glyph_index, &outline);
 	if (code < 0)
 	    return code;
 	if (outline.bits.data == 0)
@@ -671,9 +661,11 @@ gs_type42_default_get_metrics(gs_font_type42 * pfont, uint glyph_index,
 			      int wmode, float sbw[4])
 {
     gs_glyph_data_t glyph_data;
-    int code = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
+    int code;
     int result;
 
+    glyph_data.memory = pfont->memory;
+    code = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
     if (code < 0)
 	return code;
     if (glyph_data.bits.size != 0 && S16(glyph_data.bits.data) == -1) {
@@ -724,7 +716,6 @@ gs_type42_get_metrics(gs_font_type42 * pfont, uint glyph_index,
 
 /* Append a TrueType outline to a path. */
 /* Note that this does not append the final moveto for the width. */
-#if NEW_TT_INTERPRETER
 int
 gs_type42_append(uint glyph_index, gs_imager_state * pis,
 		 gx_path * ppath, const gs_log2_scale_point * pscale,
@@ -735,25 +726,12 @@ gs_type42_append(uint glyph_index, gs_imager_state * pis,
 
     if (code < 0)
 	return code;
-    /* Set the flatness for curve rendering. */
-    return gs_imager_setflat(pis, gs_char_flatness(pis, 1.0));
-}
-
-#else
-int
-gs_type42_append(uint glyph_index, gs_imager_state * pis,
-		 gx_path * ppath, const gs_log2_scale_point * pscale,
-		 bool charpath_flag, int paint_type, gs_font_type42 * pfont)
-{
-    int code = append_outline(glyph_index, &pis->ctm, ppath, pfont);
-
+    code = gx_setcurrentpoint_from_path(pis, ppath);
     if (code < 0)
 	return code;
     /* Set the flatness for curve rendering. */
     return gs_imager_setflat(pis, gs_char_flatness(pis, 1.0));
 }
-#endif
-
 
 /* Add 2nd degree Bezier to the path */
 private int
@@ -764,7 +742,6 @@ add_quadratic_curve(gx_path * const ppath, const gs_fixed_point * const a,
 	(c->x + 2*b->x)/3, (c->y + 2*b->y)/3, c->x, c->y);
 }
 
-
 /*
  * Append a simple glyph outline to a path (ppath != 0) and/or return
  * its list of points (ppts != 0).
@@ -974,6 +951,7 @@ check_component(uint glyph_index, const gs_matrix_fixed *pmat,
     int numContours;
     int code;
 
+    glyph_data.memory = pfont->memory;
     code = pfont->data.get_outline(pfont, glyph_index, &glyph_data);
     if (code < 0)
 	return code;
@@ -1000,6 +978,7 @@ append_component(uint glyph_index, const gs_matrix_fixed * pmat,
     gs_glyph_data_t glyph_data;
     int code;
 
+    glyph_data.memory = pfont->memory;
     code = check_component(glyph_index, pmat, ppath, pfont, ppts + point_index,
 			   &glyph_data, subglyph);
     if (code != 1)
@@ -1051,51 +1030,6 @@ append_component(uint glyph_index, const gs_matrix_fixed * pmat,
     return code;
 }
 
-#if !NEW_TT_INTERPRETER
-private int
-append_outline(uint glyph_index, const gs_matrix_fixed * pmat,
-	       gx_path * ppath, gs_font_type42 * pfont)
-{
-    gs_glyph_data_t glyph_data;
-    int code =
-	check_component(glyph_index, pmat, ppath, pfont, NULL, &glyph_data, false);
-
-    if (code != 1)
-	return code;
-    {
-	/*
-	 * Set up the points array (only needed for point matching, sigh).
-	 * We use stack allocation if possible, to avoid creating a sandbar
-	 * (pts will be allocated before, but also freed before, any path
-	 * elements).
-	 */
-#define MAX_STACK_PTS 150	/* usually enough */
-	int num_points = total_points(pfont, glyph_index);
-
-	if (num_points <= MAX_STACK_PTS) {
-	    gs_fixed_point pts[MAX_STACK_PTS];
-
-	    code = append_component(glyph_index, pmat, ppath, pts, 0, pfont, false);
-	} else {
-	    gs_memory_t *mem = pfont->memory; /* any memory will do */
-	    gs_fixed_point *ppts = (gs_fixed_point *)
-		gs_alloc_byte_array(mem, num_points, sizeof(gs_fixed_point),
-				    "append_outline");
-
-	    if (ppts == 0)
-		code = gs_note_error(gs_error_VMerror);
-	    else {
-		code = append_component(glyph_index, pmat, ppath, ppts, 0,
-					pfont, false);
-		gs_free_object(mem, ppts, "append_outline");
-	    }
-	}
-#undef MAX_STACK_PTS
-    }
-    gs_glyph_data_free(&glyph_data, "append_outline");
-    return code;
-}
-#else
 private int
 append_outline_fitted(uint glyph_index, const gs_matrix * pmat,
 	       gx_path * ppath, cached_fm_pair * pair, 
@@ -1110,5 +1044,4 @@ append_outline_fitted(uint glyph_index, const gs_matrix * pmat,
     gx_ttfReader__set_font(pair->ttr, NULL);
     return code;
 }
-#endif
 
