@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxcht.c,v 1.11 2002/11/05 01:03:14 dan Exp $ */
+/*$Id: gxcht.c 5567 2004-12-08 21:35:13Z stefan $ */
 /* Color halftone rendering for Ghostscript imaging library */
 #include <assert.h>
 #include "memory_.h"
@@ -382,8 +382,9 @@ gx_dc_ht_colored_read(
      * exhausted the buffer. This should not cause a problem in
      * practice.
      */
-    if (--size < 0)
+    if (size == 0)
         return_error(gs_error_rangecheck);
+    size--;
     flag_bits = *pdata++;
 
     /* read the other components provided */
@@ -393,15 +394,17 @@ gx_dc_ht_colored_read(
             int             num_bytes = (num_comps + 7) >> 3;
             int             i, shift = 0;
 
-            if ((size -= num_bytes) < 0)
+            if (size < num_bytes)
                 return_error(gs_error_rangecheck);
+	    size -= num_bytes;
             for (i = 0; i < num_bytes; i++, shift += 8)
                 base_mask |= (gx_color_index)(*pdata++) << shift;
             for (i = 0; i < num_comps; i++, base_mask >>= 1)
                 devc.colors.colored.c_base[i] = base_mask & 0x1;
         } else {
-            if ((size -= num_comps) < 0)
+            if (size < num_comps)
                 return_error(gs_error_rangecheck);
+	    size -= num_comps;
             memcpy(devc.colors.colored.c_base, pdata, num_comps);
             pdata += num_comps;
         }
@@ -666,7 +669,7 @@ gx_dc_ht_colored_fill_rectangle(const gx_device_color * pdevc,
 	    tiles.raster = raster;
 	    tiles.rep_width = tiles.size.x = lw;
 	    tiles.rep_height = tiles.size.y = lh;
-	    tiles.id = gs_next_ids(1);
+	    tiles.id = gs_next_ids(dev->memory, 1);
 	    tiles.rep_shift = tiles.shift = 0;
 	    set_color_ht((byte *)tbits, raster, 0, 0, lw, lh, depth,
 			 special, nplanes, pdevc->colors.colored.plane_mask,
