@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxclip.c,v 1.9 2002/08/22 07:12:29 henrys Exp $ */
+/* $Id: gxclip.c 5110 2004-06-24 05:03:36Z dan $ */
 /* Implementation of (path-based) clipping */
 #include "gx.h"
 #include "gxdevice.h"
@@ -97,7 +97,14 @@ private const gx_device_clip gs_clip_device =
   gx_forward_get_color_mapping_procs,
   gx_forward_get_color_comp_index,
   gx_forward_encode_color,
-  gx_forward_decode_color
+  gx_forward_decode_color,
+  gx_forward_pattern_manage,
+  gx_forward_fill_rectangle_hl_color,
+  gx_forward_include_color_space,
+  gx_default_fill_linear_color_scanline,
+  gx_default_fill_linear_color_trapezoid,
+  gx_default_fill_linear_color_triangle,
+  gx_forward_update_spot_equivalent_colors
  }
 };
 
