@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxclist.c,v 1.9 2002/09/13 22:21:22 dan Exp $ */
+/*$Id: gxclist.c 5110 2004-06-24 05:03:36Z dan $ */
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -131,7 +131,14 @@ const gx_device_procs gs_clist_device_procs = {
     gx_forward_get_color_mapping_procs,
     gx_forward_get_color_comp_index,
     gx_forward_encode_color,
-    gx_forward_decode_color
+    gx_forward_decode_color,
+    gx_default_pattern_manage,
+    gx_default_fill_rectangle_hl_color,
+    gx_default_include_color_space,
+    gx_default_fill_linear_color_scanline,
+    gx_default_fill_linear_color_trapezoid, /* fixme : write to clist. */
+    gx_default_fill_linear_color_triangle,
+    gx_forward_update_spot_equivalent_colors
 };
 
 /* ------ Define the command set and syntax ------ */
