@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxclrast.c,v 1.26 2002/10/28 19:17:19 giles Exp $ */
+/*$Id: gxclrast.c 5408 2004-10-07 05:18:34Z ray $ */
 /* Command list interpreter/rasterizer */
 #include "memory_.h"
 #include "gx.h"
@@ -139,7 +139,7 @@ top_up_cbuf(command_buf_t *pcb, const byte *cbp)
 	nread = 1;
     }
     set_cb_end(pcb, cb_top + nread);
-    process_interrupts();
+    process_interrupts(pcb->s->memory);
     return pcb->data;
 }
 
@@ -350,7 +350,7 @@ in:				/* Initialize for a new page. */
     gx_imager_setscreenphase(&imager_state, -x0, -y0, gs_color_select_all);
     halftone_type = ht_type_none;
     fill_params.fill_zero_width = false;
-    gs_cspace_init_DeviceGray(&cs);
+    gs_cspace_init_DeviceGray(mem, &cs);
     pcs = &cs;
     color_unset(&dev_color);
     color_space.params.indexed.use_proc = 0;
@@ -363,8 +363,10 @@ in:				/* Initialize for a new page. */
     }
     while (code >= 0) {
 	int op;
-	int compress, depth, raster;
-	byte *source;
+	int compress;
+	int depth = 0x7badf00d; /* Initialize against indeterminizm. */
+	int raster = 0x7badf00d; /* Initialize against indeterminizm. */
+	byte *source = NULL;  /* Initialize against indeterminizm. */
 	gx_color_index colors[2];
 	gx_color_index *pcolor;
 	gs_logical_operation_t log_op;
@@ -1059,7 +1061,7 @@ ibegin:			if_debug0('L', "\n");
 			{
 			    uint flags;
 			    int plane;
-			    uint raster;
+			    uint raster1 = 0xbaadf00d; /* Initialize against indeterminizm. */
 
 			    cmd_getw(flags, cbp);
 			    for (plane = 0;
@@ -1071,10 +1073,10 @@ ibegin:			if_debug0('L', "\n");
 					2 * cmd_max_intsize(sizeof(uint)))
 					cbp = top_up_cbuf(&cbuf, cbp);
 				    cmd_getw(planes[plane].raster, cbp);
-				    if ((raster = planes[plane].raster) != 0)
+				    if ((raster1 = planes[plane].raster) != 0)
 					cmd_getw(data_x, cbp);
 				} else {
-				    planes[plane].raster = raster;
+				    planes[plane].raster = raster1;
 				}
 				planes[plane].data_x = data_x;
 			    }
@@ -1860,15 +1862,15 @@ read_set_color_space(command_buf_t *pcb, gs_imager_state *pis,
 	      (b & 4 ? "(proc)" : ""));
     switch (index) {
     case gs_color_space_index_DeviceGray:
-        gs_cspace_init_DeviceGray(&gray_cs);
+        gs_cspace_init_DeviceGray(mem, &gray_cs);
         pcs = &gray_cs;
 	break;
     case gs_color_space_index_DeviceRGB:
-        gs_cspace_init_DeviceRGB(&rgb_cs);
+        gs_cspace_init_DeviceRGB(mem, &rgb_cs);
         pcs = &rgb_cs;
 	break;
     case gs_color_space_index_DeviceCMYK:
-        gs_cspace_init_DeviceCMYK(&cmyk_cs);
+        gs_cspace_init_DeviceCMYK(mem, &cmyk_cs);
         pcs = &cmyk_cs;
 	break;
     default:
@@ -1950,6 +1952,7 @@ read_begin_image(command_buf_t *pcb, gs_image_common_t *pic,
 
     /* This is sloppy, but we don't have enough information to do better. */
     pcb->ptr = top_up_cbuf(pcb, pcb->ptr);
+    s_init(&s, NULL);
     sread_string(&s, pcb->ptr, pcb->end - pcb->ptr);
     code = image_type->sget(pic, &s, pcs);
     pcb->ptr = sbufptr(&s);
@@ -2076,9 +2079,10 @@ read_create_compositor(
      * change the target device.
      */
     code = dev_proc(tdev, create_compositor)(tdev, &tdev, pcomp, pis, mem);
-    if (code >= 0 && tdev != *ptarget)
+    if (code >= 0 && tdev != *ptarget) {
+        rc_increment(tdev);
         *ptarget = tdev;
-
+    }
     /* free the compositor object */
     if (pcomp != 0)
         gs_free_object(mem, pcomp, "read_create_compositor");
@@ -2147,6 +2151,7 @@ cmd_read_matrix(gs_matrix * pmat, const byte * cbp)
 {
     stream s;
 
+    s_init(&s, NULL);
     sread_string(&s, cbp, 1 + sizeof(*pmat));
     sget_matrix(&s, pmat);
     return cbp + stell(&s);
