@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxclrect.c,v 1.5 2002/09/13 22:21:22 dan Exp $ */
+/* $Id: gxclrect.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Rectangle-oriented command writing for command list */
 #include "gx.h"
 #include "gserrors.h"
@@ -583,8 +583,10 @@ clist_strip_copy_rop(gx_device * dev,
 	gx_color_index D = pcls->colors_used.or;
 	int code;
 
+	/* Reducing D, S, T to rop_operand (which apparently is 32 bit) appears safe
+	   due to 'all' a has smaller snumber of significant bits. */
 	pcls->colors_used.or =
-	    ((rop_proc_table[color_rop])(D, S, T) & all) | D;
+	    ((rop_proc_table[color_rop])((rop_operand)D, (rop_operand)S, (rop_operand)T) & all) | D;
 	pcls->colors_used.slow_rop |= slow_rop;
 	if (rop3_uses_T(rop)) {
 	    if (tcolors == 0 || tcolors[0] != tcolors[1]) {
@@ -594,7 +596,7 @@ clist_strip_copy_rop(gx_device * dev,
 		    /* Change tile.  If there is no id, generate one. */
 		    if (tiles->id == gx_no_bitmap_id) {
 			tile_with_id = *tiles;
-			tile_with_id.id = gs_next_ids(1);
+			tile_with_id.id = gs_next_ids(dev->memory, 1);
 			tiles = &tile_with_id;
 		    }
 		    TRY_RECT {
@@ -625,7 +627,7 @@ clist_strip_copy_rop(gx_device * dev,
 			 * Allocate enough fake IDs, since the inner call on
 			 * clist_strip_copy_rop will need them anyway.
 			 */
-			ids = gs_next_ids(min(height, rep_height));
+			ids = gs_next_ids(dev->memory, min(height, rep_height));
 			line_tile = *tiles;
 			line_tile.size.y = 1;
 			line_tile.rep_height = 1;
