@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxcmap.c,v 1.13 2002/11/06 23:11:17 ray Exp $ */
+/* $Id: gxcmap.c 5399 2004-10-01 23:35:02Z ghostgum $ */
 /* Color mapping for Ghostscript */
 #include "gx.h"
 #include "gserrors.h"
@@ -32,13 +32,12 @@
 #include "gxcdevn.h"
 #include "string_.h"
 
-
 /* Structure descriptor */
 public_st_device_color();
 private 
 ENUM_PTRS_WITH(device_color_enum_ptrs, gx_device_color *cptr)
 {
-    return ENUM_USING(*cptr->type->stype, vptr, size, index);
+	return ENUM_USING(*cptr->type->stype, vptr, size, index);
 }
 ENUM_PTRS_END
 private RELOC_PTRS_WITH(device_color_reloc_ptrs, gx_device_color *cptr)
@@ -51,8 +50,9 @@ gx_color_index
 gx_default_encode_color(gx_device * dev, const gx_color_value cv[])
 {
     int             ncomps = dev->color_info.num_components;
-    int             i, i_gray = dev->color_info.gray_index;
+    int             i;
     const byte *    comp_shift = dev->color_info.comp_shift;
+    const byte *    comp_bits = dev->color_info.comp_bits;
     gx_color_index  color = 0;
 
 #ifdef DEBUG
@@ -62,22 +62,26 @@ gx_default_encode_color(gx_device * dev, const gx_color_value cv[])
     }
 #endif
     for (i = 0; i < ncomps; i++) {
-        ulong   cbits = cv[i] * ((i == i_gray ?
-                                 dev->color_info.max_gray :
-                                 dev->color_info.max_color ) + 1);
+	color |= (gx_color_index)(cv[i] >> (gx_color_value_bits - comp_bits[i]))
+		<< comp_shift[i];
 
-        color |= (cbits / (gx_max_color_value + 1)) << comp_shift[i];
     }
     return color;
 }
 
+/* 
+ * This routine is only used if the device is 'separable'.  See
+ * separable_and_linear in gxdevcli.h for more information.
+ */
 int
 gx_default_decode_color(gx_device * dev, gx_color_index color, gx_color_value cv[])
 {
     int                     ncomps = dev->color_info.num_components;
     int                     i;
     const byte *            comp_shift = dev->color_info.comp_shift;
+    const byte *            comp_bits = dev->color_info.comp_bits;
     const gx_color_index *  comp_mask = dev->color_info.comp_mask;
+    uint shift, ivalue, nbits, scale;
 
 #ifdef DEBUG
     if ( dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN ) {
@@ -87,12 +91,23 @@ gx_default_decode_color(gx_device * dev, gx_color_index color, gx_color_value cv
 #endif
 
     for (i = 0; i < ncomps; i++) {
-        gx_color_index  div = ( i == dev->color_info.gray_index ? 
-                                dev->color_info.max_gray : 
-                                dev->color_info.max_color ) + 1;
-
-        cv[i] = (gx_color_value)(((color & comp_mask[i]) >> comp_shift[i]) * 
-            (gx_max_color_value + 1) / div);
+	/*
+	 * Convert from the gx_color_index bits to a gx_color_value.
+	 * Split the conversion into an integer and a fraction calculation
+	 * so we can do integer arthmetic.  The calculation is equivalent
+	 * to floor(0xffff.fffff * ivalue / ((1 << nbits) - 1))
+	 */
+	nbits = comp_bits[i];
+	scale = gx_max_color_value / ((1 << nbits) - 1);
+	ivalue = (color & comp_mask[i]) >> comp_shift[i];
+	cv[i] = ivalue * scale;
+	/*
+	 * Since our scaling factor is an integer, we lost the fraction.
+	 * Determine what part of the ivalue that the faction would have 
+	 * added into the result.
+	 */
+	shift = nbits - (gx_color_value_bits % nbits);
+	cv[i] += ivalue >> shift;
     }
     return 0;
 }
@@ -193,7 +208,7 @@ static void
 cmyk_cs_to_gray_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
 {
     out[0] = color_cmyk_to_gray(c, m, y, k, NULL);
-};
+}
 
 static void
 gray_cs_to_rgb_cm(gx_device * dev, frac gray, frac out[])
@@ -214,7 +229,44 @@ static void
 cmyk_cs_to_rgb_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
 {
     color_cmyk_to_rgb(c, m, y, k, NULL, out);
-};
+}
+
+static void
+gray_cs_to_rgbk_cm(gx_device * dev, frac gray, frac out[])
+{
+    out[0] = out[1] = out[2] = frac_0;
+    out[3] = gray;
+}
+
+static void
+rgb_cs_to_rgbk_cm(gx_device * dev, const gs_imager_state *pis,
+				  frac r, frac g, frac b, frac out[])
+{
+    if ((r == g) && (g == b)) {
+	out[0] = out[1] = out[2] = frac_0;
+	out[3] = r;
+    }
+    else {
+	out[0] = r;
+	out[1] = g;
+	out[2] = b;
+	out[3] = frac_0;
+    }
+}
+
+static void
+cmyk_cs_to_rgbk_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
+{
+    frac rgb[3];
+    if ((c == frac_0) && (m == frac_0) && (y == frac_0)) {
+	out[0] = out[1] = out[2] = frac_0;
+	out[3] = frac_1 - k;
+    }
+    else {
+	color_cmyk_to_rgb(c, m, y, k, NULL, rgb);
+	rgb_cs_to_rgbk_cm(dev, NULL, rgb[0], rgb[1], rgb[2], out);
+    }
+}
 
 static void
 gray_cs_to_cmyk_cm(gx_device * dev, frac gray, frac out[])
@@ -262,7 +314,7 @@ cmyk_cs_to_cmyk_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
     out[1] = m;
     out[2] = y;
     out[3] = k;
-};
+}
 
 
 /* The list of default color space to color model conversion routines. */
@@ -279,6 +331,10 @@ static const gx_cm_color_map_procs DeviceCMYK_procs = {
     gray_cs_to_cmyk_cm, rgb_cs_to_cmyk_cm, cmyk_cs_to_cmyk_cm
 };
 
+static const gx_cm_color_map_procs DeviceRGBK_procs = {
+    gray_cs_to_rgbk_cm, rgb_cs_to_rgbk_cm, cmyk_cs_to_rgbk_cm
+};
+
 /*
  * These are the default handlers for returning the list of color space
  * to color model conversion routines.
@@ -301,6 +357,12 @@ gx_default_DevCMYK_get_color_mapping_procs(const gx_device * dev)
     return &DeviceCMYK_procs;
 }
 
+const gx_cm_color_map_procs *
+gx_default_DevRGBK_get_color_mapping_procs(const gx_device * dev)
+{
+    return &DeviceRGBK_procs;
+}
+
 const gx_cm_color_map_procs *
 gx_error_get_color_mapping_procs(const gx_device * dev)
 {
@@ -321,8 +383,8 @@ gx_error_get_color_mapping_procs(const gx_device * dev)
 
 /* Default color component to index for a DeviceGray color model */
 int
-gx_default_DevGray_get_color_comp_index(const gx_device * dev, const char * pname,
-						int name_size, int src_index)
+gx_default_DevGray_get_color_comp_index(gx_device * dev, const char * pname,
+					  int name_size, int component_type)
 {
     if (compare_color_names(pname, name_size, "Gray") ||
 	compare_color_names(pname, name_size, "Grey")) 
@@ -333,8 +395,8 @@ gx_default_DevGray_get_color_comp_index(const gx_device * dev, const char * pnam
 
 /* Default color component to index for a DeviceRGB color model */
 int
-gx_default_DevRGB_get_color_comp_index(const gx_device * dev, const char * pname,
-						int name_size, int src_index)
+gx_default_DevRGB_get_color_comp_index(gx_device * dev, const char * pname,
+					   int name_size, int component_type)
 {
     if (compare_color_names(pname, name_size, "Red"))
         return 0;
@@ -348,8 +410,8 @@ gx_default_DevRGB_get_color_comp_index(const gx_device * dev, const char * pname
 
 /* Default color component to index for a DeviceCMYK color model */
 int
-gx_default_DevCMYK_get_color_comp_index(const gx_device * dev, const char * pname,
-						int name_size, int src_index)
+gx_default_DevCMYK_get_color_comp_index(gx_device * dev, const char * pname,
+					    int name_size, int component_type)
 {
     if (compare_color_names(pname, name_size, "Cyan"))
         return 0;
@@ -363,10 +425,27 @@ gx_default_DevCMYK_get_color_comp_index(const gx_device * dev, const char * pnam
         return -1;		    /* Indicate that the component name is "unknown" */
 }
 
+/* Default color component to index for a DeviceRGBK color model */
+int
+gx_default_DevRGBK_get_color_comp_index(gx_device * dev, const char * pname,
+					    int name_size, int component_type)
+{
+    if (compare_color_names(pname, name_size, "Red"))
+        return 0;
+    if (compare_color_names(pname, name_size, "Green"))
+        return 1;
+    if (compare_color_names(pname, name_size, "Blue"))
+        return 2;
+    if (compare_color_names(pname, name_size, "Black"))
+        return 3;
+    else
+        return -1;		    /* Indicate that the component name is "unknown" */
+}
+
 /* Default color component to index for an unknown color model */
 int
-gx_error_get_color_comp_index(const gx_device * dev, const char * pname,
-						int name_size, int src_index)
+gx_error_get_color_comp_index(gx_device * dev, const char * pname,
+					int name_size, int component_type)
 {
     /*
      * We should never get here.  If we do then we do not have a "get_color_comp_index"
@@ -404,13 +483,17 @@ private cmap_proc_separation(cmap_separation_direct);
 private cmap_proc_devicen(cmap_devicen_halftoned);
 private cmap_proc_devicen(cmap_devicen_direct);
 
+private cmap_proc_is_halftoned(cmap_halftoned_is_halftoned);
+private cmap_proc_is_halftoned(cmap_direct_is_halftoned);
+
 private const gx_color_map_procs cmap_few = {
      cmap_gray_halftoned, 
      cmap_rgb_halftoned, 
      cmap_cmyk_halftoned,
      cmap_rgb_alpha_halftoned,
      cmap_separation_halftoned,
-     cmap_devicen_halftoned
+     cmap_devicen_halftoned,
+     cmap_halftoned_is_halftoned
     };
 private const gx_color_map_procs cmap_many = {
      cmap_gray_direct,
@@ -418,7 +501,8 @@ private const gx_color_map_procs cmap_many = {
      cmap_cmyk_direct,
      cmap_rgb_alpha_direct,
      cmap_separation_direct,
-     cmap_devicen_direct
+     cmap_devicen_direct,
+     cmap_direct_is_halftoned
     };
 
 const gx_color_map_procs *const cmap_procs_default = &cmap_many;
@@ -494,12 +578,20 @@ gx_default_remap_color(const gs_client_color * pcc, const gs_color_space * pcs,
 {
     frac conc[GS_CLIENT_COLOR_MAX_COMPONENTS];
     const gs_color_space *pconcs;
+    int i = pcs->type->num_components(pcs);
     int code = (*pcs->type->concretize_color)(pcc, pcs, conc, pis);
 
     if (code < 0)
 	return code;
     pconcs = cs_concrete_space(pcs, pis);
-    return (*pconcs->type->remap_concrete_color)(conc, pconcs, pdc, pis, dev, select);
+    code = (*pconcs->type->remap_concrete_color)(conc, pconcs, pdc, pis, dev, select);
+
+    /* Save original color space and color info into dev color */
+    i = any_abs(i);
+    for (i--; i >= 0; i--)
+	pdc->ccolor.paint.values[i] = pcc->paint.values[i];
+    pdc->ccolor_valid = true;
+    return code;
 }
 
 /* Color remappers for the standard color spaces. */
@@ -539,6 +631,10 @@ gx_remap_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
     float ftemp;
     frac fgray = unit_frac(pc->paint.values[0], ftemp);
 
+    /* Save original color space and color info into dev color */
+    pdc->ccolor.paint.values[0] = pc->paint.values[0];
+    pdc->ccolor_valid = true;
+
     if (pis->alpha == gx_max_color_value)
 	(*pis->cmap_procs->map_gray)
 	    (fgray, pdc, pis, dev, select);
@@ -583,6 +679,12 @@ gx_remap_DeviceRGB(const gs_client_color * pc, const gs_color_space * pcs,
     frac fred = unit_frac(pc->paint.values[0], ftemp), fgreen = unit_frac(pc->paint.values[1], ftemp),
          fblue = unit_frac(pc->paint.values[2], ftemp);
 
+    /* Save original color space and color info into dev color */
+    pdc->ccolor.paint.values[0] = pc->paint.values[0];
+    pdc->ccolor.paint.values[1] = pc->paint.values[1];
+    pdc->ccolor.paint.values[2] = pc->paint.values[2];
+    pdc->ccolor_valid = true;
+
     if (pis->alpha == gx_max_color_value)
 	gx_remap_concrete_rgb(fred, fgreen, fblue,
 			      pdc, pis, dev, select);
@@ -623,6 +725,13 @@ gx_remap_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
 /****** IGNORE alpha ******/
     float ft0, ft1, ft2, ft3;
 
+    /* Save original color space and color info into dev color */
+    pdc->ccolor.paint.values[0] = pc->paint.values[0];
+    pdc->ccolor.paint.values[1] = pc->paint.values[1];
+    pdc->ccolor.paint.values[2] = pc->paint.values[2];
+    pdc->ccolor.paint.values[3] = pc->paint.values[3];
+    pdc->ccolor_valid = true;
+
     gx_remap_concrete_cmyk((frac)unit_frac(pc->paint.values[0], ft0),
 			   (frac)unit_frac(pc->paint.values[1], ft1),
 			   (frac)unit_frac(pc->paint.values[2], ft2),
@@ -781,7 +890,7 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
 
     /* We make a test for direct vs. halftoned, rather than */
     /* duplicating most of the code of this procedure. */
-    if (gx_color_device_must_halftone(dev)) {
+    if (gx_device_must_halftone(dev)) {
 	if (gx_render_device_DeviceN(cm_comps, pdc, dev,
 		    pis->dev_ht, &pis->screen_phase[select], false) == 1)
 	    gx_color_load_select(pdc, pis, dev, select);
@@ -794,6 +903,12 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
     color = dev_proc(dev, encode_color)(dev, cv);
     if (color != gx_no_color_index) 
 	color_set_pure(pdc, color);
+    else {
+	if (gx_render_device_DeviceN(cm_comps, pdc, dev,
+		    pis->dev_ht, &pis->screen_phase[select], false) == 1)
+	    gx_color_load_select(pdc, pis, dev, select);
+	return;
+    }
 }
 
 private void
@@ -1081,6 +1196,19 @@ cmap_devicen_direct(const frac * pcc,
         cmap_devicen_halftoned(pcc, pdc, pis, dev, select);
 }
 
+/* ------ Halftoning check ----- */
+
+private bool
+cmap_halftoned_is_halftoned(const gs_imager_state * pis, gx_device * dev)
+{
+    return true;
+}
+
+private bool
+cmap_direct_is_halftoned(const gs_imager_state * pis, gx_device * dev)
+{
+    return false;
+}
 
 /* ------ Transfer function mapping ------ */
 
