@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxdhtserial.c,v 1.3 2002/10/07 08:28:56 ghostgum Exp $ */
+/* $Id: gxdhtserial.c 5567 2004-12-08 21:35:13Z stefan $ */
 /* Serialization and de-serialization for (traditional) halftones */
 
 #include "memory_.h"
@@ -117,8 +117,9 @@ gx_ht_read_tf(
     gx_transfer_map *   pmap;
 
     /* read the type byte */
-    if (--size < 0)
+    if (size == 0)
         return_error(gs_error_rangecheck);
+    --size;
     tf_type = (gx_ht_tf_type_t)*data++;
 
     /* if no transfer function, exit now */
@@ -135,7 +136,7 @@ gx_ht_read_tf(
                        return_error(gs_error_VMerror),
                        "gx_ht_read_tf" );
 
-    pmap->id = gs_next_ids(1);
+    pmap->id = gs_next_ids(mem, 1);
     pmap->closure.proc = 0;
     pmap->closure.data = 0;
     if (tf_type == gx_ht_tf_identity) {
@@ -299,8 +300,9 @@ gx_ht_read_component(
     const gx_dht_proc *     phtrp = gx_device_halftone_list;
 
     /* check the order type */
-    if (--size < 0)
+    if (size == 0)
         return_error(gs_error_rangecheck);
+    --size;
     order_type = (gx_ht_order_type_t)*data++;
 
     /* currently only the traditional halftone order are supported */
