@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxfont.h,v 1.15 2002/07/24 07:45:28 igor Exp $ */
+/* $Id: gxfont.h 5415 2004-10-13 15:31:59Z igor $ */
 /* Font object structure */
 /* Requires gsmatrix.h, gxdevice.h */
 
@@ -150,6 +150,9 @@ typedef struct gs_glyph_info_s {
     gs_glyph *pieces;		/* pieces are stored here: the caller must */
 				/* preset pieces if INFO_PIECES is set. */
 #define GLYPH_INFO_OUTLINE_WIDTHS 32 /* return unmodified widths, see above */
+#define GLYPH_INFO_VVECTOR0 64	
+#define GLYPH_INFO_VVECTOR1 128	/* must be VVECTOR0 << 1 */
+#define GLYPH_INFO_CDEVPROC 256	/* Allow CDevProc callout. */
 } gs_glyph_info_t;
 
 /* Define the "object" procedures of fonts. */
@@ -220,6 +223,13 @@ typedef struct gs_font_procs_s {
   gs_glyph proc(gs_font *, gs_char, gs_glyph_space_t)
     font_proc_encode_char((*encode_char));
 
+    /* Map a glyph name to Unicode UTF-16.
+     */
+
+#define font_proc_decode_glyph(proc)\
+  gs_char proc(gs_font *, gs_glyph)
+    font_proc_decode_glyph((*decode_glyph));
+
     /*
      * Get the next glyph in an enumeration of all glyphs defined by the
      * font.  index = 0 means return the first one; a returned index of 0
@@ -242,6 +252,17 @@ typedef struct gs_font_procs_s {
      * some member options require the caller to preset some of the elements
      * of info.  Note also that this procedure may return more information
      * than was requested.
+     *
+     * Implementations of this method must not access font->WMode,
+     * because for font descendents it is inherited from an upper font.
+     * Implementatios must derive WMode from requested flags specified
+     * in 'members' argument.
+     *
+     * Currently we do not handle requests, in which GLYPH_INFO_VVECTOR0
+     * is set, but GLYPH_INFO_WIDTH0 is not. Same for GLYPH_INFO_VVECTOR1
+     * and GLYPH_INFO_WIDTH1. Also requests, in which both GLYPH_INFO_WIDTH0 and
+     * GLYPH_INFO_WIDTH1 are set, may work wrongly. Such requests look never used 
+     * and debugged, and the implementation code requires improvements.
      */
 
 #define font_proc_glyph_info(proc)\
@@ -253,11 +274,16 @@ typedef struct gs_font_procs_s {
      * Append the outline for a glyph to a path, with the glyph origin
      * at the current point.  pmat is as for glyph_width.  The outline
      * does include a final moveto for the advance width.
+     *
+     * Implementations of this method must not access font->WMode,
+     * because for font descendents it is inherited from an upper font.
+     * This is especially important for Type 42 fonts with hmtx and vmtx.
      */
+    /* Currently glyph_outline retrieves sbw only with type 1,2,9 fonts. */
 
 #define font_proc_glyph_outline(proc)\
-  int proc(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,\
-	   gx_path *ppath)
+  int proc(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,\
+	   gx_path *ppath, double sbw[4])
     font_proc_glyph_outline((*glyph_outline));
 
     /*
@@ -317,6 +343,7 @@ font_proc_same_font(gs_default_same_font);
 font_proc_same_font(gs_base_same_font);
 /* Default glyph-level font procedures in gsfont.c */
 font_proc_encode_char(gs_no_encode_char);
+font_proc_decode_glyph(gs_no_decode_glyph);
 font_proc_enumerate_glyph(gs_no_enumerate_glyph);
 font_proc_glyph_info(gs_default_glyph_info);
 font_proc_glyph_outline(gs_no_glyph_outline);
@@ -355,6 +382,7 @@ typedef struct gs_font_name_s {
 	gs_font *base;			/* original (unscaled) base font */\
 	void *client_data;		/* additional client data */\
 	gs_matrix FontMatrix;\
+	gs_matrix orig_FontMatrix;      /* The original font matrix or zeros. */\
 	font_type FontType;\
 	bool BitmapWidths;\
 	fbit_type ExactSize, InBetweenSize, TransformedChar;\
@@ -446,10 +474,19 @@ gs_font_base *
 		     const gs_font_procs *procs, gs_font_dir *dir,
 		     client_name_t cname);
 
+/* Define a string to interact with unique_name in lib/pdf_font.ps .
+   The string is used to resolve glyph name conflict while
+   converting PDF Widths into Metrics.
+ */
+extern const char gx_extendeg_glyph_name_separator[];
+
 /*
  * Test whether a glyph is the notdef glyph for a base font.
  * The test is somewhat adhoc: perhaps this should be a virtual procedure.
  */
 bool gs_font_glyph_is_notdef(gs_font_base *bfont, gs_glyph glyph);
 
+/* Get font parent (a Type 9 font). */
+const gs_font_base *gs_font_parent(const gs_font_base *pbfont);
+
 #endif /* gxfont_INCLUDED */
