@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxht.c,v 1.14 2002/09/29 23:35:06 giles Exp $ */
+/*$Id: gxht.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Halftone rendering for imaging library */
 #include "memory_.h"
 #include "gx.h"
@@ -295,7 +295,7 @@ gx_dc_ht_binary_load(gx_device_color * pdevc, const gs_imager_state * pis,
     gx_ht_cache *pcache = porder->cache;
 
     if (pcache->order.bit_data != porder->bit_data)
-	gx_ht_init_cache(pcache, porder);
+	gx_ht_init_cache(pis->memory, pcache, porder);
     /*
      * We do not load the cache now.  Instead we wait until we are ready
      * to actually render the color.  This allows multiple colors to be
@@ -607,8 +607,9 @@ gx_dc_ht_binary_read(
     devc.colors.binary.b_tile = 0;
 
     /* verify the minimum amount of information */
-    if ((size -= 1) < 0)
+    if (size == 0)
         return_error(gs_error_rangecheck);
+    size --;
     flag_bits = *pdata++;
 
     /* read the other information provided */
@@ -641,8 +642,9 @@ gx_dc_ht_binary_read(
         size -= pdata - pdata_start;
     }
     if ((flag_bits & dc_ht_binary_has_index) != 0) {
-        if (--size < 0)
+        if (size == 0)
             return_error(gs_error_rangecheck);
+	--size;
         devc.colors.binary.b_index = *pdata++;
     }
 
@@ -703,7 +705,7 @@ gx_dc_ht_binary_get_nonzero_comps(
 /* Initialize the tile cache for a given screen. */
 /* Cache as many different levels as will fit. */
 void
-gx_ht_init_cache(gx_ht_cache * pcache, const gx_ht_order * porder)
+gx_ht_init_cache(const gs_memory_t *mem, gx_ht_cache * pcache, const gx_ht_order * porder)
 {
     uint width = porder->width;
     uint height = porder->height;
@@ -756,7 +758,7 @@ gx_ht_init_cache(gx_ht_cache * pcache, const gx_ht_order * porder)
 	raster = bitmap_raster(width_unit);
 	tile_bytes = raster * height;
     }
-    pcache->base_id = gs_next_ids(porder->num_levels + 1);
+    pcache->base_id = gs_next_ids(mem, porder->num_levels + 1);
     pcache->order = *porder;
     /* The transfer function is irrelevant, and might become dangling. */
     pcache->order.transfer = 0;
