@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxpath.c,v 1.8 2002/09/26 18:45:02 dan Exp $ */
+/*$Id: gxpath.c 4887 2004-03-13 18:28:52Z igor $ */
 /* Internal path management routines for Ghostscript library */
 #include "gx.h"
 #include "gserrors.h"
@@ -61,6 +61,41 @@ private_st_subpath();
 private rc_free_proc(rc_free_path_segments);
 private rc_free_proc(rc_free_path_segments_local);
 
+/*
+ * Define the default virtual path interface implementation.
+ */
+private int 
+    gz_path_add_point(gx_path *, fixed, fixed),
+    gz_path_add_line_notes(gx_path *, fixed, fixed, segment_notes),
+    gz_path_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
+    gz_path_close_subpath_notes(gx_path *, segment_notes);
+private byte gz_path_state_flags(gx_path *ppath, byte flags);
+
+private gx_path_procs default_path_procs = {
+    gz_path_add_point,
+    gz_path_add_line_notes,
+    gz_path_add_curve_notes,
+    gz_path_close_subpath_notes,
+    gz_path_state_flags
+};
+
+/*
+ * Define virtual path interface implementation for computing a path bbox.
+ */
+private int 
+    gz_path_bbox_add_point(gx_path *, fixed, fixed),
+    gz_path_bbox_add_line_notes(gx_path *, fixed, fixed, segment_notes),
+    gz_path_bbox_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
+    gz_path_bbox_close_subpath_notes(gx_path *, segment_notes);
+
+private gx_path_procs path_bbox_procs = {
+    gz_path_bbox_add_point,
+    gz_path_bbox_add_line_notes,
+    gz_path_bbox_add_curve_notes,
+    gz_path_bbox_close_subpath_notes,
+    gz_path_state_flags
+};
+
 private void
 gx_path_init_contents(gx_path * ppath)
 {
@@ -108,44 +143,10 @@ gx_path_init_contained_shared(gx_path * ppath, const gx_path * shared,
     }
     ppath->memory = mem;
     ppath->allocation = path_allocated_contained;
+    ppath->procs = &default_path_procs;
     return 0;
 }
 
-/*
- * Define the default virtual path interface implementation.
- */
-private int 
-    gz_path_add_point(gx_path *, fixed, fixed),
-    gz_path_add_line_notes(gx_path *, fixed, fixed, segment_notes),
-    gz_path_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
-    gz_path_close_subpath_notes(gx_path *, segment_notes);
-private byte gz_path_state_flags(gx_path *ppath, byte flags);
-
-private gx_path_procs default_path_procs = {
-    gz_path_add_point,
-    gz_path_add_line_notes,
-    gz_path_add_curve_notes,
-    gz_path_close_subpath_notes,
-    gz_path_state_flags
-};
-
-/*
- * Define virtual path interface implementation for computing a path bbox.
- */
-private int 
-    gz_path_bbox_add_point(gx_path *, fixed, fixed),
-    gz_path_bbox_add_line_notes(gx_path *, fixed, fixed, segment_notes),
-    gz_path_bbox_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
-    gz_path_bbox_close_subpath_notes(gx_path *, segment_notes);
-
-private gx_path_procs path_bbox_procs = {
-    gz_path_bbox_add_point,
-    gz_path_bbox_add_line_notes,
-    gz_path_bbox_add_curve_notes,
-    gz_path_bbox_close_subpath_notes,
-    gz_path_state_flags
-};
-
 /*
  * Allocate a path on the heap, and initialize it.  If shared is NULL,
  * allocate a segments object; if shared is an existing path, share its
@@ -519,8 +520,6 @@ gz_path_add_point(gx_path * ppath, fixed x, fixed y)
 private int
 gz_path_bbox_add_point(gx_path * ppath, fixed x, fixed y)
 {
-    ppath->outside_start.x = x;
-    ppath->outside_start.y = y;
     gz_path_bbox_move(ppath, x, y);
     return 0;
 }
@@ -770,7 +769,6 @@ gx_path_add_path(gx_path * ppath, gx_path * ppfrom)
     }
     /* Transfer the remaining state. */
     ppath->position = ppfrom->position;
-    ppath->outside_position = ppfrom->outside_position;
     ppath->state_flags = ppfrom->state_flags;
     /* Reset the source path. */
     gx_path_init_contents(ppfrom);
@@ -854,8 +852,6 @@ gz_path_close_subpath_notes(gx_path * ppath, segment_notes notes)
 private int
 gz_path_bbox_close_subpath_notes(gx_path * ppath, segment_notes notes)
 {
-    gz_path_bbox_move(ppath, (fixed)ppath->outside_start.x, 
-			     (fixed)ppath->outside_start.y);
     return 0;
 }
 
