@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxpcmap.c,v 1.7 2002/08/22 07:12:29 henrys Exp $ */
+/* $Id: gxpcmap.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Pattern color mapping for Ghostscript library */
 #include "math_.h"
 #include "memory_.h"
@@ -153,6 +153,7 @@ gx_pattern_accum_alloc(gs_memory_t * mem, client_name_t cname)
     gx_device_init((gx_device *)adev,
 		   (const gx_device *)&gs_pattern_accum_device,
 		   mem, true);
+    check_device_separable((gx_device *)adev);
     gx_device_forward_fill_in_procs((gx_device_forward *)adev);
     return adev;
 }
@@ -380,16 +381,16 @@ gx_pattern_alloc_cache(gs_memory_t * mem, uint num_tiles, ulong max_bits)
 {
     gx_pattern_cache *pcache =
     gs_alloc_struct(mem, gx_pattern_cache, &st_pattern_cache,
-		    "pattern_cache_alloc(struct)");
+		    "gx_pattern_alloc_cache(struct)");
     gx_color_tile *tiles =
     gs_alloc_struct_array(mem, num_tiles, gx_color_tile,
 			  &st_color_tile_element,
-			  "pattern_cache_alloc(tiles)");
+			  "gx_pattern_alloc_cache(tiles)");
     uint i;
 
     if (pcache == 0 || tiles == 0) {
-	gs_free_object(mem, tiles, "pattern_cache_alloc(tiles)");
-	gs_free_object(mem, pcache, "pattern_cache_alloc(struct)");
+	gs_free_object(mem, tiles, "gx_pattern_alloc_cache(tiles)");
+	gs_free_object(mem, pcache, "gx_pattern_alloc_cache(struct)");
 	return 0;
     }
     pcache->memory = mem;
@@ -443,7 +444,7 @@ gstate_set_pattern_cache(gs_state * pgs, gx_pattern_cache * pcache)
 private void
 gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile)
 {
-    if (ctile->id != gx_no_bitmap_id) {
+    if ((ctile->id != gx_no_bitmap_id) && !ctile->is_dummy) {
 	gs_memory_t *mem = pcache->memory;
 	gx_device_memory mdev;
 
@@ -539,8 +540,9 @@ gx_pattern_cache_add_entry(gs_imager_state * pis,
     ctile->step_matrix = pinst->step_matrix;
     ctile->bbox = pinst->bbox;
     ctile->is_simple = pinst->is_simple;
+    ctile->is_dummy = false;
     if (mbits != 0) {
-	make_bitmap(&ctile->tbits, mbits, gs_next_ids(1));
+	make_bitmap(&ctile->tbits, mbits, gs_next_ids(pis->memory, 1));
 	mbits->bitmap_memory = 0;	/* don't free the bits */
     } else
 	ctile->tbits.data = 0;
@@ -554,6 +556,38 @@ gx_pattern_cache_add_entry(gs_imager_state * pis,
     *pctile = ctile;
     return 0;
 }
+
+/* Add a dummy Pattern cache entry.  Stubs a pattern tile for interpreter when
+   device handles high level patterns. */
+int
+gx_pattern_cache_add_dummy_entry(gs_imager_state *pis, 
+	    gs_pattern1_instance_t *pinst, int depth)
+{
+    gx_color_tile *ctile;
+    gx_pattern_cache *pcache;
+    gx_bitmap_id id = pinst->id;
+    int code = ensure_pattern_cache(pis);
+
+    if (code < 0)
+	return code;
+    pcache = pis->pattern_cache;
+    ctile = &pcache->tiles[id % pcache->num_tiles];
+    gx_pattern_cache_free_entry(pcache, ctile);
+    ctile->id = id;
+    ctile->depth = depth;
+    ctile->uid = pinst->template.uid;
+    ctile->tiling_type = pinst->template.TilingType;
+    ctile->step_matrix = pinst->step_matrix;
+    ctile->bbox = pinst->bbox;
+    ctile->is_simple = pinst->is_simple;
+    ctile->is_dummy = true;
+    memset(&ctile->tbits, 0 , sizeof(ctile->tbits));
+    ctile->tbits.size = pinst->size;
+    ctile->tbits.id = gs_no_bitmap_id;
+    memset(&ctile->tmask, 0 , sizeof(ctile->tmask));
+    pcache->tiles_used++;
+    return 0;
+}
 private void
 make_bitmap(register gx_strip_bitmap * pbm, const gx_device_memory * mdev,
 	    gx_bitmap_id id)
@@ -671,7 +705,9 @@ gs_pattern1_remap_color(const gs_client_color * pc, const gs_color_space * pcs,
     gs_pattern1_instance_t *pinst = (gs_pattern1_instance_t *)pc->pattern;
     int code;
 
+    /* Save original color space and color info into dev color */
     pdc->ccolor = *pc;
+    pdc->ccolor_valid = true;
     if (pinst == 0) {
 	/* Null pattern */
 	color_set_null_pattern(pdc);
