@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxstroke.c,v 1.7 2002/10/31 09:00:28 igor Exp $ */
+/* $Id: gxstroke.c 5428 2004-10-18 15:23:20Z igor $ */
 /* Path stroking procedures for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -33,6 +33,7 @@
 #include "gzpath.h"
 #include "gzcpath.h"
 #include "gxpaint.h"
+#include "vdtrace.h"
 
 /*
  * We don't really know whether it's a good idea to take fill adjustment
@@ -228,8 +229,22 @@ gx_default_stroke_path(gx_device * dev, const gs_imager_state * pis,
 		       const gx_drawing_color * pdcolor,
 		       const gx_clip_path * pcpath)
 {
-    return gx_stroke_path_only(ppath, (gx_path *) 0, dev, pis, params,
+    int code;
+
+    if (vd_allowed('S')) {
+	vd_get_dc('S');
+	if (vd_enabled) {
+	    vd_set_shift(0, 100);
+	    vd_set_scale(0.03);
+	    vd_set_origin(0, 0);
+	    vd_erase(RGB(192, 192, 192));
+	}
+    }
+    code = gx_stroke_path_only(ppath, (gx_path *) 0, dev, pis, params,
 			       pdcolor, pcpath);
+    if (vd_allowed('S'))
+	vd_release_dc;
+    return code;
 }
 
 /* Fill a partial stroked path.  Free variables: */
@@ -338,7 +353,8 @@ gx_stroke_path_only(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	 pgs_lp->join == gs_join_none ? gs_join_bevel : pgs_lp->join);
     float line_width = pgs_lp->half_width;	/* (*half* the line width) */
     bool always_thin;
-    double line_width_and_scale, device_line_width_scale;
+    double line_width_and_scale;
+    double device_line_width_scale = 0; /* Quiet compiler. */
     double device_dot_length = pgs_lp->dot_length * fixed_1;
     const subpath *psub;
 
@@ -556,14 +572,18 @@ gx_stroke_path_only(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 		}
 		/*
 		 * The entire subpath is degenerate, but it includes
-		 * more than one point.  If we are using round caps or
-		 * the dot length is non-zero, draw the caps, otherwise
-		 * do nothing.
+		 * more than one point.  If the dot length is non-zero,
+		 * draw the caps, otherwise do nothing.
 		 */
-		if (!(pgs_lp->cap == gs_cap_round ||
-		      pgs_lp->dot_length != 0)
-		    )
+		if (pgs_lp->dot_length != 0)
 		    break;
+		if (pgs_lp->cap != gs_cap_round) {
+		    /* From PLRM, stroke operator :
+		       If a subpath is degenerate (consists of a single-point closed path 
+		       or of two or more points at the same coordinates), 
+		       stroke paints it only if round line caps have been specified */
+		    break;
+		}
 		/*
 		 * Orient the dot according to the previous segment if
 		 * any, or else the next segment if any, or else
@@ -628,8 +648,8 @@ gx_stroke_path_only(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 		if (uniform != 0) {
 		    /* We can save a lot of work in this case. */
 		    /* We know orient != orient_other. */
-		    float dpx = udx, dpy = udy;
-		    float wl = device_line_width_scale /
+		    double dpx = udx, dpy = udy;
+		    double wl = device_line_width_scale /
 		    hypot(dpx, dpy);
 
 		    pl.e.cdelta.x = (fixed) (dpx * wl);
@@ -1063,6 +1083,7 @@ stroke_add(gx_path * ppath, int first, pl_ptr plp, pl_ptr nplp,
   done:
     if (code < 0)
 	return code;
+    vd_closepath;
     return gx_path_close_subpath(ppath);
 }
 
@@ -1071,14 +1092,20 @@ private int
 add_points(gx_path * ppath, const gs_fixed_point * points, int npoints,
 	   bool moveto_first)
 {
+    vd_setcolor(0);
+    vd_setlinewidth(0);
     if (moveto_first) {
 	int code = gx_path_add_point(ppath, points[0].x, points[0].y);
 
+	vd_moveto(points[0].x, points[0].y);
 	if (code < 0)
 	    return code;
+	vd_lineto_multi(points + 1, npoints - 1);
 	return gx_path_add_lines(ppath, points + 1, npoints - 1);
-    } else
+    } else {
+	vd_lineto_multi(points, npoints);
 	return gx_path_add_lines(ppath, points, npoints);
+    }
 }
 
 /* ---------------- Join computation ---------------- */
