@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxtext.h,v 1.10.2.2 2004/04/03 18:52:16 igor Exp $ */
+/* $Id: gxtext.h 4930 2004-04-01 11:30:47Z igor $ */
 /* Driver text interface implementation support */
 
 #ifndef gxtext_INCLUDED
@@ -109,14 +109,9 @@ rc_free_proc(rc_free_text_enum);
     /* The following are used to return information to the client. */\
     gs_text_returned_t returned
 /* The typedef is in gstext.h. */
-/*typedef*/ struct gs_text_enum_s {
+struct gs_text_enum_s {
     gs_text_enum_common;
-} /*gs_text_enum_t*/;
-
-#if NEW_TT_INTERPRETER
-    /* The 'pair' field is added to the macro above. */
-    /* A definition of cached_fm_pair is added above */
-#endif
+};
 
 /*
  * Notes on the imaging_dev field of device enumeration structures:
@@ -154,7 +149,7 @@ rc_free_proc(rc_free_text_enum);
  * approach will fail.
  */
 
-#define st_gs_text_enum_max_ptrs (st_gs_text_params_max_ptrs + 7)
+#define st_gs_text_enum_max_ptrs (st_gs_text_params_max_ptrs + 8)
 /*extern_st(st_gs_text_enum); */
 #define public_st_gs_text_enum()	/* in gstext.c */\
   gs_public_st_composite(st_gs_text_enum, gs_text_enum_t, "gs_text_enum_t",\
