@@ -14,15 +14,12 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: ifont2.h,v 1.5 2002/06/16 04:47:10 lpd Exp $ */
+/* $Id: ifont2.h 4163 2003-08-21 14:55:14Z igor $ */
 /* Type 2 font utilities 2 */
 
 #ifndef ifont2_INCLUDED
 #  define ifont2_INCLUDED
 
-/* Declare the Type 2 interpreter. */
-extern charstring_interpret_proc(gs_type2_interpret);
-
 /* Default value of lenIV */
 #define DEFAULT_LENIV_2 (-1)
 
