@@ -14,20 +14,11 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: imain.c,v 1.27 2002/10/08 00:49:49 dan Exp $ */
+/* $Id: imain.c 5501 2004-11-14 01:41:58Z ghostgum $ */
 /* Common support for interpreter front ends */
+#include "malloc_.h"
 #include "memory_.h"
 #include "string_.h"
-/* Capture stdin/out/err before gs.h redefines them. */
-#include <stdio.h>
-#include <stdlib.h>
-private void
-set_stdfiles(FILE * stdfiles[3])
-{
-    stdfiles[0] = stdin;
-    stdfiles[1] = stdout;
-    stdfiles[2] = stderr;
-}
 #include "ghost.h"
 #include "gp.h"
 #include "gscdefs.h"		/* for gs_init_file */
@@ -36,7 +27,8 @@ set_stdfiles(FILE * stdfiles[3])
 #include "gsutil.h"		/* for bytes_compare */
 #include "gxdevice.h"
 #include "gxalloc.h"
-#include "errors.h"
+#include "gzstate.h"
+#include "ierrors.h"
 #include "oper.h"
 #include "iconf.h"		/* for gs_init_* imports */
 #include "idebug.h"
@@ -62,22 +54,39 @@ set_stdfiles(FILE * stdfiles[3])
 
 /* ------ Exported data ------ */
 
-/* Define the default instance of the interpreter. */
-/* Currently, this is the *only possible* instance, because most of */
-/* the places that need to take an explicit instance argument don't. */
-private gs_main_instance the_gs_main_instance;
-gs_main_instance *
-gs_main_instance_default(void)
+/** using backpointers retrieve minst from any memory pointer 
+ * 
+ */
+gs_main_instance* 
+get_minst_from_memory(const gs_memory_t *mem)
 {
-    /* Determine whether the instance has been initialized. */
-    if (the_gs_main_instance.memory_chunk_size == 0)
-	the_gs_main_instance = gs_main_instance_init_values;
-    return &the_gs_main_instance;
+#ifdef PSI_INCLUDED
+    extern gs_main_instance *ps_impl_get_minst( const gs_memory_t *mem );
+    return ps_impl_get_minst(mem);
+#else
+    return (gs_main_instance*)mem->gs_lib_ctx->top_of_system;  
+#endif
 }
 
-/* Define the interpreter's name table.  We'll move it somewhere better */
-/* eventually.... */
-name_table *the_gs_name_table;
+/** construct main instance caller needs to retain */
+gs_main_instance *
+gs_main_alloc_instance(gs_memory_t *mem)
+{
+    gs_main_instance *minst = 0;
+    if (mem) {
+	minst = (gs_main_instance *) gs_alloc_bytes_immovable(mem, 
+							      sizeof(gs_main_instance),
+							      "init_main_instance");
+	memcpy(minst, &gs_main_instance_init_values, sizeof(gs_main_instance_init_values));
+	minst->heap = mem;
+	
+#       ifndef PSI_INCLUDED
+	mem->gs_lib_ctx->top_of_system = minst;
+        /* else top of system is pl_universe */
+#       endif
+    }
+    return minst;
+}
 
 /* ------ Forward references ------ */
 
@@ -87,45 +96,38 @@ private void print_resource_usage(const gs_main_instance *,
 
 /* ------ Initialization ------ */
 
-/* Save the real stdio files. */
-void
-gs_get_real_stdio(FILE * stdfiles[3])
-{
-    set_stdfiles(stdfiles);
-}
-
 /* Initialization to be done before anything else. */
 int
 gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 	      int max_lib_paths)
 {
-    gs_memory_t *heap;
     ref *paths;
 
-    /* Set our versions of stdin/out/err. */
-    gs_stdin = minst->fstdin = in;
-    gs_stdout = minst->fstdout = out;
-    gs_stderr = minst->fstderr = err;
     /* Do platform-dependent initialization. */
     /* We have to do this as the very first thing, */
     /* because it detects attempts to run 80N86 executables (N>0) */
     /* on incompatible processors. */
     gp_init();
+
+    /* Initialize the imager. */     
+#   ifndef PSI_INCLUDED
+       /* Reset debugging flags */
+       memset(gs_debug, 0, 128);
+       gs_log_errors = 0;  /* gs_debug['#'] = 0 */ 
+#   else
+       /* plmain settings remain in effect */
+#   endif
     gp_get_usertime(minst->base_time);
-    /* Initialize the imager. */
-    heap = gs_lib_init0(gs_stdout);
-    if (heap == 0)
-	return_error(e_VMerror);
-    minst->heap = heap;
+
     /* Initialize the file search paths. */
-    paths = (ref *) gs_alloc_byte_array(heap, max_lib_paths, sizeof(ref),
+    paths = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
 					"lib_path array");
     if (paths == 0) {
-	gs_lib_finit(1, e_VMerror);
+	gs_lib_finit(1, e_VMerror, minst->heap);
 	return_error(e_VMerror);
     }
     make_array(&minst->lib_path.container, avm_foreign, max_lib_paths,
-	       (ref *) gs_alloc_byte_array(heap, max_lib_paths, sizeof(ref),
+	       (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
 					   "lib_path array"));
     make_array(&minst->lib_path.list, avm_foreign | a_readonly, 0,
 	       minst->lib_path.container.value.refs);
@@ -144,7 +146,7 @@ gs_main_init1(gs_main_instance * minst)
     if (minst->init_done < 1) {
 	gs_dual_memory_t idmem;
 	int code =
-	    ialloc_init(&idmem, (gs_raw_memory_t *)&gs_memory_default,
+	    ialloc_init(&idmem, minst->heap,
 			minst->memory_chunk_size, gs_have_level2());
 
 	if (code < 0)
@@ -160,9 +162,9 @@ gs_main_init1(gs_main_instance * minst)
 
 	    if (nt == 0)
 		return_error(e_VMerror);
-	    the_gs_name_table = nt;
+	    mem->gs_lib_ctx->gs_name_table = nt;
 	    code = gs_register_struct_root(mem, NULL,
-					   (void **)&the_gs_name_table,
+					   (void **)&mem->gs_lib_ctx->gs_name_table,
 					   "the_gs_name_table");
 	    if (code < 0)
 		return code;
@@ -206,6 +208,9 @@ gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
     ref refpop;
     int code;
 
+    /* set interpreter pointer to lib_path */
+    minst->i_ctx_p->lib_path = &minst->lib_path;
+
     code = gs_interpret(&minst->i_ctx_p, pref, 
 		user_errors, pexit_code, perror_object);
     while ((code == e_NeedStdin) || (code == e_NeedStdout) || 
@@ -225,7 +230,7 @@ gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
 		int count = esp[0].tas.rsize;
 		int rcode = 0;
 		if (str != NULL)
-		    rcode = gs_main_outwrite(minst, str, count);
+		    rcode = outwrite(imemory, str, count);
 		if (rcode < 0)
 		    return_error(e_ioerror);
 	    }
@@ -247,7 +252,7 @@ gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
 		int count = esp[0].tas.rsize;
 		int rcode = 0;
 		if (str != NULL)
-		    rcode = gs_main_errwrite(minst, str, count);
+		    rcode = errwrite(str, count);
 		if (rcode < 0)
 		    return_error(e_ioerror);
 	    }
@@ -265,12 +270,14 @@ gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
 	     *  esp[-1] = file, stdin stream
 	     * We read from stdin then pop these 2 items.
 	     */
-	    if (minst->stdin_fn)
-		count = (*minst->stdin_fn)(minst->caller_handle, 
-			minst->stdin_buf, count);
+	    if (minst->heap->gs_lib_ctx->stdin_fn)
+		count = (*minst->heap->gs_lib_ctx->stdin_fn)
+		    (minst->heap->gs_lib_ctx->caller_handle, 
+		     minst->stdin_buf, count);
 	    else
 		count = gp_stdin_read(minst->stdin_buf, count, 
-		    minst->stdin_is_interactive, minst->fstdin);
+				      minst->heap->gs_lib_ctx->stdin_is_interactive,
+				      minst->heap->gs_lib_ctx->fstdin);
 	    if (count < 0)
 	        return_error(e_ioerror);
 
@@ -347,8 +354,12 @@ gs_main_init2(gs_main_instance * minst)
 	    return code;
 	minst->init_done = 2;
 	i_ctx_p = minst->i_ctx_p; /* init file may change it */
+	/* NB this is to be done with device parameters
+	 * both minst->display and  display_set_callback() are going away
+	*/
 	if (minst->display)
 	    code = display_set_callback(minst, minst->display);
+
 	if (code < 0)
 	    return code;
     }
@@ -464,8 +475,11 @@ gs_main_lib_open(gs_main_instance * minst, const char *file_name, ref * pfile)
     byte fn[maxfn];
     uint len;
 
-    return lib_file_open(file_name, strlen(file_name), fn, maxfn,
-			 &len, pfile, imemory);
+    return lib_file_open( &minst->lib_path,
+			  NULL /* Don't check permissions here, because permlist 
+				  isn't ready running init files. */
+			  , file_name, strlen(file_name), fn, maxfn,
+			  &len, pfile, imemory);
 }
 
 /* Open and execute a file. */
@@ -725,7 +739,7 @@ gs_pop_string(gs_main_instance * minst, gs_string * result)
 	return code;
     switch (r_type(&vref)) {
 	case t_name:
-	    name_string_ref(&vref, &vref);
+	    name_string_ref(minst->heap, &vref, &vref);
 	    code = 1;
 	    goto rstr;
 	case t_string:
@@ -768,7 +782,7 @@ private char *gs_main_tempnames(gs_main_instance *minst)
 	/* get lengths of temporary filenames */
 	idict = dict_first(tempfiles);
 	while ((idict = dict_next(tempfiles, idict, &keyval[0])) >= 0) {
-	    if (obj_string_data(&keyval[0], &data, &size) >= 0)
+	    if (obj_string_data(minst->heap, &keyval[0], &data, &size) >= 0)
 		len += size + 1;
 	}
 	if (len != 0)
@@ -779,7 +793,7 @@ private char *gs_main_tempnames(gs_main_instance *minst)
 	    idict = dict_first(tempfiles);
 	    i = 0;
 	    while ((idict = dict_next(tempfiles, idict, &keyval[0])) >= 0) {
-		if (obj_string_data(&keyval[0], &data, &size) >= 0) {
+	        if (obj_string_data(minst->heap, &keyval[0], &data, &size) >= 0) {
 		    memcpy(tempnames+i, (const char *)data, size);
 		    i+= size;
 		    tempnames[i++] = '\0';
@@ -791,13 +805,17 @@ private char *gs_main_tempnames(gs_main_instance *minst)
 }
 
 /* Free all resources and return. */
-void
+int
 gs_main_finit(gs_main_instance * minst, int exit_status, int code)
 {
     i_ctx_t *i_ctx_p = minst->i_ctx_p;
     int exit_code;
     ref error_object;
     char *tempnames;
+
+    /* NB: need to free gs_name_table
+     */
+
     /*
      * Previous versions of this code closed the devices in the
      * device list here.  Since these devices are now prototypes,
@@ -805,10 +823,42 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      * alloc_restore_all will close dynamically allocated devices.
      */
     tempnames = gs_main_tempnames(minst);
+    /* 
+     * Close the "main" device, because it may need to write out
+     * data before destruction. pdfwrite needs so.
+     */
+    if (minst->init_done >= 1) {
+	int code;
+
+	if (idmemory->reclaim != 0) {
+	    code = interp_reclaim(&minst->i_ctx_p, avm_global);
+
+	    if (code < 0) {
+		eprintf1("ERROR %d reclaiming the memory while the interpreter finalization.\n", code);
+		return e_Fatal;
+	    }
+	    i_ctx_p = minst->i_ctx_p; /* interp_reclaim could change it. */
+	}
+	if (i_ctx_p->pgs != NULL && i_ctx_p->pgs->device != NULL) {
+	    gx_device *pdev = i_ctx_p->pgs->device;
+
+	    /* deactivate the device just before we close it for the last time */
+	    gs_main_run_string(minst, 
+		".uninstallpagedevice "
+		"serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
+		0 , &exit_code, &error_object);
+	    code = gs_closedevice(pdev);
+	    if (code < 0)
+		eprintf2("ERROR %d closing the device. See gs/src/ierrors.h for code explanation.\n", code, i_ctx_p->pgs->device->dname);
+	    if (exit_status == 0 || exit_status == e_Quit)
+		exit_status = code;
+	}
+    }
     /* Flush stdout and stderr */
     if (minst->init_done >= 2)
       gs_main_run_string(minst, 
-	"(%stdout) (w) file closefile (%stderr) (w) file closefile quit",
+	"(%stdout) (w) file closefile (%stderr) (w) file closefile "
+        "serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
 	0 , &exit_code, &error_object);
     gp_readline_finit(minst->readline_data);
     if (gs_debug_c(':'))
@@ -816,19 +866,20 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     /* Do the equivalent of a restore "past the bottom". */
     /* This will release all memory, close all open files, etc. */
     if (minst->init_done >= 1) {
-        gs_raw_memory_t *mem_raw = i_ctx_p->memory.current->parent;
+        gs_memory_t *mem_raw = i_ctx_p->memory.current->non_gc_memory;
         i_plugin_holder *h = i_ctx_p->plugin_list;
         alloc_restore_all(idmemory);
         i_plugin_finit(mem_raw, h);
     }
     /* clean up redirected stdout */
-    if (minst->fstdout2 && (minst->fstdout2 != minst->fstdout)
-	    && (minst->fstdout2 != minst->fstderr)) {
-	fclose(minst->fstdout2);
-	minst->fstdout2 = (FILE *)NULL;
+    if (minst->heap->gs_lib_ctx->fstdout2 
+	&& (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstdout)
+	&& (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstderr)) {
+	fclose(minst->heap->gs_lib_ctx->fstdout2);
+	minst->heap->gs_lib_ctx->fstdout2 = (FILE *)NULL;
     }
-    minst->stdout_is_redirected = 0;
-    minst->stdout_to_stderr = 0;
+    minst->heap->gs_lib_ctx->stdout_is_redirected = 0;
+    minst->heap->gs_lib_ctx->stdout_to_stderr = 0;
     /* remove any temporary files, after ghostscript has closed files */
     if (tempnames) {
 	char *p = tempnames;
@@ -838,22 +889,23 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
 	}
 	free(tempnames);
     }
-    gs_lib_finit(exit_status, code);
+    gs_lib_finit(exit_status, code, minst->heap);
+    return exit_status;
+}
+int
+gs_to_exit_with_code(const gs_memory_t *mem, int exit_status, int code)
+{
+    return gs_main_finit(get_minst_from_memory(mem), exit_status, code);
+}
+int
+gs_to_exit(const gs_memory_t *mem, int exit_status)
+{
+    return gs_to_exit_with_code(mem, exit_status, 0);
 }
 void
-gs_to_exit_with_code(int exit_status, int code)
+gs_abort(const gs_memory_t *mem)
 {
-    gs_finit(exit_status, code);
-}
-void
-gs_to_exit(int exit_status)
-{
-    gs_to_exit_with_code(exit_status, 0);
-}
-void
-gs_abort(void)
-{
-    gs_to_exit(1);
+    gs_to_exit(mem, 1);
     /* it's fatal calling OS independent exit() */
     gp_do_exit(1);	
 }
@@ -907,93 +959,11 @@ gs_main_dump_stack(gs_main_instance *minst, int code, ref * perror_object)
     dprintf1("\nUnexpected interpreter error %d.\n", code);
     if (perror_object != 0) {
 	dputs("Error object: ");
-	debug_print_ref(perror_object);
+	debug_print_ref(minst->heap, perror_object);
 	dputc('\n');
     }
-    debug_dump_stack(&o_stack, "Operand stack");
-    debug_dump_stack(&e_stack, "Execution stack");
-    debug_dump_stack(&d_stack, "Dictionary stack");
-}
-/* Backward compatibility */
-void
-gs_debug_dump_stack(int code, ref * perror_object)
-{
-    gs_main_dump_stack(gs_main_instance_default(), code, perror_object);
+    debug_dump_stack(minst->heap, &o_stack, "Operand stack");
+    debug_dump_stack(minst->heap, &e_stack, "Execution stack");
+    debug_dump_stack(minst->heap, &d_stack, "Dictionary stack");
 }
 
-
-/* Provide a single point for all "C" stdout and stderr.
- * Eventually these will always be referenced through an instance structure. 
- * We don't know which instance is running (and currently only one
- * instance is possible) so use the default instance.
- */
-
-int
-gs_main_outwrite(gs_main_instance *minst, const char *str, int len)
-{
-    int code;
-    FILE *fout;
-    if (len == 0)
-	return 0;
-    if (minst->stdout_is_redirected) {
-	if (minst->stdout_to_stderr)
-	    return gs_main_errwrite(minst, str, len);
-	fout = minst->fstdout2;
-    }
-    else if (minst->stdout_fn) {
-	return (*minst->stdout_fn)(minst->caller_handle, str, len);
-    }
-    else {
-	fout = minst->fstdout;
-    }
-    code = fwrite(str, 1, len, fout);
-    fflush(fout);
-    return code;
-}
-
-int
-gs_main_errwrite(gs_main_instance *minst, const char *str, int len)
-{
-    int code;
-    if (len == 0)
-	return 0;
-    if (minst->stderr_fn)
-	return (*minst->stderr_fn)(minst->caller_handle, str, len);
-    code = fwrite(str, 1, len, minst->fstderr);
-    fflush(minst->fstderr);
-    return code;
-}
-
-int outwrite(const char *str, int len)
-{
-    return gs_main_outwrite(gs_main_instance_default(), str, len);
-}
-
-int errwrite(const char *str, int len)
-{
-    return gs_main_errwrite(gs_main_instance_default(), str, len);
-}
-
-void outflush(void)
-{
-    gs_main_instance * minst = gs_main_instance_default();
-    if (minst->stdout_is_redirected) {
-	if (minst->stdout_to_stderr) {
-	    if (!minst->stderr_fn)
-		fflush(minst->fstderr);
-	}
-	else
-	    fflush(minst->fstdout2);
-    }
-    else if (!minst->stdout_fn)
-        fflush(minst->fstdout);
-}
-
-void errflush(void)
-{
-    gs_main_instance * minst = gs_main_instance_default();
-    if (!minst->stderr_fn)
-        fflush(minst->fstderr);
-}
-
-
