@@ -14,10 +14,10 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: inobtokn.c,v 1.4 2002/02/21 22:24:53 giles Exp $ */
+/* $Id: inobtokn.c 4180 2003-09-03 03:22:59Z giles $ */
 /* Dummy scan_binary_token for Level 1 systems */
 #include "ghost.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "stream.h"
 #include "iscan.h"
 #include "iscanbin.h"
