@@ -14,11 +14,11 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: isave.c,v 1.7 2002/06/16 03:48:22 lpd Exp $ */
+/* $Id: isave.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* Save/restore manager for Ghostscript interpreter */
 #include "ghost.h"
 #include "memory_.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "gsexit.h"
 #include "gsstruct.h"
 #include "stream.h"		/* for linking for forgetsave */
@@ -32,8 +32,6 @@
 #include "ivmspace.h"
 #include "gsutil.h"		/* gs_next_ids prototype */
 
-/* Imported save/restore routines */
-extern void font_restore(const alloc_save_t *);
 
 /* Structure descriptor */
 private_st_alloc_save();
@@ -311,7 +309,7 @@ alloc_save_state(gs_dual_memory_t * dmem, void *cdata)
 {
     gs_ref_memory_t *lmem = dmem->space_local;
     gs_ref_memory_t *gmem = dmem->space_global;
-    ulong sid = gs_next_ids(2);
+    ulong sid = gs_next_ids((const gs_memory_t *)lmem->stable_memory, 2);
     bool global =
 	lmem->save_level == 0 && gmem != lmem &&
 	gmem->num_contexts == 1;
@@ -387,7 +385,7 @@ alloc_save_space(gs_ref_memory_t * mem, gs_dual_memory_t * dmem, ulong sid)
 	if (cp->ctop - cp->cbot > min_inner_chunk_space) {
 	    /* Create an inner chunk to cover only the unallocated part. */
 	    chunk_t *inner =
-		gs_raw_alloc_struct_immovable(mem->parent, &st_chunk,
+		gs_raw_alloc_struct_immovable(mem->non_gc_memory, &st_chunk,
 					      "alloc_save_space(inner)");
 
 	    if (inner == 0)
@@ -415,7 +413,7 @@ alloc_save_space(gs_ref_memory_t * mem, gs_dual_memory_t * dmem, ulong sid)
     }
     save->state = save_mem;
     save->spaces = dmem->spaces;
-    save->restore_names = (name_memory() == (gs_memory_t *) mem);
+    save->restore_names = (name_memory(mem) == (gs_memory_t *) mem);
     save->is_current = (dmem->current == mem);
     save->id = sid;
     mem->saved = save;
@@ -453,7 +451,7 @@ alloc_save_change_in(gs_ref_memory_t *mem, const ref * pcont,
     else {
 	lprintf3("Bad type %u for save!  pcont = 0x%lx, where = 0x%lx\n",
 		 r_type(pcont), (ulong) pcont, (ulong) where);
-	gs_abort();
+	gs_abort((const gs_memory_t *)mem);
     }
     if (r_is_packed(where))
 	*(ref_packed *)&cp->contents = *where;
@@ -563,24 +561,30 @@ alloc_is_since_save(const void *vptr, const alloc_save_t * save)
 
 /* Test whether a name would be invalidated by a restore. */
 bool
-alloc_name_is_since_save(const ref * pnref, const alloc_save_t * save)
+alloc_name_is_since_save(const gs_memory_t *mem,
+			 const ref * pnref, const alloc_save_t * save)
 {
     const name_string_t *pnstr;
 
     if (!save->restore_names)
 	return false;
-    pnstr = names_string_inline(the_gs_name_table, pnref);
+    pnstr = names_string_inline(mem->gs_lib_ctx->gs_name_table, pnref);
     if (pnstr->foreign_string)
 	return false;
     return alloc_is_since_save(pnstr->string_bytes, save);
 }
 bool
-alloc_name_index_is_since_save(uint nidx, const alloc_save_t * save)
+alloc_name_index_is_since_save(const gs_memory_t *mem,
+			       uint nidx, const alloc_save_t *save)
 {
-    ref nref;
+    const name_string_t *pnstr;
 
-    nref.value.pname = name_index_ptr(nidx);
-    return alloc_name_is_since_save(&nref, save);
+    if (!save->restore_names)
+	return false;
+    pnstr = names_index_string_inline(mem->gs_lib_ctx->gs_name_table, nidx);
+    if (pnstr->foreign_string)
+	return false;
+    return alloc_is_since_save(pnstr->string_bytes, save);
 }
 
 /* Check whether any names have been created since a given save */
@@ -836,7 +840,7 @@ restore_resources(alloc_save_t * sprev, gs_ref_memory_t * mem)
 
     /* Adjust the name table. */
     if (sprev->restore_names)
-	names_restore(the_gs_name_table, sprev);
+	names_restore(mem->gs_lib_ctx->gs_name_table, sprev);
 }
 
 /* Release memory for a restore. */
@@ -943,7 +947,7 @@ combine_space(gs_ref_memory_t * mem)
 	    outer->rtop = cp->rtop;
 	    outer->ctop = cp->ctop;
 	    outer->has_refs |= cp->has_refs;
-	    gs_free_object(mem->parent, cp,
+	    gs_free_object(mem->non_gc_memory, cp,
 			   "combine_space(inner)");
 	}
     }
