@@ -14,13 +14,13 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: istack.c,v 1.5 2002/06/16 03:48:22 lpd Exp $ */
+/* $Id: istack.c 4180 2003-09-03 03:22:59Z giles $ */
 /* Manager for expandable stacks of refs */
 #include "memory_.h"
 #include "ghost.h"
 #include "gsstruct.h"
 #include "gsutil.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "ialloc.h"
 #include "istack.h"
 #include "istkparm.h"
