@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: macsystypes.h,v 1.6 2002/11/14 17:36:21 giles Exp $ */
+/* $Id: macsystypes.h 3565 2003-01-25 22:50:31Z giles $ */
 
 #ifndef __sys_types_h__
 #define __sys_types_h__
@@ -22,7 +22,6 @@
 #include <MacTypes.h>
 #include <unix.h>
 #define CHECK_INTERRUPTS
-#define HAVE_SYS_TIME_H
 
 /* use a 64 bit type for color vectors. (from MacTypes.h)
    this is important for devicen support, but can be safely
@@ -39,8 +38,9 @@ int myfprintf(FILE *file, const char *fmt, ...);
 int myfputs(const char *string, FILE *file);
 #endif
 
-#ifndef __MACINTOSH__
-#define __MACINTOSH__
+/* Metrowerks CodeWarrior should define this */
+#ifndef __MACOS__
+#define __MACOS__
 #endif
 
 #endif /* __sys_types_h__ */
