@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: sdctc.c,v 1.4 2002/02/21 22:24:53 giles Exp $ */
+/* $Id: sdctc.c 5250 2004-08-19 19:33:09Z stefan $ */
 /* Code common to DCT encoding and decoding streams */
 #include "stdio_.h"
 #include "jpeglib_.h"
@@ -31,7 +31,7 @@ s_DCT_set_defaults(stream_state * st)
 {
     stream_DCT_state *const ss = (stream_DCT_state *) st;
 
-    ss->jpeg_memory = &gs_memory_default;
+    ss->jpeg_memory = (gs_memory_t *)gs_lib_ctx_get_non_gc_memory_t(); /* cast away const */
     ss->data.common = 0;
 	/****************
 	  ss->data.common->Picky = 0;
