@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: stream.c,v 1.20 2002/09/05 18:07:59 igor Exp $ */
+/* $Id: stream.c 5408 2004-10-07 05:18:34Z ray $ */
 /* Stream package for Ghostscript interpreter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -325,16 +325,29 @@ s_filter_write_flush(register stream * s)
 }
 
 /* Close a filter.  If this is an encoding filter, flush it first. */
+/* If CloseTarget was specified (close_strm), then propagate the sclose */
 int
 s_filter_close(register stream * s)
 {
+    int status;
+    bool close = s->close_strm;
+    stream *stemp = s->strm;
+
     if (s_is_writing(s)) {
 	int status = s_process_write_buf(s, true);
 
+	if (status != 0 && status != EOFC)
+	    return status;
+        status = sflush(stemp);
 	if (status != 0 && status != EOFC)
 	    return status;
     }
-    return s_std_close(s);
+    status = s_std_close(s);
+    if (status != 0 && status != EOFC)
+	return status;
+    if (close && stemp != 0)
+	return sclose(stemp);
+    return status;
 }
 
 /* Disregard a stream error message. */
@@ -515,6 +528,8 @@ sgets(stream * s, byte * buf, uint nmax, uint * pn)
 		cw.limit -= min_left;
 		status = sreadbuf(s, &cw);
 		cw.limit += min_left;
+		/* Compact the stream so stell will return the right result. */
+		stream_compact(s, true);
 		/*
 		 * We know the stream buffer is empty, so it's safe to
 		 * update position.  However, we need to reset the read
@@ -792,9 +807,9 @@ sreadbuf(stream * s, stream_cursor_write * pbuf)
 	    oldpos = pw->ptr;
 	    status = (*curr->procs.process) (curr->state, pr, pw, eof);
 	    pr->limit += left;
-	    if_debug4('s', "[s]after read 0x%lx, nr=%u, nw=%u, status=%d\n",
+	    if_debug5('s', "[s]after read 0x%lx, nr=%u, nw=%u, status=%d, position=%d\n",
 		      (ulong) curr, (uint) (pr->limit - pr->ptr),
-		      (uint) (pw->limit - pw->ptr), status);
+		      (uint) (pw->limit - pw->ptr), status, s->position);
 	    if (strm == 0 || status != 0)
 		break;
 	    if (strm->end_status < 0) {
@@ -805,8 +820,9 @@ sreadbuf(stream * s, stream_cursor_write * pbuf)
 	    MOVE_AHEAD(curr, prev);
 	    stream_compact(curr, false);
 	}
-	/* If curr reached EOD and is a filter stream, close it. */
-	if (strm != 0 && status == EOFC &&
+	/* If curr reached EOD and is a filter or file stream, close it. */
+	/* (see PLRM 3rd, sec 3.8.2, p80) */
+	if ((strm != 0 || curr->file) && status == EOFC &&
 	    curr->cursor.r.ptr >= curr->cursor.r.limit &&
 	    curr->close_at_eod
 	    ) {
@@ -1036,6 +1052,15 @@ s_string_read_seek(register stream * s, long pos)
     s->srptr = s->cbuf + pos - 1;
     /* We might be seeking after a reusable string reached EOF. */
     s->srlimit = s->cbuf + s->bsize - 1;
+    /* 
+     * When the file reaches EOF,
+     * stream_compact sets s->position to its end. 
+     * Reset it now to allow stell to work properly
+     * after calls to this function.
+     * Note that if the riched EOF and this fuction
+     * was not called, stell still returns a wrong value.
+     */
+    s->position = 0;
     return 0;
 }
 
