@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: ttobjs.c,v 1.6.4.2 2004/06/24 10:28:26 igor Exp $ */
+/* $Id: ttobjs.c 5405 2004-10-06 11:32:17Z igor $ */
 
 /* Changes after FreeType: cut out the TrueType instruction interpreter. */
 
@@ -373,6 +373,8 @@ static int free_aux(ttfMemory *mem, void *ptr)
    exec->numIDefs = ins->numIDefs;
    exec->FDefs    = ins->FDefs;
    exec->IDefs    = ins->IDefs;
+   exec->countIDefs = ins->countIDefs;
+   memcpy(exec->IDefPtr, ins->IDefPtr, sizeof(exec->IDefPtr));
 
    exec->metrics  = ins->metrics;
 
@@ -415,6 +417,9 @@ static int free_aux(ttfMemory *mem, void *ptr)
    }
    exec->numFDefs = 0;
    exec->numIDefs = 0;
+   memcpy(ins->IDefPtr, exec->IDefPtr, sizeof(ins->IDefPtr));
+   ins->countIDefs = exec->countIDefs;
+   exec->countIDefs = 0;
    exec->FDefs    = 0;
    exec->IDefs    = 0;
    exec->cvtSize = 0;
@@ -569,6 +574,10 @@ static int free_aux(ttfMemory *mem, void *ptr)
 
     ins->numFDefs = maxp->maxFunctionDefs;
     ins->numIDefs = maxp->maxInstructionDefs;
+    ins->countIDefs = 0;
+    if (maxp->maxInstructionDefs > 255)
+	goto Fail_Memory;
+    memset(ins->IDefPtr, (Byte)ins->numIDefs, sizeof(ins->IDefPtr));
     ins->cvtSize  = face->cvtSize;
 
     ins->metrics.pointSize    = 10 * 64;     /* default pointsize  = 10pts */
