@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: unistd_.h,v 1.7 2002/06/24 06:20:25 jeong Exp $ */
+/* $Id: unistd_.h 5335 2004-09-15 19:41:01Z ray $ */
 /* Generic substitute for Unix unistd.h */
 
 #ifndef unistd__INCLUDED
@@ -36,11 +36,23 @@
 #  include <io.h>
 #endif
 
-#if defined(_MSC_VER) || defined(__BORLANDC__) && defined(__WIN32__) 
+#if defined(_MSC_VER) 
 #  define fsync(handle) _commit(handle)
 #  define read(fd, buf, len) _read(fd, buf, len)
+#  define isatty(fd) _isatty(fd)
+#  define setmode(fd, mode) _setmode(fd, mode)
+#  define fstat(fd, buf) _fstat(fd, buf)
+#  define dup(fd) _dup(fd)
+#  define open(fname, flags, mode) _open(fname, flags, mode)
+#  define close(fd) _close(fd)
+#elif defined(__BORLANDC__) && defined(__WIN32__) 
+#  define fsync(handle) _commit(handle)
+#  define read(fd, buf, len) _read(fd, buf, len)
+#  define isatty(fd) _isatty(fd)
+#  define setmode(fd, mode) _setmode(fd, mode)
 #else
 #  include <unistd.h>
 #endif
 
 #endif   /* unistd__INCLUDED */
+
