@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: zbfont.c,v 1.14 2002/06/16 03:43:50 lpd Exp $ */
+/* $Id: zbfont.c 5502 2004-11-15 01:12:06Z ray $ */
 /* Font creation utilities */
 #include "memory_.h"
 #include "string_.h"
@@ -63,16 +63,46 @@ zbuildfont3(i_ctx_t *i_ctx_p)
 
 /* Encode a character. */
 gs_glyph
-zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t ignored)
+zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t gspace)
 {
-    const ref *pencoding = &pfont_data(pfont)->Encoding;
+    font_data *pdata = pfont_data(pfont);
+    const ref *pencoding = &pdata->Encoding;
     ulong index = chr;	/* work around VAX widening bug */
     ref cname;
-    int code = array_get(pencoding, (long)index, &cname);
+    int code = array_get(pfont->memory, pencoding, (long)index, &cname);
 
     if (code < 0 || !r_has_type(&cname, t_name))
 	return gs_no_glyph;
-    return (gs_glyph)name_index(&cname);
+    if (pfont->FontType == ft_user_defined && r_type(&pdata->BuildGlyph) == t_null) {
+	ref nsref, tname;
+
+	name_string_ref(pfont->memory, &cname, &nsref);
+	if (r_size(&nsref) == 7 && 
+	    !memcmp(nsref.value.const_bytes, ".notdef", r_size(&nsref))) {
+	    /* A special support for high level devices.
+	       They need a glyph name but the font doesn't provide one
+	       due to an instandard BuildChar.
+	       Such fonts don't conform to PLRM section 5.3.7,
+	       but we've got real examples that we want to handle (Bug 686982).
+	       Construct a name here.
+	       Low level devices don't pass here, because regular PS interpretation 
+	       doesn't need such names.
+	    */
+	    char buf[20];
+	    int code;
+
+	    if (gspace == GLYPH_SPACE_NOGEN)
+		return gs_no_glyph;    
+	    sprintf(buf, "j%ld", chr); /* 'j' is arbutrary. */
+	    code = name_ref(pfont->memory, (const byte *)buf, strlen(buf), &tname, 1);
+	    if (code < 0) {
+		/* Can't propagate the error due to interface limitation,
+		   return with .notdef */
+	    } else
+		cname = tname;
+	}
+    }
+    return (gs_glyph)name_index(pfont->memory, &cname);
 }
 
 /* Get the name of a glyph. */
@@ -86,18 +116,80 @@ zfont_glyph_name(gs_font *font, gs_glyph index, gs_const_string *pstr)
 	int code;
 
 	sprintf(cid_name, "%lu", (ulong) index);
-	code = name_ref((const byte *)cid_name, strlen(cid_name),
+	code = name_ref(font->memory, (const byte *)cid_name, strlen(cid_name),
 			&nref, 1);
 	if (code < 0)
 	    return code;
     } else
-	name_index_ref(index, &nref);
-    name_string_ref(&nref, &sref);
+	name_index_ref(font->memory, index, &nref);
+    name_string_ref(font->memory, &nref, &sref);
     pstr->data = sref.value.const_bytes;
     pstr->size = r_size(&sref);
     return 0;
 }
 
+private gs_char 
+gs_font_map_glyph_by_dict(const gs_memory_t *mem, const ref *map, gs_glyph glyph)
+{
+    ref *v, n;
+    if (glyph >= gs_min_cid_glyph) {
+	uint cid = glyph - gs_min_cid_glyph;
+
+	if (dict_find_string(map, "CIDCount", &v) > 0) {
+	    /* This is a CIDDEcoding resource. */
+	    make_int(&n, cid / 256);
+	    if (dict_find(map, &n, &v) > 0) {
+		ref vv;
+
+		if (array_get(mem, v, cid % 256, &vv) == 0 && r_type(&vv) == t_integer)
+		    return vv.value.intval;
+	    }
+	    return GS_NO_CHAR; /* Absent in the map. */
+	}
+	/* This is GlyphNames2Unicode dictionary. */
+	make_int(&n, cid);
+    } else
+	name_index_ref(mem, glyph, &n);
+    if (dict_find(map, &n, &v) > 0) {
+	if (r_has_type(v, t_string)) {
+	    int i, l = r_size(v);
+	    gs_char c = 0;
+
+	    for (i = 0; i < l; i++)
+		c = (c << 8) | v->value.const_bytes[i];
+	    return c;
+	}
+	if (r_type(v) == t_integer)
+	    return v->value.intval;
+    }
+    return GS_NO_CHAR; /* Absent in the map. */
+}
+
+/* Get Unicode UTF-16 code for a glyph. */
+gs_char 
+gs_font_map_glyph_to_unicode(gs_font *font, gs_glyph glyph)
+{
+    font_data *pdata = pfont_data(font);
+    const ref *UnicodeDecoding;
+
+    if (r_type(&pdata->GlyphNames2Unicode) == t_dictionary) {
+	gs_char c = gs_font_map_glyph_by_dict(font->memory, 
+					      &pdata->GlyphNames2Unicode, glyph);
+
+	if (c != GS_NO_CHAR)
+	    return c;
+	/* 
+	 * Fall through, because test.ps for SF bug #529103 requres 
+	 * to examine both tables. Due to that the Unicode Decoding resource 
+	 * can't be a default value for FontInfo.GlyphNames2Unicode .
+	 */
+    }
+    UnicodeDecoding = zfont_get_to_unicode_map(font->dir);
+    if (UnicodeDecoding != NULL && r_type(UnicodeDecoding) == t_dictionary)
+	return gs_font_map_glyph_by_dict(font->memory, UnicodeDecoding, glyph);
+    return GS_NO_CHAR; /* No map. */
+}
+
 /* ------ Initialization procedure ------ */
 
 const op_def zbfont_op_defs[] =
@@ -110,7 +202,7 @@ const op_def zbfont_op_defs[] =
 
 /* Convert strings to executable names for build_proc_refs. */
 int
-build_proc_name_refs(build_proc_refs * pbuild,
+build_proc_name_refs(const gs_memory_t *mem, build_proc_refs * pbuild,
 		     const char *bcstr, const char *bgstr)
 {
     int code;
@@ -118,14 +210,16 @@ build_proc_name_refs(build_proc_refs * pbuild,
     if (!bcstr)
 	make_null(&pbuild->BuildChar);
     else {
-	if ((code = name_ref((const byte *)bcstr, strlen(bcstr), &pbuild->BuildChar, 0)) < 0)
+	if ((code = name_ref(mem, (const byte *)bcstr, 
+			     strlen(bcstr), &pbuild->BuildChar, 0)) < 0)
 	    return code;
 	r_set_attrs(&pbuild->BuildChar, a_executable);
     }
     if (!bgstr)
 	make_null(&pbuild->BuildGlyph);
     else {
-	if ((code = name_ref((const byte *)bgstr, strlen(bgstr), &pbuild->BuildGlyph, 0)) < 0)
+	if ((code = name_ref(mem, (const byte *)bgstr, 
+			     strlen(bgstr), &pbuild->BuildGlyph, 0)) < 0)
 	    return code;
 	r_set_attrs(&pbuild->BuildGlyph, a_executable);
     }
@@ -302,8 +396,20 @@ build_gs_simple_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base ** ppfont,
     gs_uid uid;
     int code;
     gs_font_base *pfont;
+    ref *pfontinfo, *g2u = NULL, Glyph2Unicode;
 
-    code = font_bbox_param(op, bbox);
+    if (dict_find_string(op, "FontInfo", &pfontinfo) <= 0 ||
+	    !r_has_type(pfontinfo, t_dictionary) ||
+	    dict_find_string(pfontinfo, "GlyphNames2Unicode", &g2u) <= 0 ||
+	    !r_has_type(pfontinfo, t_dictionary))
+	g2u = NULL;
+    else
+	/*
+	 * Since build_gs_font may resize the dictionary and cause
+	 * pointers to become invalid, save Glyph2Unicode
+	 */
+	Glyph2Unicode = *g2u;
+    code = font_bbox_param(imemory, op, bbox);
     if (code < 0)
 	return code;
     code = dict_uid_param(op, &uid, 0, imemory, i_ctx_p);
@@ -318,12 +424,18 @@ build_gs_simple_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base ** ppfont,
     pfont = *ppfont;
     pfont->procs.init_fstack = gs_default_init_fstack;
     pfont->procs.define_font = gs_no_define_font;
+    pfont->procs.decode_glyph = gs_font_map_glyph_to_unicode;
     pfont->procs.make_font = zbase_make_font;
     pfont->procs.next_char_glyph = gs_default_next_char_glyph;
     pfont->FAPI = 0;
     pfont->FAPI_font_data = 0;
     init_gs_simple_font(pfont, bbox, &uid);
     lookup_gs_simple_font_encoding(pfont);
+    if (g2u != NULL) {
+	font_data *pdata = pfont_data(pfont);
+
+	ref_assign_new(&pdata->GlyphNames2Unicode, &Glyph2Unicode);
+    }
     return 0;
 }
 
@@ -344,12 +456,12 @@ void
 lookup_gs_simple_font_encoding(gs_font_base * pfont)
 {
     const ref *pfe = &pfont_data(pfont)->Encoding;
-    uint esize = r_size(pfe);
     int index = -1;
 
     pfont->encoding_index = index;
-    if (esize <= 256) {
+    if (r_type(pfe) == t_array && r_size(pfe) <= 256) {
 	/* Look for an encoding that's "close". */
+	uint esize = r_size(pfe);
 	int near_index = -1;
 	uint best = esize / 3;	/* must match at least this many */
 	gs_const_string fstrs[256];
@@ -359,14 +471,14 @@ lookup_gs_simple_font_encoding(gs_font_base * pfont)
 	for (i = 0; i < esize; ++i) {
 	    ref fchar;
 
-	    if (array_get(pfe, (long)i, &fchar) < 0 ||
+	    if (array_get(pfont->memory, pfe, (long)i, &fchar) < 0 ||
 		!r_has_type(&fchar, t_name)
 		)
 		fstrs[i].data = 0, fstrs[i].size = 0;
 	    else {
 		ref nsref;
 
-		name_string_ref(&fchar, &nsref);
+		name_string_ref(pfont->memory, &fchar, &nsref);
 		fstrs[i].data = nsref.value.const_bytes;
 		fstrs[i].size = r_size(&nsref);
 	    }
@@ -402,21 +514,32 @@ lookup_gs_simple_font_encoding(gs_font_base * pfont)
 }
 
 /* Get FontMatrix and FontName parameters. */
-private void get_font_name(ref *, const ref *);
-private void copy_font_name(gs_font_name *, const ref *);
 private int
-sub_font_params(const ref *op, gs_matrix *pmat, ref *pfname)
+sub_font_params(const gs_memory_t *mem, const ref *op, gs_matrix *pmat, gs_matrix *pomat, ref *pfname)
 {
     ref *pmatrix;
     ref *pfontname;
+    ref *porigfont;
 
     if (dict_find_string(op, "FontMatrix", &pmatrix) <= 0 ||
-	read_matrix(pmatrix, pmat) < 0
+	read_matrix(mem, pmatrix, pmat) < 0
 	)
 	return_error(e_invalidfont);
-    if (dict_find_string(op, "FontName", &pfontname) > 0)
-	get_font_name(pfname, pfontname);
-    else
+    if (dict_find_string(op, ".OrigFont", &porigfont) <= 0)
+	porigfont = NULL;
+    if (pomat!= NULL) {
+	if (porigfont == NULL ||
+	    dict_find_string(porigfont, "FontMatrix", &pmatrix) <= 0 ||
+	    read_matrix(mem, pmatrix, pomat) < 0
+	    )
+	    memset(pomat, 0, sizeof(*pomat));
+    }
+    if (dict_find_string((porigfont != NULL ? porigfont : op), ".Alias", &pfontname) > 0) {
+        /* If we emulate the font, we want the requested name rather than a substitute. */
+	get_font_name(mem, pfname, pfontname);
+    } else if (dict_find_string((porigfont != NULL ? porigfont : op), "FontName", &pfontname) > 0) {
+	get_font_name(mem, pfname, pfontname);
+    } else
 	make_empty_string(pfname, a_readonly);
     return 0;
 }
@@ -443,7 +566,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
     ref *pfid;
     ref *aop = dict_access_ref(op);
 
-    get_font_name(&kname, op - 1);
+    get_font_name(imemory, &kname, op - 1);
     if (dict_find_string(op, "FontType", &pftype) <= 0 ||
 	!r_has_type(pftype, t_integer) ||
 	pftype->value.intval != (int)ftype
@@ -452,6 +575,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
     if (dict_find_string(op, "Encoding", &pencoding) <= 0) {
 	if (!(options & bf_Encoding_optional))
 	    return_error(e_invalidfont);
+	pencoding = 0;
     } else {
 	if (!r_is_array(pencoding))
 	    return_error(e_invalidfont);
@@ -478,7 +602,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
 	if (pfont->base == pfont) {	/* original font */
 	    if (!level2_enabled)
 		return_error(e_invalidfont);
-	    if (obj_eq(pfont_dict(pfont), op)) {
+	    if (obj_eq(pfont->memory, pfont_dict(pfont), op)) {
 		*ppfont = pfont;
 		return 1;
 	    }
@@ -492,7 +616,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
 	    gs_matrix mat;
 	    ref fname;			/* t_string */
 
-	    code = sub_font_params(op, &mat, &fname);
+	    code = sub_font_params(imemory, op, &mat, NULL, &fname);
 	    if (code < 0)
 		return code;
 	    code = 1;
@@ -542,7 +666,7 @@ build_gs_sub_font(i_ctx_t *i_ctx_p, const ref *op, gs_font **ppfont,
 		  const build_proc_refs * pbuild, const ref *pencoding,
 		  ref *fid_op)
 {
-    gs_matrix mat;
+    gs_matrix mat, omat;
     ref fname;			/* t_string */
     gs_font *pfont;
     font_data *pdata;
@@ -551,7 +675,7 @@ build_gs_sub_font(i_ctx_t *i_ctx_p, const ref *op, gs_font **ppfont,
      * in the same VM as the font dictionary.
      */
     uint space = ialloc_space(idmemory);
-    int code = sub_font_params(op, &mat, &fname);
+    int code = sub_font_params(imemory, op, &mat, &omat, &fname);
 
     if (code < 0)
 	return code;
@@ -579,6 +703,7 @@ build_gs_sub_font(i_ctx_t *i_ctx_p, const ref *op, gs_font **ppfont,
     pfont->client_data = pdata;
     pfont->FontType = ftype;
     pfont->FontMatrix = mat;
+    pfont->orig_FontMatrix = omat;
     pfont->BitmapWidths = false;
     pfont->ExactSize = fbit_use_bitmaps;
     pfont->InBetweenSize = fbit_use_outlines;
@@ -594,15 +719,15 @@ build_gs_sub_font(i_ctx_t *i_ctx_p, const ref *op, gs_font **ppfont,
 
 /* Get the string corresponding to a font name. */
 /* If the font name isn't a name or a string, return an empty string. */
-private void
-get_font_name(ref * pfname, const ref * op)
+void
+get_font_name(const gs_memory_t *mem, ref * pfname, const ref * op)
 {
     switch (r_type(op)) {
 	case t_string:
 	    *pfname = *op;
 	    break;
 	case t_name:
-	    name_string_ref(op, pfname);
+	    name_string_ref(mem, op, pfname);
 	    break;
 	default:
 	    /* This is weird, but legal.... */
@@ -611,7 +736,7 @@ get_font_name(ref * pfname, const ref * op)
 }
 
 /* Copy a font name into the gs_font structure. */
-private void
+void
 copy_font_name(gs_font_name * pfstr, const ref * pfname)
 {
     uint size = r_size(pfname);
