@@ -15,7 +15,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: zusparam.c,v 1.8 2002/11/02 07:33:48 raph Exp $ */
+/* $Id: zusparam.c 5215 2004-08-04 19:36:13Z stefan $ */
 /* User and system parameter operators */
 #include "memory_.h"
 #include "string_.h"
@@ -102,7 +102,7 @@ zcheckpassword(i_ctx_t *i_ctx_p)
     array_param_list list;
     gs_param_list *const plist = (gs_param_list *)&list;
     int result = 0;
-    int code = name_ref((const byte *)"Password", 8, &params[0], 0);
+    int code = name_ref(imemory, (const byte *)"Password", 8, &params[0], 0);
     password pass;
 
     if (code < 0)
@@ -412,6 +412,28 @@ set_MinScreenLevels(i_ctx_t *i_ctx_p, long val)
     gs_setminscreenlevels((uint) val);
     return 0;
 }
+private long
+current_AlignToPixels(i_ctx_t *i_ctx_p)
+{
+    return gs_currentaligntopixels(ifont_dir);
+}
+private int
+set_AlignToPixels(i_ctx_t *i_ctx_p, long val)
+{
+    gs_setaligntopixels(ifont_dir, (uint)val);
+    return 0;
+}
+private long
+current_GridFitTT(i_ctx_t *i_ctx_p)
+{
+    return gs_currentgridfittt(ifont_dir);
+}
+private int
+set_GridFitTT(i_ctx_t *i_ctx_p, long val)
+{
+    gs_setgridfittt(ifont_dir, (uint)val);
+    return 0;
+}
 private const long_param_def_t user_long_params[] =
 {
     {"JobTimeout", 0, MAX_UINT_PARAM,
@@ -436,7 +458,11 @@ private const long_param_def_t user_long_params[] =
      current_WaitTimeout, set_WaitTimeout},
     /* Extensions */
     {"MinScreenLevels", 0, MAX_UINT_PARAM,
-     current_MinScreenLevels, set_MinScreenLevels}
+     current_MinScreenLevels, set_MinScreenLevels},
+    {"AlignToPixels", 0, 1,
+     current_AlignToPixels, set_AlignToPixels},
+    {"GridFitTT", 0, 3, 
+     current_GridFitTT, set_GridFitTT}
 };
 
 /* Boolean values */
@@ -673,7 +699,7 @@ currentparam1(i_ctx_t *i_ctx_p, const param_set * pset)
 
     check_type(*op, t_name);
     check_ostack(2);
-    name_string_ref((const ref *)op, &sref);
+    name_string_ref(imemory, (const ref *)op, &sref);
     code = current_param_list(i_ctx_p, pset, &sref);
     if (code < 0)
 	return code;
